| package |
package := Package name: 'SmaCC CSV Parser'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains a simple CSV parser.

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #CSVFileNode;
	add: #CSVFileNodeVisitor;
	add: #CSVLineNode;
	add: #CSVParseNode;
	add: #CSVParseNodeVisitor;
	add: #CSVParser;
	add: #CSVScanner;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\SmaCC Runtime\SmaCC Runtime';
	yourself).

package!

"Class Definitions"!

SmaCCParseNode subclass: #CSVParseNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
CSVParseNode subclass: #CSVFileNode
	instanceVariableNames: 'lines eols'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
CSVParseNode subclass: #CSVLineNode
	instanceVariableNames: 'values commas'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCParseNodeVisitor subclass: #CSVFileNodeVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCParseNodeVisitor subclass: #CSVParseNodeVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCParser subclass: #CSVParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCScanner subclass: #CSVScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

CSVParseNode guid: (GUID fromString: '{AF570AA9-00D8-47DD-A51B-DF5BBDD1042E}')!
CSVParseNode comment: ''!
!CSVParseNode categoriesForClass!Unclassified! !
!CSVParseNode methodsFor!

acceptVisitor: aParseVisitor
	^aParseVisitor visitParse: self! !
!CSVParseNode categoriesFor: #acceptVisitor:!generated!public! !

CSVFileNode guid: (GUID fromString: '{28ACF34C-DC38-408D-9AD2-9434C9ECA0CE}')!
CSVFileNode comment: ''!
!CSVFileNode categoriesForClass!Unclassified! !
!CSVFileNode methodsFor!

acceptVisitor: aParseVisitor
	^aParseVisitor visitFile: self!

compositeNodeVariables
	^#(#lines)!

compositeTokenVariables
	^#(#eols)!

eols
	^eols!

eols: anOrderedCollection
	eols := anOrderedCollection!

initialize
	super initialize.
	lines := OrderedCollection new: 2.
	eols := OrderedCollection new: 2.!

lines
	^lines!

lines: anOrderedCollection
	self setParents: self lines to: nil.
	lines := anOrderedCollection.
	self setParents: self lines to: self! !
!CSVFileNode categoriesFor: #acceptVisitor:!generated!public! !
!CSVFileNode categoriesFor: #compositeNodeVariables!generated!public! !
!CSVFileNode categoriesFor: #compositeTokenVariables!generated!public! !
!CSVFileNode categoriesFor: #eols!generated!public! !
!CSVFileNode categoriesFor: #eols:!generated!public! !
!CSVFileNode categoriesFor: #initialize!generated-initialize-release!public! !
!CSVFileNode categoriesFor: #lines!generated!public! !
!CSVFileNode categoriesFor: #lines:!generated!public! !

CSVLineNode guid: (GUID fromString: '{C05E4CA4-6AAA-441B-81C0-A3DB0DDC3E2D}')!
CSVLineNode comment: ''!
!CSVLineNode categoriesForClass!Unclassified! !
!CSVLineNode methodsFor!

acceptVisitor: aParseVisitor
	^aParseVisitor visitLine: self!

commas
	^commas!

commas: anOrderedCollection
	commas := anOrderedCollection!

compositeTokenVariables
	^#(#values #commas)!

initialize
	super initialize.
	values := OrderedCollection new: 2.
	commas := OrderedCollection new: 2.!

values
	^values!

values: anOrderedCollection
	values := anOrderedCollection! !
!CSVLineNode categoriesFor: #acceptVisitor:!generated!public! !
!CSVLineNode categoriesFor: #commas!generated!public! !
!CSVLineNode categoriesFor: #commas:!generated!public! !
!CSVLineNode categoriesFor: #compositeTokenVariables!generated!public! !
!CSVLineNode categoriesFor: #initialize!generated-initialize-release!public! !
!CSVLineNode categoriesFor: #values!generated!public! !
!CSVLineNode categoriesFor: #values:!generated!public! !

CSVFileNodeVisitor guid: (GUID fromString: '{02F2C7DA-F1F0-4843-A791-A4281C284544}')!
CSVFileNodeVisitor comment: ''!
!CSVFileNodeVisitor categoriesForClass!Unclassified! !
!CSVFileNodeVisitor methodsFor!

visitFile: aFile
	^self visitSmaCCParseNode: aFile!

visitLine: aLine
	^self visitFile: aLine! !
!CSVFileNodeVisitor categoriesFor: #visitFile:!generated!public! !
!CSVFileNodeVisitor categoriesFor: #visitLine:!generated!public! !

CSVParseNodeVisitor guid: (GUID fromString: '{927604E7-4F4B-4015-9E99-379B0453EBE9}')!
CSVParseNodeVisitor comment: ''!
!CSVParseNodeVisitor categoriesForClass!Unclassified! !
!CSVParseNodeVisitor methodsFor!

visitFile: aFile
	^self visitParse: aFile!

visitLine: aLine
	^self visitParse: aLine!

visitParse: aParse
	^self visitSmaCCParseNode: aParse! !
!CSVParseNodeVisitor categoriesFor: #visitFile:!generated!public! !
!CSVParseNodeVisitor categoriesFor: #visitLine:!generated!public! !
!CSVParseNodeVisitor categoriesFor: #visitParse:!generated!public! !

CSVParser guid: (GUID fromString: '{B8C4AFBC-5417-4B6E-BB40-BB779BD4B971}')!
CSVParser comment: ''!
!CSVParser categoriesForClass!Unclassified! !
!CSVParser methodsFor!

reduceActionForFile1: nodes
	| result |
	result := CSVFileNode new.
	result addNodes: ((nodes at: 1) at: 1) to: result lines.
	result addTokens: ((nodes at: 1) at: 2) to: result eols.
	^result!

reduceActionForLine1: nodes
	| result |
	result := CSVLineNode new.
	result addToken: (nodes at: 1) to: result values.
	^result!

reduceActionForLine2: nodes
	| result |
	result := nodes at: 1.
	result addToken: (nodes at: 2) to: result commas.
	result addToken: (nodes at: 3) to: result values.
	^result!

reduceActionForLines1: nodes
	| result |
	result := Array new: 2.
	result at: 1 put: (OrderedCollection new: 2).
	result at: 2 put: (OrderedCollection new: 2).
	self add: (nodes at: 1) to: (result at: 1).
	^result!

reduceActionForLines2: nodes
	| result |
	result := Array new: 2.
	result at: 1 put: (OrderedCollection new: 2).
	result at: 2 put: (OrderedCollection new: 2).
	self addAll: ((nodes at: 1) at: 1) to: (result at: 1).
	self addAll: ((nodes at: 1) at: 2) to: (result at: 2).
	self add: (nodes at: 2) to: (result at: 2).
	self add: (nodes at: 3) to: (result at: 1).
	^result!

reduceActionForMissing1: nodes
	^SmaCCToken
		value: ''
		start: self position
		ids: #()!

reduceActionForValue1: nodes
	^nodes at: 1!

reduceTable
^#(
	#(9 0 #reduceActionForMissing1:) 
	#(7 1 #reduceActionForValue1:) 
	#(6 1 #reduceActionForLines1:) 
	#(8 1 #reduceActionForFile1:) 
	#(5 1 #reduceActionForLine1:) 
	#(5 3 #reduceActionForLine2:) 
	#(6 3 #reduceActionForLines2:)
	).!

symbolTypes
	^#(#SmaCCToken #SmaCCToken #SmaCCToken #CSVFileNode #CSVLineNode #OrderedCollection #SmaCCToken #CSVFileNode #SmaCCToken #SmaCCToken #SmaCCToken)!

transitionTable
^#(
#[1 0 6 0 1 0 9 0 2 0 6 0 3 0 13 0 5 0 17 0 6 0 21 0 7 0 25 0 8 0 29 0 9 0 6 0 10] 
#[0 0 10 0 1 0 3 0 10] 
#[1 0 33 0 1 0 14 0 3 0 14 0 10] 
#[1 0 37 0 3 0 18 0 10] 
#[0 0 22 0 1 0 3 0 10] 
#[0 0 0 0 10] 
#[0 0 10 0 1 0 3 0 10] 
#[1 0 6 0 1 0 9 0 2 0 6 0 3 0 41 0 7 0 29 0 9 0 6 0 10] 
#[1 0 6 0 1 0 9 0 2 0 6 0 3 0 45 0 5 0 21 0 7 0 29 0 9 0 6 0 10] 
#[0 0 26 0 1 0 3 0 10] 
#[1 0 33 0 1 0 30 0 3 0 30 0 10]
	).! !
!CSVParser categoriesFor: #reduceActionForFile1:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForLine1:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForLine2:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForLines1:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForLines2:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForMissing1:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceActionForValue1:!generated-reduction actions!public! !
!CSVParser categoriesFor: #reduceTable!generated!public! !
!CSVParser categoriesFor: #symbolTypes!generated!public! !
!CSVParser categoriesFor: #transitionTable!generated!public! !

!CSVParser class methodsFor!

dataValueFrom: aSmaCCToken
	| string |
	string := aSmaCCToken value.
	string isEmpty ifTrue: [^nil].
	string first = $' ifTrue: [^(string copyFrom: 2 to: string size - 1) copyReplaceAll: '''''' with: ''''].
	string first = $" ifTrue: [^(string copyFrom: 2 to: string size - 1) copyReplaceAll: '""' with: '"'].
	^string!

definitionComment
"%prefix CSV;
%suffix Node;

%root Parse;


<entry> 
	: [^\""\'\,\r\n] [^\,\r\n]*
	| (\"" [^\""]* \"") +
	| (\' [^\']* \') +
	;
<newline>
	: \r \n? | \n
	;

File
	: Lines {{}}
	;
Lines
	: Line 'line'
	| Lines <newline> 'eol' Line 'line'
	;
Line
	: Value 'value' {{}}
	| Line "","" 'comma' Value 'value' {{}}
	;
Value
	: <entry>
	| Missing
	;
Missing
	: {SmaCCToken value: '' start: self position ids: #()}
	;"!

fileExtension
	^'.csv'!

fileExtensionType
	^'CSV files'!

parseFileWithHeader: aFilename
	| lines headerLine headerNames |
	lines := self parseFileWithoutHeader: aFilename.
	lines isEmpty ifTrue: [^lines].
	headerLine := lines removeFirst.
	headerNames := (1 to: headerLine size) collect: [:i | (headerLine at: i) ifNil: ['Column ' , i printString]].
	1
		to: lines size
		do:
			[:i | 
			| line dict |
			dict := Dictionary new: headerNames size.
			headerNames with: (lines at: i) do: [:header :value | dict at: header put: value].
			lines at: i put: dict].
	^lines!

parseFileWithoutHeader: aFilename
	| file maxSize lines |
	file := self parseFile: aFilename.
	maxSize := file lines inject: 0 into: [:sum :each | sum max: each values size].
	lines := file lines
		collect:
			[:each | 
			| line |
			line := Array new: maxSize.
			1 to: each values size do: [:i | line at: i put: (self dataValueFrom: (each values at: i))].
			line].
	(file lines size > 0
		and: [file lines last commas isEmpty and: [(self dataValueFrom: file lines last values first) isNil]])
		ifTrue: [lines removeLast].
	^lines!

scannerClass
	^CSVScanner!

startingStateForFile
	^1! !
!CSVParser class categoriesFor: #dataValueFrom:!private! !
!CSVParser class categoriesFor: #definitionComment!generated!public! !
!CSVParser class categoriesFor: #fileExtension!file types!public! !
!CSVParser class categoriesFor: #fileExtensionType!file types!public! !
!CSVParser class categoriesFor: #parseFileWithHeader:!parsing!public! !
!CSVParser class categoriesFor: #parseFileWithoutHeader:!parsing!public! !
!CSVParser class categoriesFor: #scannerClass!generated-accessing!public! !
!CSVParser class categoriesFor: #startingStateForFile!generated!public! !

CSVScanner guid: (GUID fromString: '{1191FCC9-2715-4720-BEB1-A4882B62429D}')!
CSVScanner comment: ''!
!CSVScanner categoriesForClass!Unclassified! !
!CSVScanner methodsFor!

emptySymbolTokenId
	^10!

errorTokenId
	^11!

scan1
	[self step.
	currentCharacter == $"
		ifTrue:
			[self recordMatch: #(2).
			self step.
			currentCharacter == $" ifTrue: [^self scan1].
			^self reportLastMatch]] repeat!

scan2
	[self step.
	currentCharacter == $'
		ifTrue:
			[self recordMatch: #(2).
			self step.
			currentCharacter == $' ifTrue: [^self scan2].
			^self reportLastMatch]] repeat!

scanForToken
	self step.
	currentCharacter isLinefeed ifTrue: [^self recordAndReportMatch: #(3)].
	currentCharacter == ##(Character value: 13)
		ifTrue:
			[self recordMatch: #(3).
			self step.
			currentCharacter isLinefeed ifTrue: [^self recordAndReportMatch: #(3)].
			^self reportLastMatch].
	currentCharacter == $" ifTrue: [^self scan1].
	currentCharacter == $' ifTrue: [^self scan2].
	currentCharacter == $, ifTrue: [^self recordAndReportMatch: #(1)].
	true
		ifTrue:
			[[self recordMatch: #(2).
			self step.
			currentCharacter <= ##(Character value: 9)
				or:
					[(currentCharacter between: ##(Character value: 11) and: ##(Character value: 12))
						or: [(currentCharacter between: ##(Character value: 14) and: $+) or: [currentCharacter >= $-]]]] whileTrue].
	^self reportLastMatch! !
!CSVScanner categoriesFor: #emptySymbolTokenId!generated!public! !
!CSVScanner categoriesFor: #errorTokenId!generated!public! !
!CSVScanner categoriesFor: #scan1!generated!public! !
!CSVScanner categoriesFor: #scan2!generated!public! !
!CSVScanner categoriesFor: #scanForToken!generated!public! !

"Binary Globals"!

