| package |
package := Package name: 'SmaCC Development UI'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains a GUI to create new parsers. It was seperated from the SmaCC Development package to make porting SmaCC easier. The GUI can be accessed through the Tools menu in VW, and through the Additional Tools item in Dolphin.

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.'.

package basicScriptAt: #postinstall put: 'Smalltalk developmentSystem addAdditionalToolsFolderIcon: SmaCCParserGenerator toolsFolderIcon!!'.
package basicScriptAt: #preuninstall put: 'Smalltalk developmentSystem removeSystemFolderIcon: SmaCCParserGenerator toolsFolderIcon!!'.

package classNames
	add: #SmaCCGrammarStyler;
	add: #SmaCCGrammarStylerScanner;
	add: #SmaCCParserGenerator;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\IDE\Base\Development System';
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\..\Object Arts\Dolphin\MVP\Views\Cards\Dolphin Card Containers';
	add: '..\..\..\Object Arts\Dolphin\MVP\Presenters\Prompters\Dolphin Choice Prompter';
	add: '..\..\..\Object Arts\Dolphin\MVP\Views\Common Controls\Dolphin Common Controls';
	add: '..\..\..\Object Arts\Dolphin\MVP\Models\List\Dolphin List Models';
	add: '..\..\..\Object Arts\Dolphin\MVP\Presenters\List\Dolphin List Presenter';
	add: '..\..\..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	add: '..\..\..\Object Arts\Dolphin\MVP\Presenters\Prompters\Dolphin Prompter';
	add: '..\..\..\Object Arts\Dolphin\MVP\Views\Scintilla\Dolphin Scintilla View';
	add: '..\..\..\Object Arts\Dolphin\MVP\Views\Sliding Tray\Dolphin Slidey-Inney-Outey Thing';
	add: '..\..\..\Object Arts\Dolphin\MVP\Presenters\Text\Dolphin Text Presenter';
	add: '..\..\..\Object Arts\Dolphin\MVP\Type Converters\Dolphin Type Converters';
	add: '..\..\..\Object Arts\Dolphin\MVP\Models\Value\Dolphin Value Models';
	add: '..\SmaCC Development\SmaCC Development';
	add: '..\SmaCC Runtime\SmaCC Runtime';
	add: '..\..\..\Object Arts\Dolphin\System\Compiler\Smalltalk Parser';
	yourself).

package!

"Class Definitions"!

Shell subclass: #SmaCCParserGenerator
	instanceVariableNames: 'allowUnicode type parserClassName scannerClassName codeGeneratorType'
	classVariableNames: 'ShowPrecedenceConflicts'
	poolDictionaries: 'ScintillaConstants'
	classInstanceVariableNames: ''!
ScintillaStyler subclass: #SmaCCGrammarStyler
	instanceVariableNames: 'parseStringSelector'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCDefinitionScanner subclass: #SmaCCGrammarStylerScanner
	instanceVariableNames: 'view offset insideRule token startLine stopLine insideClassDefinition afterColon'
	classVariableNames: ''
	poolDictionaries: 'ScintillaConstants'
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCParserGenerator guid: (GUID fromString: '{DA5AC953-F2B8-4A4F-AA72-B85DB3758B8B}')!
SmaCCParserGenerator comment: ''!
!SmaCCParserGenerator categoriesForClass!Unclassified! !
!SmaCCParserGenerator methodsFor!

addMenuItems
	| classes menu |
	classes := self codeGeneratorClasses.
	classes size > 1 ifFalse: [^self].
	menu := self view menuBar items
		detect: [:each | (each text copyWithout: $&) sameAs: 'Options']
		ifNone: [self view menuBar items last].
	menu addSeparator.
	classes
		do: [:each | (menu addCommand: (self commandForGenerator: each) description: each typeName) isRadioButtonStyle: true]!

allWords
	| words stream scintillaView lastId mask ids nextId |
	scintillaView := self definitionPresenter view.
	words := Set new.
	stream := WriteStream on: String new.
	lastId := nil.
	mask := scintillaView maxStyle.
	ids := Array
		with: (scintillaView styleNamed: #nonTerminal) id
		with: (scintillaView styleNamed: #terminal) id
		with: (scintillaView styleNamed: #keyword) id
		with: (scintillaView styleNamed: #variable) id.
	1
		to: scintillaView textLength
		do:
			[:index | 
			nextId := (scintillaView styleMaskAt: index) bitAnd: mask.
			nextId ~= lastId
				ifTrue:
					[(stream position > 0 and: [ids includes: lastId]) ifTrue: [words add: stream contents].
					stream reset.
					lastId := nextId].
			stream nextPut: (scintillaView characterAt: index)].
	^words!

codeGeneratorClass
	^SmaCCCodeGenerator withAllSubclasses detect: [:each | each typeName = codeGeneratorType]
		ifNone: [SmaCCSmalltalkCodeGenerator new]!

codeGeneratorClasses
	^SmaCCCodeGenerator withAllSubclasses select: [:each | each typeName notNil]!

collapseAll
	self showCollapsed: true!

collapseNode
	self showNodeCollapsed: true!

commandForGenerator: each
	^Message selector: #setCodeGenerator: argument: each typeName!

compile
	^self compile: type!

compile: aSymbol
	| grammar stream oldCharacterSize parserCompiler messages |
	(scannerClassName isEmpty or: [parserClassName isEmpty])
		ifTrue: [^MessageBox warning: 'Both classes are not specified'].
	messages := OrderedCollection new.
	Cursor wait
		showWhile:
			[self verifyParser ifFalse: [^self].
			oldCharacterSize := SmaCCGrammar maximumCharacterValue.
			[SmaCCGrammar maximumCharacterValue: (allowUnicode ifTrue: [2 ** 16 - 1] ifFalse: [255]).
			parserCompiler := self compiler.
			parserCompiler
				codeGenerator: self codeGeneratorClass new;
				buildDefinition: self definitionPresenter value;
				scannerClass: scannerClassName;
				parserClass: parserClassName.
			grammar := parserCompiler grammar.
			grammar type: aSymbol.
			stream := WriteStream on: String new.
			[parserCompiler createChanges.
			parserCompiler codeGenerator compileChanges.
			self symbolsPresenter value: parserCompiler symbolsComment.
			self itemSetsPresenter value: parserCompiler itemSetsComment]
				on: SmaCCCompilationNotification
				do:
					[:ex | 
					(ShowPrecedenceConflicts or: [('*precedence*' match: ex messageText) not])
						ifTrue: [messages add: ex messageText , String lineDelimiter , String lineDelimiter , ex tag].
					ex pass].
			messages asSortedCollection
				do:
					[:each | 
					stream
						nextPutAll: '-------------------------';
						cr;
						nextPutAll: each;
						cr].
			self messagesPresenter value: stream contents.
			self messagesPresenter value notEmpty
				ifTrue:
					[(view viewNamed: 'slidey') pin.
					self messagesPresenter setFocus].
			self definitionPresenter isModified: false] ensure: [SmaCCGrammar maximumCharacterValue: oldCharacterSize]]!

compiler
	^SmaCCGrammarCompiler new!

createComponents
	super createComponents.
	self add: TextPresenter new name: 'Definition'.
	self add: TextPresenter new name: 'Messages'.
	self add: TextPresenter new name: 'Test'.
	self add: TextPresenter new name: 'Item Sets'.
	self add: TextPresenter new name: 'Symbols'!

createSchematicWiring
	super createSchematicWiring.
	self
		when: #closeRequested:
		send: #onCloseRequested:
		to: self.
	(view viewNamed: 'slidey') unpin.
	(self definitionPresenter)
		when: #valueChanged
			send: #updateLineNumberWidth:
			to: self
			with: self definitionPresenter view;
		when: #timerTick:
			send: #onTimerTick:
			to: self;
		when: #charAdded:
			send: #onCharAdded:
			to: self.
	(self definitionPresenter view)
		hasFoldMargin: true;
		foldMarkerStyle: #boxTree;
		isFoldingEnabled: true;
		styler: SmaCCGrammarStyler new;
		autoCompletionSeparator: $ ;
		foldFlags: SC_FOLDFLAG_LINEBEFORE_EXPANDED.
	self testPresenter
		when: #valueChanged
		send: #updateLineNumberWidth:
		to: self
		with: self testPresenter view.
	self messagesPresenter
		when: #valueChanged
		send: #updateLineNumberWidth:
		to: self
		with: self messagesPresenter view.
	self
		updateLineNumberWidth: self definitionPresenter view;
		updateLineNumberWidth: self testPresenter view;
		updateLineNumberWidth: self messagesPresenter view!

definitionPresenter
	^self presenterNamed: 'Definition'!

displayError: aString at: aSmallInteger for: aTextPresenter
	| originalString |
	aTextPresenter setFocus.
	originalString := aTextPresenter value.
	aSmallInteger = 0 
		ifTrue: 
			[aTextPresenter value: aString.
			aTextPresenter view selectionStart: 1 end: aString size]
		ifFalse: 
			[aTextPresenter value: (originalString copyFrom: 1 to: aSmallInteger - 1) , aString 
						, (originalString copyFrom: aSmallInteger to: originalString size).
			aTextPresenter view selectionStart: aSmallInteger end: aSmallInteger + aString size - 1]!

expandAll
	self showCollapsed: false!

expandNode
	self showNodeCollapsed: false!

findParserClass
	| class |
	self definitionPresenter isModified ifTrue: [self promptToSaveChanges ifFalse: [^self]].
	class := ChoicePrompter
		choices: (SmaCCParser allSubclasses asSortedCollection: [:a :b | a name < b name])
		caption: 'What class do you want to use?'.
	class isNil ifTrue: [^self].
	parserClassName := class name.
	self updateDefinitionFor: class.
	self definitionPresenter isModified: false!

getWord
	| start stop index scintillaView mask id |
	scintillaView := self definitionPresenter view.
	mask := scintillaView maxStyle.
	(0 to: 1)
		detect:
			[:i | 
			index := scintillaView caretPosition - i.
			id := (scintillaView styleMaskAt: index) bitAnd: mask.
			id = (scintillaView styleNamed: #nonTerminal) id
				or: [id = (scintillaView styleNamed: #terminal) id]]
		ifNone: [^nil].
	stop := start := index.
	[start > 0 and: [id = ((scintillaView styleMaskAt: start) bitAnd: mask)]]
		whileTrue: [start := start - 1].
	[stop <= scintillaView textLength and: [id = ((scintillaView styleMaskAt: stop) bitAnd: mask)]]
		whileTrue: [stop := stop + 1].
	^scintillaView plainTextFrom: start + 1 to: stop - 1!

gotoDefinition
	self getWord ifNotNil: [:word | self gotoDefinitionOf: word]!

gotoDefinitionOf: aString
	| id scintillaView mask index ids |
	scintillaView := self definitionPresenter view.
	mask := scintillaView maxStyle.
	index := 1.
	ids := Array
		with: (scintillaView styleNamed: #nonTerminalDefinition) id
		with: (scintillaView styleNamed: #terminalDefinition) id.
	[index <= scintillaView textLength]
		whileTrue:
			[id := (scintillaView styleMaskAt: index) bitAnd: mask.
			(ids includes: id)
				ifTrue:
					[(aString = (scintillaView plainTextFrom: index to: index + aString size - 1)
						and: [(ids includes: ((scintillaView styleMaskAt: index + aString size) bitAnd: mask)) not])
						ifTrue: [^scintillaView selectionStart: index end: index + aString size - 1].
					[index <= scintillaView textLength
						and: [ids includes: ((scintillaView styleMaskAt: index) bitAnd: mask)]]
						whileTrue: [index := index + 1]]
				ifFalse: [index := index + 1]]!

initialize
	super initialize.
	allowUnicode := false.
	type := #LALR1.
	parserClassName := ''.
	scannerClassName := ''.
	codeGeneratorType := #Smalltalk!

itemSetsPresenter
	^self presenterNamed: 'Item Sets'!

maybeShowCompletionList
	| position start scintillaView string words |
	scintillaView := self definitionPresenter view.
	start := position := scintillaView caretPosition - 1.
	[start >= 1 and: [(scintillaView characterAt: start) isWhitespace not]]
		whileTrue: [start := start - 1].
	start := start + 1.
	start >= position ifTrue: [^self].
	string := scintillaView plainTextFrom: start to: position.
	words := self allWords
		select:
			[:each | 
			string ~= each
				and:
					[(each beginsWith: string ignoreCase: true)
						and: [(each includes: scintillaView autoCompletionSeparator) not]]].
	scintillaView showAutoCompletionList: words prefixLength: string size!

messagesPresenter
	^self presenterNamed: 'Messages'!

onCharAdded: aCharacter
	self definitionPresenter view setTimer: SCI_AUTOCSHOW interval: 30!

onCloseRequested: boolValueHolder
	boolValueHolder value: (self definitionPresenter isModified not or: [self promptToSaveChanges])!

onTimerTick: anInteger
	anInteger == SCI_AUTOCSHOW ifFalse: [^self].
	self definitionPresenter view killTimer: SCI_AUTOCSHOW.
	self definitionPresenter view isAutoCompletionActive ifFalse: [self maybeShowCompletionList]!

onViewOpened
	self addMenuItems.
	^super onViewOpened!

parse
	self parseAndEvaluate: [:each | MessageBox notify: 'Parses without errors']!

parseAndEvaluate: aBlock
	| class |
	class := self parserClass.
	class isNil ifTrue: [^MessageBox warning: 'No parser defined'].
	aBlock
		value:
			(class
				parse: self testPresenter value
				onError:
					[:aString :position | 
					self
						displayError: aString
						at: position
						for: self testPresenter.
					^self])!

parseAndInspect
	self parseAndEvaluate: [:each | each inspect]!

parserClass
	^Smalltalk at: parserClassName asSymbol ifAbsent: [nil]!

promptToSaveChanges
	^(MessageBox confirm: 'The definition has been modified. Do you wish to retain your changes?') not!

queryCommand: aCommandQuery
	| cmd lalr1 |
	cmd := aCommandQuery commandSymbol.
	cmd = #setScannerClass
		ifTrue:
			[aCommandQuery
				text:
					(scannerClassName isEmpty ifTrue: ['Scanner Class...'] ifFalse: ['Scanner Class (' , scannerClassName , ')...'])].
	cmd = #setParserClass
		ifTrue:
			[aCommandQuery
				text: (parserClassName isEmpty ifTrue: ['Parser Class...'] ifFalse: ['Parser Class (' , parserClassName , ')...'])].
	cmd = #setCodeGenerator:
		ifTrue:
			[self codeGeneratorClasses
				do:
					[:each | 
					aCommandQuery commandDescription command arguments first = each typeName
						ifTrue:
							[^aCommandQuery
								isChecked: codeGeneratorType == each typeName;
								isEnabled: aCommandQuery isChecked not]]].
	lalr1 := type == #LALR1.
	cmd = #setLALR1
		ifTrue:
			[^aCommandQuery
				isEnabled: lalr1 not;
				isChecked: lalr1].
	cmd = #setLR1
		ifTrue:
			[^aCommandQuery
				isEnabled: lalr1;
				isChecked: lalr1 not].
	cmd = #toggleUnicodeCharacters
		ifTrue:
			[^aCommandQuery
				beDisabled;
				isChecked: allowUnicode].
	super queryCommand: aCommandQuery!

removeCommentedPartOf: aString 
	| inStream outStream |
	inStream := ReadStream on: aString.
	outStream := WriteStream on: String new.
	inStream upTo: $".
	[inStream atEnd] whileFalse: 
			[(inStream peekFor: $") ifTrue: [outStream nextPut: $"].
			outStream nextPutAll: (inStream upTo: $")].
	^outStream contents!

restyleText
	| scintillaView |
	scintillaView := self definitionPresenter view.
	(scintillaView styler)
		parseStringSelector: self codeGeneratorClass parseSourceSelector;
		colorText: (scintillaView plainTextFrom: 1 to: scintillaView textLength)
			in: scintillaView
			startingAt: 1!

setCodeGenerator: aSymbol
	codeGeneratorType := aSymbol.
	self restyleText!

setLALR1
	type := #LALR1!

setLR1
	type := #LR1!

setParserClass
	| name |
	name := Prompter prompt: 'Enter parser class name:'.
	name isNil ifTrue: [^self].
	(name notEmpty and: [name first isUpperCase and: [SmalltalkScanner isVariable: name]])
		ifFalse: [^MessageBox warning: 'Invalid Class Name'].
	parserClassName := name asSymbol!

setScannerClass
	| name |
	name := Prompter prompt: 'Enter scanner class name:'.
	name isNil ifTrue: [^self].
	(name notEmpty and: [name first isUpperCase and: [SmalltalkScanner isVariable: name]])
		ifFalse: [^MessageBox warning: 'Invalid Class Name'].
	scannerClassName := name asSymbol!

showCollapsed: aBoolean
	self showCollapsed: aBoolean singleNode: false!

showCollapsed: aBoolean singleNode: singleBoolean
	| scintillaView endLine |
	scintillaView := self definitionPresenter view.
	endLine := singleBoolean
		ifTrue: [scintillaView lineFromPosition: scintillaView caretPosition]
		ifFalse: [scintillaView lineCount].
	endLine
		to: 1
		by: -1
		do:
			[:i | 
			((scintillaView sciGetFoldLevel: i - 1) bitAnd: SC_FOLDLEVELHEADERFLAG) = SC_FOLDLEVELHEADERFLAG
				ifTrue:
					[(scintillaView sciGetFoldExpanded: i - 1) = aBoolean ifTrue: [scintillaView toggleFold: i].
					singleBoolean ifTrue: [^self]]]!

showNodeCollapsed: aBoolean
	self showCollapsed: aBoolean singleNode: true!

styler
	^self definitionPresenter view styler!

symbolsPresenter
	^self presenterNamed: 'Symbols'!

testPresenter
	^self presenterNamed: 'Test'!

toggleUnicodeCharacters
	allowUnicode := allowUnicode not!

updateDefinitionFor: parserClass
	| scannerClass |
	scannerClass := [parserClass scannerClass] on: Error do: [:ex | ex return: nil].
	scannerClass notNil ifTrue: [scannerClassName := scannerClass name].
	(parserClass class includesSelector: #definitionComment)
		ifTrue:
			[self definitionPresenter value: (self removeCommentedPartOf: (parserClass class sourceCodeAt: #definitionComment))]
		ifFalse:
			[| source |
			source := ''.
			(parserClass class includesSelector: #parserDefinitionComment)
				ifTrue: [source := self removeCommentedPartOf: (parserClass class sourceCodeAt: #parserDefinitionComment)].
			(scannerClass notNil and: [scannerClass class includesSelector: #scannerDefinitionComment])
				ifTrue:
					[source := (self removeCommentedPartOf: (scannerClass class sourceCodeAt: #scannerDefinitionComment))
						, String lineDelimiter , source].
			self definitionPresenter value: source].
	self messagesPresenter value: ''.
	self symbolsPresenter value: ''.
	self itemSetsPresenter value: ''.
	self updateLineNumberWidth: self definitionPresenter view!

updateLineNumberWidth: aScintillaView
	aScintillaView lineNumberMargin
		ifNotNil: [:margin | margin width: aScintillaView requiredLineMarginWidth]!

verifyParser
	^[SmaCCGrammarCompiler new buildDefinition: self definitionPresenter value.
	true]
		on: SmaCCParserError
		do:
			[:ex | 
			self
				displayError: ex description
				at: ex tag position
				for: self definitionPresenter.
			ex return: false]! !
!SmaCCParserGenerator categoriesFor: #addMenuItems!private! !
!SmaCCParserGenerator categoriesFor: #allWords!autocompletion!public! !
!SmaCCParserGenerator categoriesFor: #codeGeneratorClass!private! !
!SmaCCParserGenerator categoriesFor: #codeGeneratorClasses!private! !
!SmaCCParserGenerator categoriesFor: #collapseAll!actions!public! !
!SmaCCParserGenerator categoriesFor: #collapseNode!actions!public! !
!SmaCCParserGenerator categoriesFor: #commandForGenerator:!private! !
!SmaCCParserGenerator categoriesFor: #compile!actions!private! !
!SmaCCParserGenerator categoriesFor: #compile:!private! !
!SmaCCParserGenerator categoriesFor: #compiler!private! !
!SmaCCParserGenerator categoriesFor: #createComponents!private! !
!SmaCCParserGenerator categoriesFor: #createSchematicWiring!private! !
!SmaCCParserGenerator categoriesFor: #definitionPresenter!presenters!private! !
!SmaCCParserGenerator categoriesFor: #displayError:at:for:!public! !
!SmaCCParserGenerator categoriesFor: #expandAll!actions!public! !
!SmaCCParserGenerator categoriesFor: #expandNode!actions!public! !
!SmaCCParserGenerator categoriesFor: #findParserClass!actions!private! !
!SmaCCParserGenerator categoriesFor: #getWord!private! !
!SmaCCParserGenerator categoriesFor: #gotoDefinition!actions!private! !
!SmaCCParserGenerator categoriesFor: #gotoDefinitionOf:!actions!private! !
!SmaCCParserGenerator categoriesFor: #initialize!initialize/release!private! !
!SmaCCParserGenerator categoriesFor: #itemSetsPresenter!presenters!public! !
!SmaCCParserGenerator categoriesFor: #maybeShowCompletionList!autocompletion!private! !
!SmaCCParserGenerator categoriesFor: #messagesPresenter!presenters!private! !
!SmaCCParserGenerator categoriesFor: #onCharAdded:!autocompletion!private! !
!SmaCCParserGenerator categoriesFor: #onCloseRequested:!actions!public! !
!SmaCCParserGenerator categoriesFor: #onTimerTick:!autocompletion!private! !
!SmaCCParserGenerator categoriesFor: #onViewOpened!private! !
!SmaCCParserGenerator categoriesFor: #parse!actions!private! !
!SmaCCParserGenerator categoriesFor: #parseAndEvaluate:!private! !
!SmaCCParserGenerator categoriesFor: #parseAndInspect!actions!private! !
!SmaCCParserGenerator categoriesFor: #parserClass!private! !
!SmaCCParserGenerator categoriesFor: #promptToSaveChanges!actions!private! !
!SmaCCParserGenerator categoriesFor: #queryCommand:!public! !
!SmaCCParserGenerator categoriesFor: #removeCommentedPartOf:!private! !
!SmaCCParserGenerator categoriesFor: #restyleText!actions!public! !
!SmaCCParserGenerator categoriesFor: #setCodeGenerator:!public! !
!SmaCCParserGenerator categoriesFor: #setLALR1!actions!public! !
!SmaCCParserGenerator categoriesFor: #setLR1!actions!public! !
!SmaCCParserGenerator categoriesFor: #setParserClass!actions!public! !
!SmaCCParserGenerator categoriesFor: #setScannerClass!actions!public! !
!SmaCCParserGenerator categoriesFor: #showCollapsed:!actions!public! !
!SmaCCParserGenerator categoriesFor: #showCollapsed:singleNode:!actions!public! !
!SmaCCParserGenerator categoriesFor: #showNodeCollapsed:!actions!public! !
!SmaCCParserGenerator categoriesFor: #styler!actions!public! !
!SmaCCParserGenerator categoriesFor: #symbolsPresenter!presenters!public! !
!SmaCCParserGenerator categoriesFor: #testPresenter!presenters!public! !
!SmaCCParserGenerator categoriesFor: #toggleUnicodeCharacters!actions!public! !
!SmaCCParserGenerator categoriesFor: #updateDefinitionFor:!private! !
!SmaCCParserGenerator categoriesFor: #updateLineNumberWidth:!private! !
!SmaCCParserGenerator categoriesFor: #verifyParser!private! !

!SmaCCParserGenerator class methodsFor!

initialize
	ShowPrecedenceConflicts := false.
	Smalltalk developmentSystem addAdditionalToolsFolderIcon: self toolsFolderIcon!

resource_Default_view
	"Answer the literal data from which the 'Default view' resource can be reconstituted.
	DO NOT EDIT OR RECATEGORIZE THIS METHOD.

	If you wish to modify this resource evaluate:
	ViewComposer openOn: (ResourceIdentifier class: self selector: #resource_Default_view)
	"

	^#(#'!!STL' 3 788558 10 ##(Smalltalk.STBViewProxy)  8 ##(Smalltalk.ShellView)  98 27 0 0 98 2 27131905 131073 416 0 524550 ##(Smalltalk.ColorRef)  8 4278190080 328198 ##(Smalltalk.Point)  1141 921 551 0 0 0 416 1180166 ##(Smalltalk.ProportionalLayout)  234 240 98 0 32 234 256 608 0 461638 4 ##(Smalltalk.MenuBar)  0 16 98 3 265030 4 ##(Smalltalk.Menu)  0 16 98 4 984134 2 ##(Smalltalk.CommandMenuItem)  1 1180998 4 ##(Smalltalk.CommandDescription)  8 #findParserClass 8 'Open' 9375 1 0 0 0 738 1 770 8 #compile 8 'Save' 13479 1 0 0 0 983366 1 ##(Smalltalk.DividerMenuItem)  4097 738 1 770 8 #exit 8 '&Close' 17639 1 263494 3 ##(Smalltalk.Icon)  0 16 1114638 ##(Smalltalk.STBSingletonProxy)  8 ##(Smalltalk.ImageRelativeFileLocator)  8 #current 8 'CloseWindow.ico' 2032142 ##(Smalltalk.STBExternalResourceLibraryProxy)  8 'dolphindr006.dll' 0 0 0 8 '&File' 0 134217729 0 0 32703 0 0 690 0 16 98 15 738 1 770 8 #editFind 8 '&Find...' 9357 1 994 0 16 1040 47 1034 8 ##(Smalltalk.ShellLibrary)  8 #default 0 0 738 1 770 8 #findNext 8 'Find &Next' 1 1 0 0 0 738 1 770 8 #findReplace 8 '&Replace' 9361 1 0 0 0 898 4097 738 1 770 8 #cutSelection 8 'Cu&t' 9393 1 994 0 16 1040 8 'EditCut.ico' 1120 0 0 738 1 770 8 #copySelection 8 '&Copy' 9351 1 994 0 16 1040 8 'EditCopy.ico' 1120 0 0 738 1 770 8 #pasteClipboard 8 '&Paste' 9389 1 994 0 16 1040 8 'EditPaste.ico' 1120 0 0 898 4097 738 1 770 8 #expandNode 8 'Expand' 9355 1 0 0 0 738 1 770 8 #collapseNode 8 'Collapse' 9381 1 0 0 0 898 4097 738 1 770 8 #expandAll 8 'E&xpand All' 13451 1 0 0 0 738 1 770 8 #collapseAll 8 'Co&llapse All' 13477 1 0 0 0 898 4097 738 1 770 8 #gotoDefinition 8 'Goto &Definition' 1271 1 0 0 0 8 '&Edit' 0 134217729 0 0 32727 0 0 690 0 16 98 5 738 1 770 8 #setScannerClass 8 '&Scanner Class...' 1025 1 0 0 0 738 1 770 8 #setParserClass 8 '&Parser Class...' 1 1 0 0 0 898 4097 738 1025 770 8 #setLALR1 8 'L&ALR(1)' 1 1 0 0 0 738 1025 770 8 #setLR1 8 '&LR(1)' 1 1 0 0 0 8 '&Options' 0 134217729 0 0 32737 0 0 8 '' 0 134217729 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 983302 ##(Smalltalk.MessageSequence)  202 208 98 3 721670 ##(Smalltalk.MessageSend)  8 #createAt:extent: 98 2 530 2879 21 530 1141 921 416 2546 8 #text: 98 1 8 'SmaCC Parser Generator' 416 2546 8 #updateMenuBar 608 416 983302 ##(Smalltalk.WINDOWPLACEMENT)  8 #[44 0 0 0 0 0 0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 159 5 0 0 10 0 0 0 217 7 0 0 214 1 0 0] 98 1 410 8 ##(Smalltalk.ContainerView)  98 15 0 416 98 2 8 1140850688 131073 2800 0 0 0 7 0 0 0 2800 562 234 240 608 16 234 256 98 4 410 8 ##(Smalltalk.SlideyInneyOuteyThing)  98 23 0 2800 98 2 8 1140850688 131073 2944 0 482 8 4278190080 0 519 0 0 0 2944 655878 ##(Smalltalk.CardLayout)  202 208 98 4 721414 ##(Smalltalk.Association)  8 'Messages' 410 8 ##(Smalltalk.ScintillaView)  98 46 0 410 8 ##(Smalltalk.SlidingCardTray)  98 22 0 2944 98 2 8 1140850688 131073 3216 0 482 3040 0 7 0 0 0 3216 3072 234 256 98 8 410 3184 98 46 0 3216 98 2 8 1176571972 1025 3344 721990 2 ##(Smalltalk.ValueHolder)  0 32 1034 8 ##(Smalltalk.SearchPolicy)  8 #equality 0 482 8 4278190080 0 5 0 0 0 3344 0 8 4294903837 852486 ##(Smalltalk.NullConverter)  0 0 9 0 234 256 98 2 8 #normal 1182726 ##(Smalltalk.ScintillaTextStyle)  1 0 0 1 0 0 0 0 3600 0 0 0 98 40 3632 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1245510 1 ##(Smalltalk.NullScintillaStyler)  3600 234 256 98 2 1312 1639942 ##(Smalltalk.ScintillaMarkerDefinition)  1 1 786694 ##(Smalltalk.IndexedColor)  33554433 3762 33554471 3344 8 #circle 202 208 608 0 63 0 0 0 0 0 3762 33554447 0 0 0 0 0 0 8 '' 3 234 256 98 2 8 #container 3568 0 0 0 0 1 0 0 2482 202 208 98 8 2546 2576 98 2 530 1 37 530 1093 305 3344 2546 8 #selectionRange: 98 1 525062 ##(Smalltalk.Interval)  3 1 3 3344 2546 8 #isTextModified: 98 1 32 3344 2546 8 #modificationEventMask: 98 1 9215 3344 2546 8 #indicatorDefinitions: 98 1 98 3 1836038 ##(Smalltalk.ScintillaIndicatorDefinition)  1 3344 65025 3 4274 3 3344 33423361 5 4274 5 3344 511 1 3344 2546 8 #margins: 98 1 98 3 984582 ##(Smalltalk.ScintillaMargin)  1 3344 1 3 32 1 4402 3 3344 33 1 16 67108863 4402 5 3344 1 1 16 -67108863 3344 2546 8 #tabIndents: 98 1 16 3344 2546 8 #tabWidth: 98 1 9 3344 2738 8 #[44 0 0 0 0 0 0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 18 0 0 0 34 2 0 0 170 0 0 0] 98 0 530 193 193 0 27 8 'Symbols' 410 3184 98 46 0 3216 98 2 8 1174475012 1025 4640 3410 0 32 3440 0 482 8 4278190080 0 5 690 0 16 98 10 738 1 770 1232 8 '&Find' 1025 1 994 0 16 1040 47 1120 0 0 738 1 770 1360 8 'Find &Next' 1 1 0 0 0 738 1 770 1424 8 '&Replace' 1025 1 0 0 0 898 4097 738 1 770 1504 8 'Cu&t' 1025 1 994 0 16 1040 8 'EditCut.ico' 1120 0 0 738 1 770 1600 8 '&Copy' 1025 1 994 0 16 1040 8 'EditCopy.ico' 1120 0 0 738 1 770 1696 8 '&Paste' 1025 1 994 0 16 1040 8 'EditPaste.ico' 1120 0 0 898 4097 738 1 770 8 #parse 8 'Evaluate' 1025 1 0 0 0 738 1 770 8 #parseAndInspect 8 'Inspect' 1025 1 0 0 0 8 '' 0 134217729 0 0 0 0 0 0 0 4640 0 8 4294903837 3538 0 0 9 0 234 256 98 6 8 #lineNumber 3618 67 0 0 1 0 0 0 0 5424 0 0 0 8 #indentGuide 3618 75 0 0 1 0 0 0 0 5456 0 0 0 3600 3618 1 0 0 1 0 0 0 0 3600 0 0 0 98 40 5488 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 5440 0 0 0 5472 0 0 3666 3600 234 256 98 2 1312 3730 0 1 3776 3792 4640 3808 202 208 608 0 63 0 0 0 0 0 3840 0 0 0 0 0 0 8 '' 1 234 256 608 0 0 0 0 1 0 0 2482 202 208 98 10 2546 2576 98 2 530 1 37 530 1093 305 4640 2546 8 #contextMenu: 98 1 4752 4640 2546 4048 98 1 4082 3 1 3 4640 2546 4128 98 1 32 4640 2546 4176 98 1 9215 4640 2546 4224 98 1 98 3 4274 1 4640 65025 3 4274 3 4640 33423361 5 4274 5 4640 511 1 4640 2546 8 #wordWrap: 98 1 16 4640 2546 4352 98 1 98 3 4402 1 4640 1 3 32 1 4402 3 4640 1 1 32 67108863 4402 5 4640 1 1 32 1 4640 2546 8 #canHScroll: 98 1 32 4640 2546 4480 98 1 16 4640 2738 8 #[44 0 0 0 0 0 0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 18 0 0 0 34 2 0 0 170 0 0 0] 98 0 4608 0 27 8 'Test' 410 3184 98 46 0 3216 98 2 8 1176571972 1025 6288 3410 0 32 3440 0 482 3504 0 5 0 0 0 6288 0 8 4294903837 3538 0 0 9 0 234 256 98 2 3600 3618 1 0 0 1 0 0 0 0 3600 0 0 0 98 40 6448 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3666 3600 234 256 98 2 1312 3730 1 1 3776 3792 6288 3808 202 208 608 0 63 0 0 0 0 0 3840 0 0 0 0 0 0 3856 3 234 256 98 2 3904 6416 0 0 0 0 1 0 0 2482 202 208 98 8 2546 2576 98 2 530 1 37 530 1093 305 6288 2546 4048 98 1 4082 3 1 3 6288 2546 4128 98 1 32 6288 2546 4176 98 1 9215 6288 2546 4224 98 1 98 3 4274 1 6288 65025 3 4274 3 6288 33423361 5 4274 5 6288 511 1 6288 2546 4352 98 1 98 3 4402 1 6288 1 3 32 1 4402 3 6288 33 1 16 67108863 4402 5 6288 1 1 16 -67108863 6288 2546 4480 98 1 16 6288 2546 4528 98 1 9 6288 2738 8 #[44 0 0 0 0 0 0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 18 0 0 0 34 2 0 0 170 0 0 0] 98 0 4608 0 27 8 'Item Sets' 3168 8 'Messages' 0 410 8 ##(Smalltalk.TabViewXP)  98 28 0 2944 98 2 8 1140916802 1 7152 590662 2 ##(Smalltalk.ListModel)  202 208 98 4 3152 8 'Symbols' 8 'Item Sets' 8 'Test' 0 1034 3456 8 #identity 482 3040 0 1 0 0 0 7152 0 8 4294903267 8 ##(Smalltalk.BasicListAbstract)  8 ##(Smalltalk.IconicListAbstract)  1034 8 ##(Smalltalk.IconImageManager)  1072 0 0 0 0 0 8 #noIcons 0 0 0 0 0 2482 202 208 98 3 2546 2576 98 2 530 1 1 530 1109 397 7152 2546 8 #selectionByIndex:ifAbsent: 98 2 3 2546 8 #yourself 608 0 7152 2546 8 #tcmSetExtendedStyle:dwExStyle: 98 2 -1 1 7152 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 42 2 0 0 198 0 0 0] 98 0 4608 0 27 0 2944 530 33 33 1049862 ##(Smalltalk.ButtonInteractor)  3216 0 590342 ##(Smalltalk.Rectangle)  530 1057 3 530 1089 35 1 770 8 #togglePin 8 'Pin or Unpin the tray' 1 1 0 0 0 2482 202 208 98 1 2546 2576 98 2 530 9 9 530 1093 341 3216 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 4 0 0 0 4 0 0 0 38 2 0 0 174 0 0 0] 98 4 3168 3344 6288 4640 4608 0 27 98 2 8 1174475012 1025 3168 3410 0 32 3440 0 482 4736 0 7 0 0 0 3168 0 8 4294903837 3538 0 0 11 0 234 256 98 6 5424 3618 67 0 0 16385 0 0 0 0 5424 0 0 0 5456 3618 75 0 0 1 0 0 0 0 5456 0 0 0 3600 3618 1 0 0 1 0 0 0 0 3600 0 0 0 98 40 8256 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 8224 0 0 0 8240 0 0 3666 3600 234 256 98 2 1312 3730 0 1 3776 3792 3168 3808 202 208 608 0 63 0 0 0 0 0 3840 0 0 0 0 0 0 5600 1 234 256 98 2 3904 8192 0 0 0 0 1 0 0 2482 202 208 98 7 2546 2576 98 2 530 1 37 530 1093 305 3168 2546 4048 98 1 4082 3 1 3 3168 2546 4128 98 1 32 3168 2546 4176 98 1 9215 3168 2546 4224 98 1 98 3 4274 1 3168 65025 3 4274 3 3168 33423361 5 4274 5 3168 511 1 3168 2546 4352 98 1 98 3 4402 1 3168 1 3 32 1 4402 3 3168 1 1 32 67108863 4402 5 3168 1 1 32 1 3168 2546 4480 98 1 16 3168 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 18 0 0 0 34 2 0 0 170 0 0 0] 98 0 4608 0 27 3122 7328 4640 3122 7296 3344 3122 7312 6288 3168 234 256 608 0 7152 3216 530 201 201 401 1 524319 0 0 2482 202 208 98 1 2546 2576 98 2 530 1 415 530 1109 397 2944 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 207 0 0 0 42 2 0 0 149 1 0 0] 98 2 3216 7152 4608 0 27 8 'slidey' 410 3184 98 46 0 2800 98 2 8 1174475012 1025 9152 3410 0 32 3440 0 482 4736 0 7 690 0 16 98 10 738 1 770 1232 8 '&Find' 1025 1 994 0 16 1040 47 1120 0 0 738 1 770 1360 8 'Find &Next' 1 1 0 0 0 738 1 770 1424 8 '&Replace' 1025 1 0 0 0 898 4097 738 1 770 1504 8 'Cu&t' 1025 1 994 0 16 1040 8 'EditCut.ico' 1120 0 0 738 1 770 1600 8 '&Copy' 1025 1 994 0 16 1040 8 'EditCopy.ico' 1120 0 0 738 1 770 1696 8 '&Paste' 1025 1 994 0 16 1040 8 'EditPaste.ico' 1120 0 0 898 4097 738 1 770 1952 8 'Expand All' 1025 1 0 0 0 738 1 770 2016 8 'Collapse All' 1025 1 0 0 0 8 '' 0 134217729 0 0 0 0 0 0 0 9152 0 8 4294903837 3538 0 0 9 0 234 256 98 32 8 #class 3618 11 3762 33554457 0 1 0 0 0 0 9888 8 'class' 0 0 8 #variable 3618 15 9920 0 1 0 0 0 0 9952 0 0 0 8 #internalWhitespace 3618 23 0 0 1 0 0 0 0 9984 0 0 0 8 #nonTerminalDefinition 3618 25 3762 33554441 0 3 0 0 0 0 10016 0 0 0 8 #comment 3618 5 3840 0 9 0 0 0 0 10064 0 0 0 5424 3618 67 0 0 1 0 0 0 0 5424 0 0 0 8 #character 3618 21 0 0 3 0 0 0 0 10112 0 0 0 5456 3618 75 0 0 1 0 0 0 0 5456 0 0 0 3600 3618 1 0 0 1 0 0 0 0 3600 0 0 0 8 #keyword 3618 19 196934 1 ##(Smalltalk.RGB)  20971521 0 3 0 0 0 0 10176 0 0 0 8 #terminal 3618 7 3762 33554465 0 3 0 0 0 0 10240 0 0 0 8 #whitespace 3618 13 0 0 1 0 0 0 0 10288 0 0 0 8 #error 3618 17 3762 33554459 0 3 0 0 0 0 10320 0 0 0 8 #code 3618 9 9920 0 1 0 0 0 0 10368 0 0 0 8 #terminalDefinition 3618 27 10272 0 3 0 0 0 0 10400 0 0 0 8 #nonTerminal 3618 3 10048 0 3 0 0 0 0 10432 0 0 0 98 40 10160 10448 10080 10256 10384 9904 10304 9968 10336 10192 10128 10000 10032 10416 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 10096 0 0 0 10144 0 0 3666 3600 234 256 608 202 208 608 0 63 0 0 0 0 0 3840 0 0 0 0 0 0 5600 1 234 256 98 2 3904 9856 0 0 0 0 1 0 0 2482 202 208 98 11 2546 2576 98 2 530 1 1 530 1109 397 9152 2546 5760 98 1 9248 9152 2546 4048 98 1 4082 3 1 3 9152 2546 4128 98 1 32 9152 2546 4176 98 1 9215 9152 2546 4224 98 1 98 3 4274 1 9152 65025 3 4274 3 9152 33423361 5 4274 5 9152 511 1 9152 2546 6016 98 1 16 9152 2546 4352 98 1 98 3 4402 1 9152 1 3 32 1 4402 3 9152 1 1 32 67108863 4402 5 9152 1 1 16 -67108863 9152 2546 6160 98 1 32 9152 2546 4480 98 1 16 9152 2546 8 #isAutoCompletionCaseInsensitive: 98 1 16 9152 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 42 2 0 0 198 0 0 0] 98 0 4608 0 27 8 'Definition' 0 2482 202 208 98 1 2546 2576 98 2 530 1 1 530 1109 811 2800 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 42 2 0 0 149 1 0 0] 98 3 9152 410 8 ##(Smalltalk.Splitter)  98 12 0 2800 98 2 8 1140850688 1 11376 0 482 8 4278190080 0 519 0 0 0 11376 2482 202 208 98 1 2546 2576 98 2 530 1 397 530 1109 19 11376 2738 8 #[44 0 0 0 0 0 0 0 1 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 198 0 0 0 42 2 0 0 207 0 0 0] 98 0 4608 0 27 2944 4608 0 27 4608 0 27 )!

toolsFolderIcon
	^SmalltalkSystemIcon 
		icon: self icon
		description: 'SmaCC Parser Generator'
		openBlock: [:folder :item | self show]!

uninitialize
	Smalltalk developmentSystem removeSystemFolderIcon: self toolsFolderIcon! !
!SmaCCParserGenerator class categoriesFor: #initialize!public! !
!SmaCCParserGenerator class categoriesFor: #resource_Default_view!public!resources-views! !
!SmaCCParserGenerator class categoriesFor: #toolsFolderIcon!public! !
!SmaCCParserGenerator class categoriesFor: #uninitialize!public! !

SmaCCGrammarStyler guid: (GUID fromString: '{3465D494-5421-48D2-8220-97006666A9F3}')!
SmaCCGrammarStyler comment: ''!
!SmaCCGrammarStyler categoriesForClass!Unclassified! !
!SmaCCGrammarStyler methodsFor!

colorText: aString in: aScintillaView startingAt: aSmallInteger 
	| scanner |
	scanner := SmaCCGrammarStylerScanner on: (ReadStream on: aString).
	scanner parseSourceSelector: parseStringSelector.
	scanner for: aScintillaView startingAt: aSmallInteger!

initialize
	super initialize.
	parseStringSelector := SmaCCSmalltalkCodeGenerator parseSourceSelector!

parseStringSelector
	^parseStringSelector!

parseStringSelector: aSymbol 
	parseStringSelector := aSymbol! !
!SmaCCGrammarStyler categoriesFor: #colorText:in:startingAt:!public! !
!SmaCCGrammarStyler categoriesFor: #initialize!initialize/release!public! !
!SmaCCGrammarStyler categoriesFor: #parseStringSelector!accessing!private! !
!SmaCCGrammarStyler categoriesFor: #parseStringSelector:!accessing!private! !

SmaCCGrammarStylerScanner guid: (GUID fromString: '{BDD6D8F7-C064-4865-A20A-088DC4B45F8B}')!
SmaCCGrammarStylerScanner comment: ''!
!SmaCCGrammarStylerScanner categoriesForClass!Unclassified! !
!SmaCCGrammarStylerScanner methodsFor!

comment
	view applyStyle: #comment toNext: lastOutputStreamMatchPosition.
	self resetScanner.
	^self scanForToken!

for: aScintillaView startingAt: aSmallInteger
	view := aScintillaView.
	offset := aSmallInteger.
	insideRule := false.
	afterColon := false.
	self styleText!

initializeFoldingForLines
	stopLine + 1 to: startLine do: [:i | view foldLine: i level: SC_FOLDLEVELBASE].
	stopLine := self lineForToken.
	stopLine > startLine
		ifTrue:
			[view foldLine: startLine level: (SC_FOLDLEVELBASE bitOr: SC_FOLDLEVELHEADERFLAG).
			startLine + 1 to: stopLine do: [:i | view foldLine: i level: SC_FOLDLEVELBASE + 1].
			view foldLine: stopLine level: (SC_FOLDLEVELBASE + 1 bitOr: SC_FOLDLEVELBOXFOOTERFLAG)]
		ifFalse: [view foldLine: startLine level: SC_FOLDLEVELBASE]!

lineForToken
	^view lineFromPosition: token startPosition + offset!

styleForToken
	| style id |
	id := token ids class == Array ifTrue: [token ids first] ifFalse: [token ids].
	style := #normal.
	token value notEmpty ifTrue: [token value first == $" ifTrue: [style := #keyword]].
	id = self classLeftId
		ifTrue:
			[insideClassDefinition := true.
			style := #class].
	id = self classRightId
		ifTrue:
			[insideClassDefinition := false.
			style := #class].
	id = self symbolnameId
		ifTrue:
			[style := insideClassDefinition
				ifTrue: [#class]
				ifFalse: [afterColon ifTrue: [#nonTerminal] ifFalse: [#nonTerminalDefinition]]].
	id = self tokennameId ifTrue: [style := afterColon ifTrue: [#terminal] ifFalse: [#terminalDefinition]].
	id = self codeId ifTrue: [style := #code].
	id = self errorTokenId ifTrue: [style := #error].
	id = self variablenameId ifTrue: [style := #variable].
	state = #scanner ifTrue: [style := #character].
	state = #directives ifTrue: [style := #normal].
	^style!

styleText
	| style |
	insideClassDefinition := false.
	stopLine := view lineFromPosition: offset.
	[self atEnd]
		whileFalse:
			[| id |
			token := self next.
			insideRule := true.
			style := self styleForToken.
			id := token ids class == Array ifTrue: [token ids first] ifFalse: [token ids].
			afterColon ifFalse: [afterColon := token value = ':' or: [self directiveId = id]].
			state = #default
				ifTrue:
					[id = self tokennameId ifTrue: [state := #scanner].
					id = self productionId ifTrue: [state := #parser].
					id = self directiveId ifTrue: [state := #directives].
					startLine isNil ifTrue: [startLine := self lineForToken]]
				ifFalse:
					[id = self semicolonId
						ifTrue:
							[insideRule := false.
							afterColon := false.
							state := #default.
							self initializeFoldingForLines.
							startLine := nil]].
			view applyStyle: style toNext: token value size]!

whitespace
	view
		applyStyle: (insideRule ifTrue: [#internalWhitespace] ifFalse: [#whitespace])
		toNext: lastOutputStreamMatchPosition.
	self resetScanner.
	^self scanForToken! !
!SmaCCGrammarStylerScanner categoriesFor: #comment!public! !
!SmaCCGrammarStylerScanner categoriesFor: #for:startingAt:!public! !
!SmaCCGrammarStylerScanner categoriesFor: #initializeFoldingForLines!public! !
!SmaCCGrammarStylerScanner categoriesFor: #lineForToken!public! !
!SmaCCGrammarStylerScanner categoriesFor: #styleForToken!public! !
!SmaCCGrammarStylerScanner categoriesFor: #styleText!public! !
!SmaCCGrammarStylerScanner categoriesFor: #whitespace!public! !

"Binary Globals"!

