| package |
package := Package name: 'SmaCC GLR Runtime'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains the runtime part of the GLR parser for SmaCC. All GLR parsers created using SmaCC will need this code. 

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #SmaCCAmbiguousResultNotification;
	add: #SmaCCDeferredChoice;
	add: #SmaCCDeferredReduceAction;
	add: #SmaCCGLRParser;
	add: #SmaCCParserState;
	yourself.

package methodNames
	add: #Object -> #performOnSmaCCParser:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\SmaCC Runtime\SmaCC Runtime';
	yourself).

package!

"Class Definitions"!

Object subclass: #SmaCCDeferredChoice
	instanceVariableNames: 'possibleItems choiceBlock'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCDeferredReduceAction
	instanceVariableNames: 'selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCParserState
	instanceVariableNames: 'stateStack nodeStack isAccepted position scannerState'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Notification subclass: #SmaCCAmbiguousResultNotification
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCParser subclass: #SmaCCGLRParser
	instanceVariableNames: 'states parseAll currentState tryAllStates lastToken lastPosition lastState nextScannerState nextScannerPosition'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'cachedTrees'!

"Global Aliases"!


"Loose Methods"!

!Object methodsFor!

performOnSmaCCParser: aSmaCCParser
	^self! !
!Object categoriesFor: #performOnSmaCCParser:!private!SmaCC! !

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCDeferredChoice guid: (GUID fromString: '{06EB08E9-1DE2-443D-8161-AC868CF66370}')!
SmaCCDeferredChoice comment: 'SmaCCDeferredChoice represents a choice between multiple parses.

Instance Variables
	choiceBlock	<BlockClosure>	a two argument block that decides what choice should be chosen
	possibleItems	<SequenceableCollection of: Object>	the choices

'!
!SmaCCDeferredChoice categoriesForClass!Unclassified! !
!SmaCCDeferredChoice methodsFor!

choiceBlock: aBlock
	choiceBlock := aBlock!

performOnSmaCCParser: aSmaCCParser
	^(choiceBlock value: possibleItems value: aSmaCCParser) performOnSmaCCParser: aSmaCCParser!

possibleItems: aSequenceableCollection
	possibleItems := aSequenceableCollection! !
!SmaCCDeferredChoice categoriesFor: #choiceBlock:!accessing!public! !
!SmaCCDeferredChoice categoriesFor: #performOnSmaCCParser:!accessing!private! !
!SmaCCDeferredChoice categoriesFor: #possibleItems:!accessing!public! !

!SmaCCDeferredChoice class methodsFor!

mergeStacks: stacksOrderedCollection
	| result choiceBlock choices |
	choices := 1 to: stacksOrderedCollection size.
	choiceBlock := [:possibleChoices :parser | 
	| returnValue newChoices |
	choices size == 1
		ifTrue: [possibleChoices at: choices first]
		ifFalse:
			[returnValue := (SmaCCAmbiguousResultNotification forParser: parser)
				signal: 'Ambiguous results'
				with: (choices collect: [:each | possibleChoices at: each]).
			newChoices := choices select: [:each | (possibleChoices at: each) == returnValue].
			newChoices notEmpty ifTrue: [choices := newChoices].
			returnValue]].
	result := OrderedCollection new.
	1
		to: stacksOrderedCollection first size
		do:
			[:i | 
			| object |
			object := stacksOrderedCollection first at: i.
			(stacksOrderedCollection allSatisfy: [:each | object == (each at: i)])
				ifTrue: [result add: object]
				ifFalse:
					[result
						add:
							((self new)
								possibleItems: (stacksOrderedCollection collect: [:each | each at: i]);
								choiceBlock: choiceBlock;
								yourself)]].
	^result! !
!SmaCCDeferredChoice class categoriesFor: #mergeStacks:!accessing!private! !

SmaCCDeferredReduceAction guid: (GUID fromString: '{6B8F2D94-AC74-4750-8E1E-2316836F36C9}')!
SmaCCDeferredReduceAction comment: 'SmaCCDeferredReduceAction represents a deferred message that will be sent once we figure out what we are parsing.

Instance Variables
	arguments	<Array of: Object>	the arguments of the message
	selector	<Symbol>	the selector to send'!
!SmaCCDeferredReduceAction categoriesForClass!Unclassified! !
!SmaCCDeferredReduceAction methodsFor!

= anObject
	^self class = anObject class and: [self selector = anObject selector and: [self arguments = anObject arguments]]!

arguments
	^arguments!

argumentsFor: aSmaCCParser
	1 to: arguments size do: [:i | arguments at: i put: ((arguments at: i) performOnSmaCCParser: aSmaCCParser)].
	^arguments!

hash
	^self class hash bitXor: (selector hash bitXor: arguments hash)!

performOnSmaCCParser: aSmaCCParser
	^selector argumentCount = 0
		ifTrue: [aSmaCCParser perform: selector]
		ifFalse: [aSmaCCParser perform: selector with: (self argumentsFor: aSmaCCParser)]!

selector
	^selector!

selector: aSymbol arguments: aCollection
	selector := aSymbol.
	arguments := aCollection! !
!SmaCCDeferredReduceAction categoriesFor: #=!comparing!private! !
!SmaCCDeferredReduceAction categoriesFor: #arguments!accessing!private! !
!SmaCCDeferredReduceAction categoriesFor: #argumentsFor:!initialize-release!private! !
!SmaCCDeferredReduceAction categoriesFor: #hash!comparing!private! !
!SmaCCDeferredReduceAction categoriesFor: #performOnSmaCCParser:!accessing!private! !
!SmaCCDeferredReduceAction categoriesFor: #selector!accessing!private! !
!SmaCCDeferredReduceAction categoriesFor: #selector:arguments:!initialize-release!private! !

!SmaCCDeferredReduceAction class methodsFor!

selector: aSymbol arguments: aCollection
	^(self new)
		selector: aSymbol arguments: aCollection;
		yourself! !
!SmaCCDeferredReduceAction class categoriesFor: #selector:arguments:!instance creation!private! !

SmaCCParserState guid: (GUID fromString: '{02539718-224A-409A-991D-8310EF33E363}')!
SmaCCParserState comment: 'SmaCCParserState represents a particular parse in a GLR parser.

Instance Variables:
	isAccepted	<Boolean>	has this parse been accepted
	nodeStack	<SequenceableCollection of: Object>	the stack of objects for this parse
	position	<Integer>	the current position of the scanner
	scannerState	<Symbol> the current state of the scanner
	stateStack	<SequenceableCollection of: Integer>	the stack of states for this parse

'!
!SmaCCParserState categoriesForClass!Unclassified! !
!SmaCCParserState methodsFor!

= anObject
	^self == anObject or: [self class = anObject class and: [self stateStack = anObject stateStack and: [self nodeStack = anObject nodeStack]]]!

canBeMergedWith: aSmaCCParserState
	^self stateStack = aSmaCCParserState stateStack
		and: [self position = aSmaCCParserState position and: [self scannerState = aSmaCCParserState scannerState]]!

hash
	^self stateStack hash!

isAccepted
	^isAccepted!

markAccepted
	isAccepted := true!

mergeWith: aSmaCCParserStateCollection
	| stacks |
	stacks := OrderedCollection new: aSmaCCParserStateCollection size + 1.
	stacks add: nodeStack.
	aSmaCCParserStateCollection do: [:each | (stacks includes: each nodeStack) ifFalse: [stacks add: each nodeStack]].
	nodeStack := SmaCCDeferredChoice mergeStacks: stacks!

nodeStack
	^nodeStack!

position
	^position!

position: anObject
	position := anObject!

runDeferredActionsFor: aSmaCCParser
	1
		to: nodeStack size
		do:
			[:i | 
			| each |
			each := nodeStack at: i.
			nodeStack at: i put: (each performOnSmaCCParser: aSmaCCParser)]!

scannerState
	^scannerState!

scannerState: anObject
	scannerState := anObject!

stateStack
	^stateStack!

stateStack: stateCollection nodeStack: nodeCollection
	stateStack := stateCollection.
	nodeStack := nodeCollection.
	isAccepted := false! !
!SmaCCParserState categoriesFor: #=!comparing!public! !
!SmaCCParserState categoriesFor: #canBeMergedWith:!private!testing! !
!SmaCCParserState categoriesFor: #hash!comparing!private! !
!SmaCCParserState categoriesFor: #isAccepted!private!testing! !
!SmaCCParserState categoriesFor: #markAccepted!accessing!private! !
!SmaCCParserState categoriesFor: #mergeWith:!accessing!private! !
!SmaCCParserState categoriesFor: #nodeStack!accessing!private! !
!SmaCCParserState categoriesFor: #position!accessing!private! !
!SmaCCParserState categoriesFor: #position:!accessing!private! !
!SmaCCParserState categoriesFor: #runDeferredActionsFor:!accessing!private! !
!SmaCCParserState categoriesFor: #scannerState!accessing!private! !
!SmaCCParserState categoriesFor: #scannerState:!accessing!private! !
!SmaCCParserState categoriesFor: #stateStack!accessing!private! !
!SmaCCParserState categoriesFor: #stateStack:nodeStack:!initialize-release!private! !

!SmaCCParserState class methodsFor!

stateStack: stateStack nodeStack: nodeStack
	^(self new)
		stateStack: stateStack nodeStack: nodeStack;
		yourself!

stateStack: stateStack nodeStack: nodeStack scannerState: aSymbol position: anInteger
	^(self new)
		stateStack: stateStack nodeStack: nodeStack;
		scannerState: aSymbol;
		position: anInteger;
		yourself! !
!SmaCCParserState class categoriesFor: #stateStack:nodeStack:!instance creation!private! !
!SmaCCParserState class categoriesFor: #stateStack:nodeStack:scannerState:position:!instance creation!private! !

SmaCCAmbiguousResultNotification guid: (GUID fromString: '{1C3B7A32-BB02-4274-AFAC-4ACF3971FCCA}')!
SmaCCAmbiguousResultNotification comment: 'SmaCCAmbiguousResultNotification is a notification that is signaled when the GLR parser accepts multiple parses. The user can catch this signal and resume it with the correct parse. The parameters of the notification are the potential parses.

Instance Variables:
	parser	<SmaCCGLRParser>	the parser that parsed the ambiguous results

'!
!SmaCCAmbiguousResultNotification categoriesForClass!Unclassified! !
!SmaCCAmbiguousResultNotification methodsFor!

defaultAction
	^self tag first!

parser
	^parser!

parser: anObject
	parser := anObject! !
!SmaCCAmbiguousResultNotification categoriesFor: #defaultAction!private!private - actions! !
!SmaCCAmbiguousResultNotification categoriesFor: #parser!accessing!private! !
!SmaCCAmbiguousResultNotification categoriesFor: #parser:!accessing!private! !

!SmaCCAmbiguousResultNotification class methodsFor!

forParser: aSmaCCParser
	^(self new)
		parser: aSmaCCParser;
		yourself! !
!SmaCCAmbiguousResultNotification class categoriesFor: #forParser:!instance creation!private! !

SmaCCGLRParser guid: (GUID fromString: '{DEE9A03F-AA79-4ED2-895E-82181F08AD1C}')!
SmaCCGLRParser comment: 'SmaCCGLRParser is an abstract superclass for all GLR generated parsers in SmaCC.

Instance Variables:
	currentState	<SmaCCParserState>	the current parse that we are trying
	lastPosition	<Integer>	the starting location of the scanner before calling getNextToken
	lastState	<Symbol>	the state of the scanner before calling getNextToken
	lastToken	<SmaCCToken>	the token returned from getNextToken (if the lastState and lastToken are the same as the current token, then this value is returned without scanning anything)
	nextScannerPosition	<Integer>	the ending location of the scanner after calling getNextToken
	nextScannerState	<Symbol>	the ending state of the scanner after calling getNextToken
	parseAll	<Boolean>	should we return a collection of all potential parses or just one
	states	<SequenceableCollection of: SmaCCParserState>	the current list of valid parses
	tryAllStates	<Boolean>	should we try to parse starting from any state instead of the starting state
'!
!SmaCCGLRParser categoriesForClass!Unclassified! !
!SmaCCGLRParser methodsFor!

actionsForCurrentToken
	| actions |
	actions := OrderedCollection new: 1.
	self tryAllTokens
		ifTrue: [currentToken ids do: [:each | self addActionsFor: each to: actions]]
		ifFalse: [self addActionsFor: currentToken ids first to: actions].
	^actions!

addAction: action to: anOrderedCollection
	(anOrderedCollection includes: action) ifTrue: [^self].
	anOrderedCollection add: action!

addActionsFor: anInteger to: anOrderedCollection
	| state |
	state := self currentState.
	self
		addActionsForState: state
		symbol: anInteger
		to: anOrderedCollection!

addActionsForState: state symbol: anInteger to: anOrderedCollection
	| action |
	action := self actionForState: state and: anInteger.
	action = self errorAction ifTrue: [^self].
	(action bitAnd: self actionMask) = 2r11
		ifTrue: [self ambiguousTransitionsAt: (action bitShift: -2) into: anOrderedCollection]
		ifFalse: [self addAction: action to: anOrderedCollection]!

alwaysPerformReduceAction
	^false!

ambiguousTransitions
	^#()!

ambiguousTransitionsAt: anIndex into: anOrderedCollection
	| ambiguousTransitions |
	ambiguousTransitions := self ambiguousTransitions at: anIndex.
	1
		to: ambiguousTransitions size
		by: 2
		do:
			[:i | self addAction: ((ambiguousTransitions at: i) bitShift: 8) + (ambiguousTransitions at: i + 1) to: anOrderedCollection]!

copyStackObject: anObject
	(anObject isKindOf: String) ifTrue: [^anObject].
	(anObject isKindOf: Collection) ifTrue: [^anObject collect: [:each | self copyStackObject: each]].
	^anObject!

duplicateState
	| state |
	parseAll ifTrue: [nodeStack := nodeStack collect: [:each | each copy]] ifFalse: [nodeStack := nodeStack copy].
	stateStack := stateStack copy.
	state := currentState copy.
	state stateStack: stateStack nodeStack: nodeStack.
	state position: scanner position.
	state scannerState: scanner state.
	states add: state.
	^state!

fixObject: anObject
	| result |
	result := anObject performOnSmaCCParser: self.
	(result isKindOf: SmaCCParseNode) ifTrue: [result fixParentPointers].
	^result!

getNextToken
	currentToken isNil ifFalse: [^self].
	(lastState = currentState scannerState and: [lastPosition = currentState position])
		ifTrue:
			[currentToken := lastToken.
			scanner
				setPosition: nextScannerPosition;
				state: nextScannerState.
			currentState
				scannerState: nextScannerState;
				position: nextScannerPosition]
		ifFalse:
			[lastPosition := currentState position.
			lastState := currentState scannerState.
			scanner
				state: currentState scannerState;
				setPosition: currentState position.
			lastToken := currentToken := scanner next.
			nextScannerPosition := scanner position.
			nextScannerState := scanner state].
	currentState
		scannerState: nextScannerState;
		position: nextScannerPosition!

initialize
	super initialize.
	states := OrderedCollection new.
	parseAll := false.
	tryAllStates := false!

killState
	states remove: currentState.
	states isEmpty ifTrue: [self handleError: 0].
	(states size = 1 and: [parseAll not]) ifTrue: [states first runDeferredActionsFor: self]!

mergeStates
	| i j state otherStates otherState |
	i := 1.
	[i < states size]
		whileTrue:
			[state := states at: i.
			otherStates := OrderedCollection new.
			j := i := i + 1.
			[j <= states size]
				whileTrue:
					[otherState := states at: j.
					(parseAll ifTrue: [state = otherState] ifFalse: [state canBeMergedWith: otherState])
						ifTrue:
							[otherStates add: otherState.
							states removeAtIndex: j]
						ifFalse: [j := j + 1]].
			(parseAll not and: [otherStates notEmpty]) ifTrue: [state mergeWith: otherStates]]!

parseAll: aBoolean
	parseAll := aBoolean!

performParsingLoop
	| result |
	self setStartingStatesIfNone.
	[states size
		to: 1
		by: -1
		do:
			[:i | 
			self restoreState: (states at: i).
			self getNextToken.
			errorToken := nil.
			self performParsingStep].
	self mergeStates.
	states anySatisfy: [:each | each isAccepted not]] whileTrue.
	self checkForErrors.
	result := parseAll
		ifTrue: [states collect: [:each | self fixObject: each nodeStack first]]
		ifFalse: [self fixObject: states first nodeStack first].
	nodeStack := OrderedCollection with: result!

performParsingStep
	| actions action |
	currentState isAccepted ifTrue: [^self].
	self restoreState: currentState.
	(tryAllStates and: [self isEOFToken]) ifTrue: [^self processEOFToken].
	currentToken isPattern ifTrue: [^self splitForPatternToken].
	[actions := self actionsForCurrentToken.
	actions isEmpty
		ifTrue:
			[self killState.
			currentToken := nil.
			^self].
	2 to: actions size do: [:i | self splitWithAction: (actions at: i)].
	action := actions first.
	action = self acceptAction
		or:
			[self performAction: action.
			currentToken isNil]] whileFalse.
	action = self acceptAction ifTrue: [currentState markAccepted]!

performReduceMethod: aSymbol with: items
	^(states size = 1 or: [parseAll])
		ifTrue:
			[1 to: items size do: [:i | items at: i put: ((items at: i) performOnSmaCCParser: self)].
			super performReduceMethod: aSymbol with: items]
		ifFalse: [SmaCCDeferredReduceAction selector: aSymbol arguments: items]!

popNodeStack
	| result |
	result := nodeStack removeLast.
	(self alwaysPerformReduceAction and: [states size > 1]) ifTrue: [result := self copyStackObject: result].
	^result!

position: anInteger
	super position: anInteger.
	currentState position: anInteger!

processEOFToken
	| performedActions token |
	performedActions := Set new.
	token := currentToken.
	self symbolTypes
		keysAndValuesDo:
			[:i :each | 
			| actions |
			(each = #SmaCCToken and: [i ~= scanner errorTokenId])
				ifTrue:
					[actions := OrderedCollection new.
					self addActionsFor: i to: actions.
					actions
						do:
							[:action | 
							(performedActions includes: action)
								ifFalse:
									[performedActions add: action.
									(action bitAnd: self actionMask) = self reduceAction
										ifTrue:
											[currentToken := token.
											self splitWithAction: action]]]]].
	(nodeStack size = 1 and: [nodeStack first isKindOf: SmaCCParseNode])
		ifTrue: [currentState markAccepted]
		ifFalse: [self killState]!

reduce: anInteger
	| reduceEntry items size |
	reduceEntry := self reduceTable at: anInteger.
	items := Array new: (size := reduceEntry at: 2).
	nodeStack size < size
		ifTrue:
			[self killState.
			^currentToken := nil].
	size
		to: 1
		by: -1
		do:
			[:i | 
			items at: i put: self popNodeStack.
			stateStack removeLast].
	nodeStack addLast: (self performReduceMethod: (reduceEntry at: 3) with: items).
	stateStack addLast: ((self actionFor: (reduceEntry at: 1)) bitShift: -2)!

restoreState: aSmaCCParserState
	currentState := aSmaCCParserState.
	nodeStack := aSmaCCParserState nodeStack.
	stateStack := aSmaCCParserState stateStack!

setAllStartingStates
	tryAllStates := true.
	self statesThatShift
		do:
			[:each | 
			states
				add:
					(SmaCCParserState
						stateStack: (OrderedCollection with: each)
						nodeStack: OrderedCollection new
						scannerState: scanner state
						position: scanner position)]!

setStartingStatesIfNone
	states isEmpty
		ifTrue:
			[states
				add:
					(SmaCCParserState
						stateStack: stateStack
						nodeStack: nodeStack
						scannerState: scanner state
						position: scanner position)]!

splitForPatternToken
	| token patternNode performedActions nodeClass |
	performedActions := Set new.
	token := currentToken copy.
	token ids: OrderedCollection new.
	patternNode := SmaCCPatternNode onToken: token copy.
	nodeClass := token nodeClassName
		ifNil: [SmaCCParseNode]
		ifNotNil: [:name | self class environment at: name ifAbsent: [SmaCCParseNode]].
	self symbolTypes
		keysAndValuesDo:
			[:i :each | 
			| actions |
			(i ~= scanner emptySymbolTokenId and: [i ~= scanner errorTokenId])
				ifTrue:
					[actions := OrderedCollection new.
					(each = #SmaCCToken
						or: [each notNil and: [(self class environment at: each ifAbsent: [Object]) includesBehavior: nodeClass]])
						ifTrue:
							[self addActionsFor: i to: actions.
							actions
								do:
									[:action | 
									(performedActions includes: action)
										ifFalse:
											[performedActions add: action.
											each = #SmaCCToken
												ifTrue:
													[token ids add: i.
													((action bitAnd: self actionMask) = self reduceAction or: [token isToken])
														ifTrue:
															[currentToken := token.
															self splitWithAction: action]]
												ifFalse:
													[token isNode
														ifTrue:
															[currentToken := nil.
															self duplicateState.
															stateStack addLast: (action bitShift: -2).
															nodeStack addLast: patternNode.
															self restoreState: currentState]]]]]]].
	self killState.
	currentToken := nil!

splitWithAction: anInteger
	| previousToken previousState |
	previousState := currentState.
	previousToken := currentToken.
	[currentState := self duplicateState.
	self performAction: anInteger.
	currentToken notNil ifTrue: [self performParsingStep]]
		ensure:
			[self restoreState: previousState.
			currentToken := previousToken]!

state: aSymbol
	super state: aSymbol.
	currentState scannerState: aSymbol!

statesThatShift
	^(1 to: self transitionTable size)
		select:
			[:each | 
			(1 to: self symbolTypes size)
				anySatisfy:
					[:i | 
					| actions |
					actions := OrderedCollection new.
					self
						addActionsForState: each
						symbol: i
						to: actions.
					actions anySatisfy: [:action | (action bitAnd: self actionMask) = self shiftAction]]]! !
!SmaCCGLRParser categoriesFor: #actionsForCurrentToken!private! !
!SmaCCGLRParser categoriesFor: #addAction:to:!private! !
!SmaCCGLRParser categoriesFor: #addActionsFor:to:!private! !
!SmaCCGLRParser categoriesFor: #addActionsForState:symbol:to:!private! !
!SmaCCGLRParser categoriesFor: #alwaysPerformReduceAction!private! !
!SmaCCGLRParser categoriesFor: #ambiguousTransitions!private! !
!SmaCCGLRParser categoriesFor: #ambiguousTransitionsAt:into:!private! !
!SmaCCGLRParser categoriesFor: #copyStackObject:!private! !
!SmaCCGLRParser categoriesFor: #duplicateState!private! !
!SmaCCGLRParser categoriesFor: #fixObject:!private! !
!SmaCCGLRParser categoriesFor: #getNextToken!public! !
!SmaCCGLRParser categoriesFor: #initialize!initialize-release!private! !
!SmaCCGLRParser categoriesFor: #killState!private! !
!SmaCCGLRParser categoriesFor: #mergeStates!private! !
!SmaCCGLRParser categoriesFor: #parseAll:!accessing!private! !
!SmaCCGLRParser categoriesFor: #performParsingLoop!private! !
!SmaCCGLRParser categoriesFor: #performParsingStep!private! !
!SmaCCGLRParser categoriesFor: #performReduceMethod:with:!private! !
!SmaCCGLRParser categoriesFor: #popNodeStack!private! !
!SmaCCGLRParser categoriesFor: #position:!private! !
!SmaCCGLRParser categoriesFor: #processEOFToken!private! !
!SmaCCGLRParser categoriesFor: #reduce:!private! !
!SmaCCGLRParser categoriesFor: #restoreState:!private! !
!SmaCCGLRParser categoriesFor: #setAllStartingStates!accessing!private! !
!SmaCCGLRParser categoriesFor: #setStartingStatesIfNone!private! !
!SmaCCGLRParser categoriesFor: #splitForPatternToken!private! !
!SmaCCGLRParser categoriesFor: #splitWithAction:!private! !
!SmaCCGLRParser categoriesFor: #state:!private! !
!SmaCCGLRParser categoriesFor: #statesThatShift!private! !

!SmaCCGLRParser class methodsFor!

parseAll: aString startingAt: anInteger
	| parser |
	parser := self on: (ReadStream on: aString).
	parser setStartingState: anInteger.
	parser parseAll: true.
	^parser parse!

parseAllPossibleStates: aString
	| parser |
	parser := self on: (ReadStream on: aString).
	parser
		parseAll: true;
		setAllStartingStates.
	^parser parse!

parseAndCacheAll: aString startingAt: anInteger
	cachedTrees isNil ifTrue: [cachedTrees := Dictionary new].
	^cachedTrees at: (Array with: aString with: anInteger) ifAbsentPut: [self parseAll: aString startingAt: anInteger]!

resetCaches
	super resetCaches.
	cachedTrees := nil! !
!SmaCCGLRParser class categoriesFor: #parseAll:startingAt:!parsing!private! !
!SmaCCGLRParser class categoriesFor: #parseAllPossibleStates:!parsing!private! !
!SmaCCGLRParser class categoriesFor: #parseAndCacheAll:startingAt:!parsing!private! !
!SmaCCGLRParser class categoriesFor: #resetCaches!accessing!public! !

"Binary Globals"!

