| package |
package := Package name: 'SmaCC Rewrite Server Worker'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains workers that the server use to distribute work across several different workers/images. Once the server has been started a worker can be started by evaluating:
	SmaCCRewriteServerWorker new startConnectingTo: ''localhost'' onPort: 36rSM

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.'.

package imageStripperBytes: (ByteArray fromBase64String: 'IVNUQiAzIEYPDQAEAAAASW1hZ2VTdHJpcHBlcgAAAABSAAAAGwAAAFNtYUNDIFJld3JpdGUgU2Vy
dmVyIFdvcmtlclIAAAApAAAAWDpcUHJvZ3JhbXNcU21hQ0NUcmFuc2Zvcm1hdGlvbldvcmtlci5l
eGWaAAAAUgAAABsAAABTbWFDQyBSZXdyaXRlIFNlcnZlciBXb3JrZXJSAAAAJgAAAFNtYUNDUmV3
cml0ZVNlcnZlcldvcmtlclNlc3Npb25NYW5hZ2VyDzUEAQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAA=').

package classNames
	add: #SmaCCRewriteServerWorker;
	add: #SmaCCRewriteServerWorkerSessionManager;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	add: '..\..\..\Object Arts\Dolphin\Sockets\Dolphin Sockets';
	add: '..\..\..\Object Arts\Dolphin\Lagoon\Lagoon Image Stripper';
	add: '..\SmaCC CSharp\SmaCC CSharp';
	add: '..\SmaCC Delphi Forms\SmaCC Delphi Forms';
	add: '..\SmaCC Delphi Symbol Table\SmaCC Delphi Symbol Table';
	add: '..\SmaCC EGL Parser\SmaCC EGL Parser';
	add: '..\SmaCC Hashing\SmaCC Hashing';
	add: '..\SmaCC Java\SmaCC Java';
	add: '..\SmaCC Javascript Parser\SmaCC Javascript Parser';
	add: '..\SmaCC Powerbuilder Symbol Table\SmaCC Powerbuilder Symbol Table';
	add: '..\SmaCC Rewrite Engine\SmaCC Rewrite Engine';
	add: '..\SmaCC Server Session\SmaCC Server Session';
	yourself).

package setManualPrerequisites: #(
	'SmaCC CSharp'
	'SmaCC Delphi Forms'
	'SmaCC Delphi Symbol Table'
	'SmaCC EGL Parser'
	'SmaCC Hashing'
	'SmaCC Java'
	'SmaCC Javascript Parser'
	'SmaCC Powerbuilder Symbol Table').

package!

"Class Definitions"!

Object subclass: #SmaCCRewriteServerWorker
	instanceVariableNames: 'socket rewriteFiles remotePrefix localPrefix'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCServerSessionManager subclass: #SmaCCRewriteServerWorkerSessionManager
	instanceVariableNames: 'server port'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCRewriteServerWorker guid: (GUID fromString: '{4195AFE3-FEE4-43B4-8970-9632D4E73AC5}')!
SmaCCRewriteServerWorker comment: ''!
!SmaCCRewriteServerWorker categoriesForClass!Kernel-Objects! !
!SmaCCRewriteServerWorker methodsFor!

classNamed: aString
	^Smalltalk at: aString asSymbol!

loadRewrites: aString
	[| rewrites |
	rewrites := rewriteFiles at: aString ifAbsentPut: [SmaCCRewriteRuleFileParser parseFile: aString].
	rewrites isUpToDate ifTrue: [^rewrites].
	rewriteFiles removeKey: aString] repeat!

localizeFilename: aString
	| newString |
	newString := (self remotePrefix notEmpty and: [aString beginsWith: self remotePrefix])
		ifTrue: [self localPrefix , (aString copyFrom: self remotePrefix size + 1)]
		ifFalse: [aString].
	^(newString copyReplacing: $\ withObject: File pathDelimiter) copyReplacing: $/ withObject: File pathDelimiter!

localPrefix
	^localPrefix ifNil: ['']!

localPrefix: aString
	localPrefix := aString!

processRequest: aString
	| file outputDirectory rewrites inputParser outputParser result resultBytes baseDirectory |
	aString
		splitFirstOn: $;
		andThen: $=
		do:
			[:type :value | 
			type = 'FILE' ifTrue: [file := self localizeFilename: value].
			type = 'OUTPUTDIRECTORY' ifTrue: [outputDirectory := self localizeFilename: value].
			type = 'BASEDIRECTORY' ifTrue: [baseDirectory := self localizeFilename: value].
			type = 'REWRITEFILE' ifTrue: [rewrites := self loadRewrites: (self localizeFilename: value)].
			type = 'INPUTPARSER' ifTrue: [inputParser := self classNamed: value].
			type = 'OUTPUTPARSER' ifTrue: [outputParser := self classNamed: value]].
	result := SmaCCRewriteResult new.
	(SmaCCRewriteProcessor new)
		rules: rewrites;
		parserClass: inputParser;
		validationParserClass: outputParser;
		outputDirectory: outputDirectory;
		baseDirectory: baseDirectory;
		processFile: file usingResult: result.
	resultBytes := result asSocketString asByteArray.
	self writeBytes: resultBytes!

processRequests
	[[| bytes |
	bytes := self readBytes.
	self processRequest: bytes asString] repeat]
		on: Error
		do:
			[:ex | 
			SessionManager current cmdLineFlags do: [:each | (each sameAs: 'error') ifTrue: [ex pass]].
			SessionManager current quit]!

readBytes
	^socket readStream next: socket readStream nextDWORD!

remotePrefix
	^remotePrefix ifNil: ['']!

remotePrefix: aString
	remotePrefix := aString!

startConnectingTo: aString onPort: anInteger
	rewriteFiles := Dictionary new.
	socket := [Socket2 port: anInteger host: aString] on: Error do: [:ex | SessionManager current quit].
	socket connect.
	[(socket writeStream)
		nextPut: $0 asInteger;
		flush.
	self processRequests] ensure: [socket close]!

writeBytes: resultBytes
	(socket writeStream)
		nextDWORDPut: resultBytes size;
		nextPutAll: resultBytes;
		flush! !
!SmaCCRewriteServerWorker categoriesFor: #classNamed:!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #loadRewrites:!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #localizeFilename:!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #localPrefix!accessing!private! !
!SmaCCRewriteServerWorker categoriesFor: #localPrefix:!accessing!private! !
!SmaCCRewriteServerWorker categoriesFor: #processRequest:!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #processRequests!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #readBytes!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #remotePrefix!accessing!private! !
!SmaCCRewriteServerWorker categoriesFor: #remotePrefix:!accessing!private! !
!SmaCCRewriteServerWorker categoriesFor: #startConnectingTo:onPort:!public!running! !
!SmaCCRewriteServerWorker categoriesFor: #writeBytes:!public!running! !

SmaCCRewriteServerWorkerSessionManager guid: (GUID fromString: '{6D715D85-02F4-4EE3-B127-4312E93CF9BE}')!
SmaCCRewriteServerWorkerSessionManager comment: ''!
!SmaCCRewriteServerWorkerSessionManager categoriesForClass!Kernel-Objects! !
!SmaCCRewriteServerWorkerSessionManager methodsFor!

notifyIcon
	^Icon fromId: 'ClassBrowserShell.ico'!

notifyMessage
	^'Connected to ' , self server , ':' , self port printString!

port
	^port
		ifNil:
			[| arg parts |
			arg := 'localhost'.
			self arguments notEmpty ifTrue: [arg := self arguments last].
			parts := (arg subStrings: $:) last.
			port := (parts size = 2 and: [parts last notEmpty and: [parts last allSatisfy: [:each | each isDigit]]])
				ifTrue: [Number fromString: parts last]
				ifFalse: [36rSM]]!

server
	^server
		ifNil:
			[| arg |
			arg := 'localhost'.
			self arguments notEmpty ifTrue: [arg := self arguments last].
			server := (arg subStrings: $:) first]!

startServer
	[| worker |
	worker := SmaCCRewriteServerWorker new.
	self cmdLineFlags
		do:
			[:each | 
			(each beginsWith: 'remote=') ifTrue: [worker remotePrefix: (each copyFrom: 8)].
			(each beginsWith: 'local=') ifTrue: [worker localPrefix: (each copyFrom: 7)]].
	worker startConnectingTo: self server onPort: self port] on: Error do: [:ex | self logError: ex]! !
!SmaCCRewriteServerWorkerSessionManager categoriesFor: #notifyIcon!public! !
!SmaCCRewriteServerWorkerSessionManager categoriesFor: #notifyMessage!public! !
!SmaCCRewriteServerWorkerSessionManager categoriesFor: #port!public! !
!SmaCCRewriteServerWorkerSessionManager categoriesFor: #server!public! !
!SmaCCRewriteServerWorkerSessionManager categoriesFor: #startServer!public! !

"Binary Globals"!

