| package |
package := Package name: 'SmaCC Rewrite Server'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains server that can be used to run the rewrite engine in multiple processes. A server can be started by evaluating:
	SmaCCServer startOn: self port

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.'.

package imageStripperBytes: (ByteArray fromBase64String: 'IVNUQiAzIEYPDQAEAAAASW1hZ2VTdHJpcHBlcgAAAABSAAAAFAAAAFNtYUNDIFJld3JpdGUgU2Vy
dmVyUgAAABsAAABYOlxQcm9ncmFtc1xTbWFDQ1NlcnZlci5leGWaAAAAUgAAABQAAABTbWFDQyBS
ZXdyaXRlIFNlcnZlclIAAAAgAAAAU21hQ0NSZXdyaXRlU2VydmVyU2Vzc2lvbk1hbmFnZXLvtwUA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==').

package classNames
	add: #SmaCCRewriteServerSessionManager;
	add: #SmaCCServer;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	add: '..\..\..\Object Arts\Dolphin\Sockets\Dolphin Sockets';
	add: '..\..\..\Object Arts\Dolphin\MVP\Models\Value\Dolphin Value Models';
	add: '..\..\..\Object Arts\Dolphin\Lagoon\Lagoon Image Stripper';
	add: '..\SmaCC Server Session\SmaCC Server Session';
	yourself).

package!

"Class Definitions"!

Object subclass: #SmaCCServer
	instanceVariableNames: 'socket availableServers registeredServers'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCServerSessionManager subclass: #SmaCCRewriteServerSessionManager
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCServer guid: (GUID fromString: '{E1BC4860-6F1D-4FF1-BF8D-B97056549E36}')!
SmaCCServer comment: ''!
!SmaCCServer categoriesForClass!Kernel-Objects! !
!SmaCCServer methodsFor!

process: aSocket 
	| type |
	type := aSocket readStream next.
	type = $0 asInteger ifTrue: [^self register: aSocket].
	type = $1 asInteger ifTrue: [^self processRequest: aSocket].
	self error: 'Invalid request'!

processRequest: aSocket
	[[| string items canceled results |
	string := (self readBytesFrom: aSocket) asString.
	items := string subStrings: (Character value: 0).
	canceled := ValueHolder with: false.
	results := SharedQueue new.
	[items
		do:
			[:each | 
			canceled value
				ifFalse:
					[| server |
					server := availableServers next.
					[self
						processRequest: each
						on: server
						notifying: results
						canceled: canceled] fork]]] fork.
	[items size
		timesRepeat:
			[| bytes |
			bytes := results next.
			self writeBytes: bytes on: aSocket]] on: Error do: [:ex | canceled value: true]] ensure: [aSocket close]] fork!

processRequest: aByteArray on: aSocket notifying: aSharedQueue canceled: aValueHolder
	| resultBytes |
	[self writeBytes: aByteArray on: aSocket.
	resultBytes := self readBytesFrom: aSocket.
	availableServers nextPut: aSocket]
		on: Error
		do:
			[:ex | 
			registeredServers remove: aSocket.
			[aSocket close] on: Error do: [:e | ].
			aValueHolder value ifFalse: [^self].
			^self
				processRequest: aByteArray
				on: availableServers next
				notifying: aSharedQueue
				canceled: aValueHolder].
	aSharedQueue nextPut: resultBytes!

readBytesFrom: aSocket
	^aSocket readStream next: aSocket readStream nextDWORD!

register: aSocket
	availableServers nextPut: aSocket.
	registeredServers add: aSocket!

run
	[self process: socket accept] repeat!

startOn: anInteger
	availableServers := SharedQueue new.
	registeredServers := SharedIdentitySet new.
	socket := ServerSocket2 port: anInteger.
	[self run] ensure: [socket close]!

writeBytes: bytes on: aSocket
	(aSocket writeStream)
		nextDWORDPut: bytes size;
		nextPutAll: bytes;
		flush! !
!SmaCCServer categoriesFor: #process:!public!running! !
!SmaCCServer categoriesFor: #processRequest:!public!running! !
!SmaCCServer categoriesFor: #processRequest:on:notifying:canceled:!public!running! !
!SmaCCServer categoriesFor: #readBytesFrom:!public!running! !
!SmaCCServer categoriesFor: #register:!public!running! !
!SmaCCServer categoriesFor: #run!public!running! !
!SmaCCServer categoriesFor: #startOn:!public!running! !
!SmaCCServer categoriesFor: #writeBytes:on:!public!running! !

!SmaCCServer class methodsFor!

startOn: anInteger
	^(self new)
		startOn: anInteger;
		yourself! !
!SmaCCServer class categoriesFor: #startOn:!instance creation!public! !

SmaCCRewriteServerSessionManager guid: (GUID fromString: '{784BB0E6-0313-4ABC-AF8B-CD3F02FD0685}')!
SmaCCRewriteServerSessionManager comment: ''!
!SmaCCRewriteServerSessionManager categoriesForClass!Kernel-Objects! !
!SmaCCRewriteServerSessionManager methodsFor!

allowDuplicate
	^false!

notifyIcon
	^Icon fromId: 'ClassHierarchyDiagram.ico'!

notifyMessage
	^'Started on port ' , self port printString!

port
	port isNil
		ifTrue:
			[port := 36rSM.
			self arguments notEmpty ifTrue: [[port := Number fromString: self arguments last] on: Error do: [:ex | ]]].
	^port!

startServer
	[SmaCCServer startOn: self port] on: Error do: [:ex | self logError: ex]! !
!SmaCCRewriteServerSessionManager categoriesFor: #allowDuplicate!public! !
!SmaCCRewriteServerSessionManager categoriesFor: #notifyIcon!public! !
!SmaCCRewriteServerSessionManager categoriesFor: #notifyMessage!public! !
!SmaCCRewriteServerSessionManager categoriesFor: #port!public! !
!SmaCCRewriteServerSessionManager categoriesFor: #startServer!public! !

"Binary Globals"!

