| package |
package := Package name: 'SmaCC Runtime'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains the runtime part of SmaCC. All parsers created using SmaCC will need this code. 

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #SmaCCCaseInsensitiveToken;
	add: #SmaCCLineNumberStream;
	add: #SmaCCParseNode;
	add: #SmaCCParseNodeVisitor;
	add: #SmaCCParser;
	add: #SmaCCParserError;
	add: #SmaCCPatternNode;
	add: #SmaCCPatternToken;
	add: #SmaCCScanner;
	add: #SmaCCToken;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	yourself).

package!

"Class Definitions"!

Object subclass: #SmaCCParseNode
	instanceVariableNames: 'parent attributes'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCParseNodeVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCParser
	instanceVariableNames: 'scanner currentToken errorToken stateStack nodeStack'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCScanner
	instanceVariableNames: 'stream start matchActions matchEnd currentCharacter outputStream lastOutputStreamMatchPosition lastMatchWasEmpty returnMatchBlock state comments'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'keywordMap'!
Object subclass: #SmaCCToken
	instanceVariableNames: 'start ids value'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #SmaCCParserError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCParseNode subclass: #SmaCCPatternNode
	instanceVariableNames: 'patternToken'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCToken subclass: #SmaCCCaseInsensitiveToken
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCToken subclass: #SmaCCPatternToken
	instanceVariableNames: 'isList isToken isNode testBlock nodeClassName'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SequencedStream subclass: #SmaCCLineNumberStream
	instanceVariableNames: 'sourceStream previousWasCR eolPositions lastPosition'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCParseNode guid: (GUID fromString: '{BB35454C-2ED0-425A-A683-9281B31139CC}')!
SmaCCParseNode comment: 'SmaCCParseNode is an abstract class that represents parse nodes generated by SmaCC.

Subclasses must implement the following messages:
	visitor-double dispatching
		acceptVisitor:

Instance Variables:
	attributes	<Dictionary>	a dictionary that holds special properties for each node
	parent	<SmaCCParseNode>	the parent node in the parse tree

'!
!SmaCCParseNode categoriesForClass!Unclassified! !
!SmaCCParseNode methodsFor!

= anObject
	^self == anObject
		or:
			[self class = anObject class
				and: [self variableSelectors allSatisfy: [:symbol | (self perform: symbol) allSatisfy: [:each | (self perform: each) = (anObject perform: each)]]]]!

acceptVisitor: aSmaCCParseNodeVisitor
	^aSmaCCParseNodeVisitor visitSmaCCParseNode: self!

addFirstNode: aSmaCCParseNode to: aCollection
	aSmaCCParseNode isNil ifTrue: [^self].
	aCollection addFirst: aSmaCCParseNode.
	aSmaCCParseNode parent: self!

addFirstNodes: aSmaCCParseNodeCollection to: aCollection
	aSmaCCParseNodeCollection isNil ifTrue: [^self].
	aSmaCCParseNodeCollection reverseDo: [:each | self addFirstNode: each to: aCollection]!

addFirstObject: anObject to: aCollection
	anObject isNil ifTrue: [^self].
	aCollection addFirst: anObject.
	(anObject isKindOf: SmaCCParseNode) ifTrue: [anObject parent: self]!

addFirstObjects: addCollection to: aCollection
	addCollection isNil ifTrue: [^self].
	addCollection reverseDo: [:each | self addFirstObject: each to: aCollection]!

addFirstToken: aSmaCCToken to: aCollection
	aSmaCCToken isNil ifTrue: [^self].
	aCollection addFirst: aSmaCCToken!

addFirstTokens: aSmaCCTokenCollection to: aCollection
	aSmaCCTokenCollection isNil ifTrue: [^self].
	aCollection addAll: aSmaCCTokenCollection beforeIndex: 1!

addNode: aSmaCCParseNode to: aCollection
	aSmaCCParseNode isNil ifTrue: [^self].
	aCollection add: aSmaCCParseNode.
	aSmaCCParseNode parent: self!

addNodes: aSmaCCParseNodeCollection to: aCollection
	aSmaCCParseNodeCollection isNil ifTrue: [^self].
	aSmaCCParseNodeCollection do: [:each | self addNode: each to: aCollection]!

addObject: anObject to: aCollection
	anObject isNil ifTrue: [^self].
	aCollection add: anObject.
	(anObject isKindOf: SmaCCParseNode) ifTrue: [anObject parent: self]!

addObjects: addCollection to: aCollection
	addCollection isNil ifTrue: [^self].
	addCollection do: [:each | self addObject: each to: aCollection]!

addToken: aSmaCCToken to: aCollection
	aSmaCCToken isNil ifTrue: [^self].
	aCollection add: aSmaCCToken!

addTokens: aSmaCCTokenCollection to: aCollection
	aSmaCCTokenCollection isNil ifTrue: [^self].
	aCollection addAll: aSmaCCTokenCollection!

addVariablesFrom: anObject
	| myCompositeVariables mySingularVariables |
	(anObject isKindOf: SmaCCParseNode) ifFalse: [^self].
	myCompositeVariables := self compositeVariables.
	mySingularVariables := self singularVariables.
	anObject compositeVariables
		do: [:each | (myCompositeVariables includes: each) ifTrue: [(self perform: each) addAll: (anObject perform: each)]].
	anObject singularVariables
		do:
			[:each | 
			| newName |
			newName := self pluralVariablesMap at: each ifAbsent: [each].
			(myCompositeVariables includes: newName)
				ifTrue: [(self perform: newName asSymbol) add: (anObject perform: each)]
				ifFalse:
					[(mySingularVariables includes: each) ifTrue: [self perform: (each , ':') asSymbol with: (anObject perform: each)]]]!

allGettersDo: aBlock
	self allVariableSelectors do: [:sel | (self perform: sel) do: aBlock]!

allNodesDo: nodeBlock
	self allNodesDo: nodeBlock andTokensDo: nil!

allNodesDo: nodeBlock andTokensDo: tokenBlock
	self
		nodesDo:
			[:each | 
			nodeBlock value: each.
			each allNodesDo: nodeBlock andTokensDo: tokenBlock]
		andTokensDo: tokenBlock!

allTokens
	| tokens |
	tokens := OrderedCollection new.
	self allTokensInto: tokens.
	^tokens asSortedCollection: [:a :b | a startPosition < b startPosition]!

allTokensInto: aCollection
	self tokenVariables do: [:each | (self perform: each) ifNotNil: [:token | aCollection add: token]].
	self compositeTokenVariables
		do: [:each | (self perform: each) ifNotNil: [:tokens | aCollection addAll: tokens]].
	self nodesDo: [:each | each allTokensInto: aCollection]!

allVariableSelectors
	^#(#tokenVariables #compositeTokenVariables #nodeVariables #compositeNodeVariables #otherVariables #ignoredTokenVariables #ignoredCompositeTokenVariables #ignoredNodeVariables #ignoredCompositeNodeVariables #ignoredOtherVariables)!

attributeNamed: aSymbol
	attributes isNil ifTrue: [^nil].
	^attributes at: aSymbol ifAbsent: [nil]!

attributeNamed: aSymbol ifAbsent: aBlock
	^self attributes at: aSymbol ifAbsent: aBlock!

attributeNamed: aSymbol ifAbsentPut: aBlock
	^self attributes at: aSymbol ifAbsentPut: aBlock!

attributeNamed: aSymbol put: aValue
	^self attributes at: aSymbol put: aValue!

attributes
	^attributes ifNil: [attributes := IdentityDictionary new]!

children
	| result |
	result := OrderedCollection new.
	self nodesDo: [:each | each notNil ifTrue: [result add: each]].
	^result!

comments
	^self attributeNamed: #comments!

comments: anObject
	self attributeNamed: #comments put: anObject!

completeSource
	^parent isNil ifTrue: [self attributeNamed: #source] ifFalse: [parent completeSource]!

completeSource: aString
	^parent isNil
		ifTrue:
			[(Smalltalk at: #SmaCCString ifAbsent: [nil])
				ifNotNil: [:stringClass | self attributeNamed: #source put: (stringClass on: aString)]]
		ifFalse: [parent completeSource: aString]!

compositeNodeVariables
	^#()!

compositeTokenVariables
	^#()!

compositeVariables
	^self compositeVariableSelectors
		inject: Set new
		into:
			[:sum :each | 
			sum
				addAll: (self perform: each);
				yourself]!

compositeVariableSelectors
	^#(#compositeTokenVariables #compositeNodeVariables #ignoredCompositeTokenVariables #ignoredCompositeNodeVariables)!

convertToSetter: selector
	^(selector , ':') asSymbol!

copyInContext: aDictionary
	| copy |
	copy := self copy.
	self tokenVariables
		do:
			[:selector | copy perform: (self convertToSetter: selector) with: ((self perform: selector) copyInContext: aDictionary)].
	self nodeVariables
		do:
			[:selector | copy perform: (self convertToSetter: selector) with: ((self perform: selector) copyInContext: aDictionary)].
	self compositeTokenVariables
		do:
			[:selector | 
			copy
				perform: (self convertToSetter: selector)
				with: ((self perform: selector) collect: [:each | each copyInContext: aDictionary])].
	self compositeNodeVariables
		do:
			[:selector | 
			copy
				perform: (self convertToSetter: selector)
				with: ((self perform: selector) collect: [:each | each copyInContext: aDictionary])].
	^copy!

deleteSource
	self completeSource deleteFrom: self startPosition to: self stopPosition!

deleteToken: aSmaCCToken
	self completeSource deleteFrom: aSmaCCToken startPosition to: aSmaCCToken stopPosition!

filename
	^self attributeNamed: #filename!

filename: aString
	self attributeNamed: #filename put: aString!

fixParentPointers
	self
		nodesDo:
			[:each | 
			each parent: self.
			each fixParentPointers]!

hasDescendant: aSmaCCNode
	^aSmaCCNode isDescendantFrom: self!

hash
	^self variableSelectors
		inject: self class hash
		into: [:sum :symbol | (self perform: symbol) inject: sum into: [:hash :each | hash bitXor: (self perform: each) hash]]!

ignoredCompositeNodeVariables
	^#()!

ignoredCompositeTokenVariables
	^#()!

ignoredNodeVariables
	^#()!

ignoredOtherVariables
	^#()!

ignoredTokenVariables
	^#()!

insertSourceAfter: aString
	self completeSource insert: aString at: self stopPosition + 1!

insertSourceBefore: aString
	self completeSource insert: aString at: self startPosition!

insertSourceBefore: beginString andAfter: endString
	self
		insertSourceBefore: beginString;
		insertSourceAfter: endString!

isDescendantFrom: aSmaCCNode
	| current |
	current := self.
	[current isNil or: [current == aSmaCCNode]] whileFalse: [current := current parent].
	^current notNil!

isListMatch
	^false!

lookupNode: aBlock
	| current |
	current := self.
	[current isNil or: [aBlock value: current]] whileFalse: [current := current parent].
	^current!

match: aSmaCCParseNode inContext: aDictionary
	self class = aSmaCCParseNode class ifFalse: [^false].
	(self
		matchVariables: self tokenVariables
		to: aSmaCCParseNode
		inContext: aDictionary) ifFalse: [^false].
	(self
		matchVariables: self nodeVariables
		to: aSmaCCParseNode
		inContext: aDictionary) ifFalse: [^false].
	(self
		matchCollectionVariables: self compositeTokenVariables
		to: aSmaCCParseNode
		inContext: aDictionary) ifFalse: [^false].
	^self
		matchCollectionVariables: self compositeNodeVariables
		to: aSmaCCParseNode
		inContext: aDictionary!

matchCollection: matchNodes index: matchIndex to: programNodes index: programIndex inContext: aDictionary
	| node currentIndex currentDictionary nodes |
	matchNodes size < matchIndex ifTrue: [^programNodes size < programIndex].
	node := matchNodes at: matchIndex.
	node isListMatch
		ifTrue:
			[currentIndex := programIndex - 1.
			[currentDictionary := aDictionary copy.
			programNodes size < currentIndex
				or:
					[(self
						matchCollection: matchNodes
						index: matchIndex + 1
						to: programNodes
						index: currentIndex + 1
						inContext: currentDictionary)
						and:
							[nodes := programNodes copyFrom: programIndex to: currentIndex.
							(node match: nodes inContext: currentDictionary)
								ifTrue:
									[currentDictionary keysAndValuesDo: [:key :value | aDictionary at: key put: value].
									^true].
							false]]] whileFalse: [currentIndex := currentIndex + 1].
			^false].
	programNodes size < programIndex ifTrue: [^false].
	(node match: (programNodes at: programIndex) inContext: aDictionary) ifFalse: [^false].
	^self
		matchCollection: matchNodes
		index: matchIndex + 1
		to: programNodes
		index: programIndex + 1
		inContext: aDictionary!

matchCollection: receiverCollection to: argumentCollection inContext: aDictionary 
	^self 
		matchCollection: receiverCollection
		index: 1
		to: argumentCollection
		index: 1
		inContext: aDictionary!

matchCollectionVariables: aCollection to: aSmaCCParseNode inContext: aDictionary
	^aCollection
		allSatisfy:
			[:each | 
			self
				matchCollection: (self perform: each)
				to: (aSmaCCParseNode perform: each)
				inContext: aDictionary]!

matchVariables: aCollection to: aSmaCCParseNode inContext: aDictionary
	^aCollection
		allSatisfy:
			[:each | 
			| myValue otherValue |
			myValue := self perform: each.
			otherValue := aSmaCCParseNode perform: each.
			myValue isNil ifTrue: [otherValue isNil] ifFalse: [myValue match: otherValue inContext: aDictionary]]!

nodeOfType: aSmaCCParseNodeClassOrCollection
	| isCollection |
	isCollection := aSmaCCParseNodeClassOrCollection isKindOf: Collection.
	^self
		lookupNode:
			[:each | 
			isCollection
				ifTrue: [aSmaCCParseNodeClassOrCollection anySatisfy: [:class | each isKindOf: class]]
				ifFalse: [each isKindOf: aSmaCCParseNodeClassOrCollection]]!

nodesDo: nodeBlock
	self nodesDo: nodeBlock andTokensDo: nil!

nodesDo: nodeBlock andTokensDo: tokenBlock
	| block |
	nodeBlock notNil
		ifTrue:
			[block := [:each | (self perform: each) ifNotNil: nodeBlock].
			self nodeVariables do: block.
			self ignoredNodeVariables do: block.
			block := [:each | (self perform: each) ifNotNil: [:nodes | nodes do: nodeBlock]].
			self compositeNodeVariables do: block.
			self ignoredCompositeNodeVariables do: block].
	tokenBlock notNil
		ifTrue:
			[block := [:each | (self perform: each) ifNotNil: tokenBlock].
			self tokenVariables do: block.
			self ignoredTokenVariables do: block.
			block := [:each | (self perform: each) ifNotNil: [:nodes | nodes do: tokenBlock]].
			self compositeTokenVariables do: block.
			self ignoredCompositeTokenVariables do: block].
	block := [:each | 
	nodeBlock notNil ifTrue: [(each isKindOf: SmaCCParseNode) ifTrue: [nodeBlock value: each]].
	tokenBlock notNil ifTrue: [(each isKindOf: SmaCCToken) ifTrue: [tokenBlock value: each]].
	(each isKindOf: Collection) ifTrue: [each do: block]].
	self otherVariables do: [:sel | block value: (self perform: sel)].
	self ignoredOtherVariables do: [:sel | block value: (self perform: sel)]!

nodeVariables
	^#()!

otherVariables
	^#()!

parent
	^parent!

parent: aSmaCCParseNode
	parent := aSmaCCParseNode!

parentOfType: aSmaCCParseNodeClassOrCollection
	^parent notNil ifTrue: [parent nodeOfType: aSmaCCParseNodeClassOrCollection] ifFalse: [nil]!

pluralVariablesMap
	^##(Dictionary new)!

postCopy
	super postCopy.
	self
		allGettersDo:
			[:each | 
			self
				perform: (self convertToSetter: each)
				with:
					((self compositeVariableSelectors includes: each)
						ifTrue: [(self perform: each) collect: [:item | item copy]]
						ifFalse: [(self perform: each) copy])]!

publishedAspects
	| aspects aspectClass |
	aspectClass := Smalltalk at: #Aspect.
	aspects := LookupTable new.
	aspects add: (self newAspect: #yourself class: aspectClass).
	aspects add: (aspectClass multilineString: #source) beReadOnly.
	aspectClass := Smalltalk at: #InstVarAspect.
	1 to: self class instSize do: [:i | aspects add: ((self instVarAt: i) newAspect: i class: aspectClass)].
	^aspects!

removeAttributeNamed: aSymbol
	attributes isNil ifTrue: [^self].
	attributes removeKey: aSymbol ifAbsent: []!

removePositions
	self removeAttributeNamed: #startPosition.
	self removeAttributeNamed: #stopPosition!

replace: originalSmaCCParseNode with: newSmaCCParseNode
	SmaCCParseNode instSize + 1
		to: self class instSize
		do:
			[:i | 
			| each |
			each := self instVarAt: i.
			each == originalSmaCCParseNode
				ifTrue: [self instVarAt: i put: newSmaCCParseNode]
				ifFalse:
					[(each isKindOf: SequenceableCollection)
						ifTrue:
							[each keysAndValuesDo: [:index :value | value == originalSmaCCParseNode ifTrue: [each at: index put: newSmaCCParseNode]]]]].
	self completeSource
		ifNotNil:
			[:source | 
			source
				replaceFrom: originalSmaCCParseNode startPosition
				to: originalSmaCCParseNode stopPosition
				with: newSmaCCParseNode source].
	newSmaCCParseNode parent: self!

replaceToken: aSmaCCToken withSource: aString
	self completeSource
		replaceFrom: aSmaCCToken startPosition
		to: aSmaCCToken stopPosition
		with: aString!

replaceWith: aSmaCCParseNode
	parent replace: self with: aSmaCCParseNode!

replaceWithSource: aString
	^self completeSource
		replaceFrom: self startPosition
		to: self stopPosition
		with: aString!

safePerform: aSymbol
	^(self respondsTo: aSymbol) ifTrue: [self perform: aSymbol] ifFalse: [nil]!

setParents: aCollection to: anObject
	aCollection isNil ifTrue: [^self].
	aCollection do: [:each | each parent: anObject]!

singularVariables
	^self singularVariableSelectors
		inject: Set new
		into:
			[:sum :each | 
			sum
				addAll: (self perform: each);
				yourself]!

singularVariableSelectors
	^#(#tokenVariables #nodeVariables #ignoredTokenVariables #ignoredNodeVariables #otherVariables #ignoredOtherVariables)!

sortedChildren
	^self children
		asSortedCollection:
			[:a :b | 
			| startA startB |
			startA := a startPosition.
			startB := b startPosition.
			startA isNil
				or:
					[b notNil and: [startA class = startB class ifTrue: [startA < startB] ifFalse: [startA class name < startB class name]]]]!

source
	^self completeSource ifNotNil: [:value | (value copyFrom: self startPosition to: self stopPosition) asString]!

startPosition
	"Hackish way to get the minimum for all nodes. You might want to override to get a better implementation."

	^self
		attributeNamed: #startPosition
		ifAbsentPut:
			[| min doBlock |
			min := SmallInteger maximum.
			doBlock := [:each | each startPosition ifNotNil: [:pos | min := min min: pos]].
			self nodesDo: doBlock andTokensDo: doBlock.
			min]!

stopPosition
	"Hackish way to get the maximum for all nodes. You might want to override to get a better implementation."

	^self
		attributeNamed: #stopPosition
		ifAbsentPut:
			[| max doBlock |
			max := 0.
			doBlock := [:each | each stopPosition ifNotNil: [:pos | max := max max: pos]].
			self nodesDo: doBlock andTokensDo: doBlock.
			max]!

tokenVariables
	^#()!

topParent
	| current |
	current := self.
	[current parent notNil] whileTrue: [current := current parent].
	^current!

variableSelectors
	^#(#tokenVariables #compositeTokenVariables #nodeVariables #compositeNodeVariables #otherVariables)! !
!SmaCCParseNode categoriesFor: #=!comparing!public! !
!SmaCCParseNode categoriesFor: #acceptVisitor:!private!visitor-double dispatching! !
!SmaCCParseNode categoriesFor: #addFirstNode:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addFirstNodes:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addFirstObject:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addFirstObjects:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addFirstToken:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addFirstTokens:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addNode:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addNodes:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addObject:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addObjects:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addToken:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addTokens:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #addVariablesFrom:!private!variables! !
!SmaCCParseNode categoriesFor: #allGettersDo:!iterating!private! !
!SmaCCParseNode categoriesFor: #allNodesDo:!iterating!public! !
!SmaCCParseNode categoriesFor: #allNodesDo:andTokensDo:!iterating!private! !
!SmaCCParseNode categoriesFor: #allTokens!accessing!public! !
!SmaCCParseNode categoriesFor: #allTokensInto:!accessing!public! !
!SmaCCParseNode categoriesFor: #allVariableSelectors!private!variables! !
!SmaCCParseNode categoriesFor: #attributeNamed:!accessing-attributes!private! !
!SmaCCParseNode categoriesFor: #attributeNamed:ifAbsent:!accessing-attributes!private! !
!SmaCCParseNode categoriesFor: #attributeNamed:ifAbsentPut:!accessing-attributes!private! !
!SmaCCParseNode categoriesFor: #attributeNamed:put:!accessing-attributes!private! !
!SmaCCParseNode categoriesFor: #attributes!accessing-attributes!private! !
!SmaCCParseNode categoriesFor: #children!accessing!private! !
!SmaCCParseNode categoriesFor: #comments!accessing!private! !
!SmaCCParseNode categoriesFor: #comments:!accessing!private! !
!SmaCCParseNode categoriesFor: #completeSource!private!source! !
!SmaCCParseNode categoriesFor: #completeSource:!private!source! !
!SmaCCParseNode categoriesFor: #compositeNodeVariables!private!variables! !
!SmaCCParseNode categoriesFor: #compositeTokenVariables!private!variables! !
!SmaCCParseNode categoriesFor: #compositeVariables!private!variables! !
!SmaCCParseNode categoriesFor: #compositeVariableSelectors!private!variables! !
!SmaCCParseNode categoriesFor: #convertToSetter:!matching!private! !
!SmaCCParseNode categoriesFor: #copyInContext:!matching!private! !
!SmaCCParseNode categoriesFor: #deleteSource!private!source! !
!SmaCCParseNode categoriesFor: #deleteToken:!private!source! !
!SmaCCParseNode categoriesFor: #filename!accessing!public! !
!SmaCCParseNode categoriesFor: #filename:!accessing!public! !
!SmaCCParseNode categoriesFor: #fixParentPointers!initialize-release!private! !
!SmaCCParseNode categoriesFor: #hasDescendant:!public!testing! !
!SmaCCParseNode categoriesFor: #hash!comparing!private! !
!SmaCCParseNode categoriesFor: #ignoredCompositeNodeVariables!private!variables! !
!SmaCCParseNode categoriesFor: #ignoredCompositeTokenVariables!private!variables! !
!SmaCCParseNode categoriesFor: #ignoredNodeVariables!private!variables! !
!SmaCCParseNode categoriesFor: #ignoredOtherVariables!private!variables! !
!SmaCCParseNode categoriesFor: #ignoredTokenVariables!private!variables! !
!SmaCCParseNode categoriesFor: #insertSourceAfter:!private!source! !
!SmaCCParseNode categoriesFor: #insertSourceBefore:!private!source! !
!SmaCCParseNode categoriesFor: #insertSourceBefore:andAfter:!private!source! !
!SmaCCParseNode categoriesFor: #isDescendantFrom:!public!testing! !
!SmaCCParseNode categoriesFor: #isListMatch!private!testing! !
!SmaCCParseNode categoriesFor: #lookupNode:!accessing!public! !
!SmaCCParseNode categoriesFor: #match:inContext:!matching!public! !
!SmaCCParseNode categoriesFor: #matchCollection:index:to:index:inContext:!matching!private! !
!SmaCCParseNode categoriesFor: #matchCollection:to:inContext:!matching!private! !
!SmaCCParseNode categoriesFor: #matchCollectionVariables:to:inContext:!matching!public! !
!SmaCCParseNode categoriesFor: #matchVariables:to:inContext:!matching!public! !
!SmaCCParseNode categoriesFor: #nodeOfType:!accessing!public! !
!SmaCCParseNode categoriesFor: #nodesDo:!iterating!public! !
!SmaCCParseNode categoriesFor: #nodesDo:andTokensDo:!iterating!public! !
!SmaCCParseNode categoriesFor: #nodeVariables!private!variables! !
!SmaCCParseNode categoriesFor: #otherVariables!private!variables! !
!SmaCCParseNode categoriesFor: #parent!accessing!private! !
!SmaCCParseNode categoriesFor: #parent:!accessing!private! !
!SmaCCParseNode categoriesFor: #parentOfType:!accessing!public! !
!SmaCCParseNode categoriesFor: #pluralVariablesMap!public!variables! !
!SmaCCParseNode categoriesFor: #postCopy!copying!private! !
!SmaCCParseNode categoriesFor: #publishedAspects!development!public! !
!SmaCCParseNode categoriesFor: #removeAttributeNamed:!accessing-attributes!public! !
!SmaCCParseNode categoriesFor: #removePositions!accessing!private! !
!SmaCCParseNode categoriesFor: #replace:with:!private!replacing! !
!SmaCCParseNode categoriesFor: #replaceToken:withSource:!private!source! !
!SmaCCParseNode categoriesFor: #replaceWith:!private!replacing! !
!SmaCCParseNode categoriesFor: #replaceWithSource:!private!source! !
!SmaCCParseNode categoriesFor: #safePerform:!private! !
!SmaCCParseNode categoriesFor: #setParents:to:!collection handling!private! !
!SmaCCParseNode categoriesFor: #singularVariables!private!variables! !
!SmaCCParseNode categoriesFor: #singularVariableSelectors!private!variables! !
!SmaCCParseNode categoriesFor: #sortedChildren!accessing!private! !
!SmaCCParseNode categoriesFor: #source!private!source! !
!SmaCCParseNode categoriesFor: #startPosition!accessing!private! !
!SmaCCParseNode categoriesFor: #stopPosition!accessing!private! !
!SmaCCParseNode categoriesFor: #tokenVariables!private!variables! !
!SmaCCParseNode categoriesFor: #topParent!accessing!public! !
!SmaCCParseNode categoriesFor: #variableSelectors!private!variables! !

!SmaCCParseNode class methodsFor!

collectionVariableIn: aSmaCCParseNode collectionGetter: aSymbol variableGetter: varSymbol
	(aSmaCCParseNode respondsTo: aSymbol) ifTrue: [^aSmaCCParseNode perform: aSymbol].
	(aSmaCCParseNode respondsTo: varSymbol) ifTrue: [^Array with: (aSmaCCParseNode perform: varSymbol)].
	^#()!

new
	^(super new)
		initialize;
		yourself!

variableIn: aSmaCCParseNode getter: aSymbol
	(aSmaCCParseNode respondsTo: aSymbol) ifTrue: [^aSmaCCParseNode perform: aSymbol].
	^nil! !
!SmaCCParseNode class categoriesFor: #collectionVariableIn:collectionGetter:variableGetter:!accessing!private! !
!SmaCCParseNode class categoriesFor: #new!instance creation!private! !
!SmaCCParseNode class categoriesFor: #variableIn:getter:!accessing!private! !

SmaCCParseNodeVisitor guid: (GUID fromString: '{923B67EA-E50C-4DA5-B1A4-AFF14D165053}')!
SmaCCParseNodeVisitor comment: 'SmaCCParseNodeVisitor is an abstract visitor for the SmaCC parse trees.

'!
!SmaCCParseNodeVisitor categoriesForClass!Unclassified! !
!SmaCCParseNodeVisitor methodsFor!

accept: anObject
	(anObject isKindOf: SmaCCParseNode) ifTrue: [^self acceptNode: anObject].
	(anObject isKindOf: Collection) ifTrue: [^self acceptNodes: anObject].
	^anObject!

acceptNode: aSmaCCParseNode
	aSmaCCParseNode isNil ifTrue: [^nil].
	^aSmaCCParseNode acceptVisitor: self!

acceptNodes: aCollection
	aCollection do: [:each | self acceptNode: each].
	^aCollection!

visitSmaCCParseNode: aSmaCCParseNode
	self acceptNodes: aSmaCCParseNode sortedChildren.
	^aSmaCCParseNode! !
!SmaCCParseNodeVisitor categoriesFor: #accept:!private!visiting! !
!SmaCCParseNodeVisitor categoriesFor: #acceptNode:!private!visiting! !
!SmaCCParseNodeVisitor categoriesFor: #acceptNodes:!private!visiting! !
!SmaCCParseNodeVisitor categoriesFor: #visitSmaCCParseNode:!private!visiting! !

SmaCCParser guid: (GUID fromString: '{D34DD6D6-CF03-4A07-94AF-1FAEE684B152}')!
SmaCCParser comment: 'SmaCCParser is an abstract class that defines most of the parsing actions. Subclasses will define methods that specify their transitions and reduction actions. These are normally defined automatically when compiling the parser.

Subclasses must implement the following messages:
	accessing
		reduceTable
		transitionTable

Instance Variables:
	currentToken	<SmaCCToken>	the token last returned by the scanner that has not been shifted (reduce actions leave the current token alone)
	errorToken	<SmaCCToken>	the token where a parse error occurred
	nodeStack	<OrderedCollection of: Object>	collection of items on stack. These items are specific to the parser and can be any object. 
	scanner	<SmaCCScanner>	our scanner
	stateStack	<OrderedCollection of: Integer>	the stack of states for our parser (standard LR state stack)

'!
!SmaCCParser categoriesForClass!Unclassified! !
!SmaCCParser methodsFor!

acceptAction
	^0!

actionFor: aSymbolIndex 
	^self actionForState: self currentState and: aSymbolIndex!

actionForCurrentToken
	self tryAllTokens
		ifTrue:
			[| ids action |
			ids := currentToken ids.
			1
				to: ids size
				do:
					[:i | 
					action := self actionFor: (ids at: i).
					(action bitAnd: self actionMask) = self errorAction ifFalse: [^action]].
			^self errorAction]
		ifFalse: [^self actionFor: currentToken ids first]!

actionForState: stateIndex and: aSymbolIndex
	| index row |
	row := self transitionTable at: stateIndex.
	^(row at: 1) == 0
		ifTrue:
			[index := self
				binarySearchIn: row
				for: aSymbolIndex
				size: 2.
			index == 0 ifTrue: [^self errorAction] ifFalse: [^((row at: 2) bitShift: 8) + (row at: 3)]]
		ifFalse:
			[index := self
				binarySearchIn: row
				for: aSymbolIndex
				size: 4.
			index == 0 ifTrue: [^self errorAction] ifFalse: [^((row at: index - 2) bitShift: 8) + (row at: index - 1)]]!

actionMask
	^2r11!

add: anObject to: aCollection
	anObject isNil ifTrue: [^self].
	aCollection add: anObject!

addAll: origCollection to: aCollection
	origCollection isNil ifTrue: [^self].
	origCollection do: [:each | each notNil ifTrue: [aCollection add: each]]!

binarySearchIn: aRow for: aSymbolIndex size: step
	| start mid length high low midItem stop |
	high := aSymbolIndex bitShift: -8.
	low := aSymbolIndex bitAnd: 16rFF.
	start := 4.
	stop := aRow size - 1.
	length := (stop - start) // step.
	[length > 4]
		whileTrue:
			[length := length bitShift: -1.
			mid := length * step + start.
			midItem := aRow at: mid.
			((midItem == high and: [(aRow at: mid + 1) <= low]) or: [midItem < high])
				ifTrue: [start := mid]
				ifFalse: [stop := mid]].
	[start <= stop]
		whileTrue:
			[((aRow at: start) == high and: [(aRow at: start + 1) == low]) ifTrue: [^start].
			start := start + step].
	^0!

checkForErrors
	"If we have an error correction installed, we might have handled the errors. If we did, we don't 
	want to return the result, so we raise a final exception that can't be proceeded."

	errorToken isNil ifTrue: [^self].
	currentToken := errorToken.
	self reportErrorMessage: 'Token not expected'!

currentState
	^stateStack last!

dismissErrorToken
	currentToken := nil.
	self getNextToken!

dismissStackTopForErrorRecovery
	stateStack removeLast.
	^nodeStack removeLast!

emptySymbolTokenId
	^scanner emptySymbolTokenId!

errorAction
	^3!

errorHandlerStates
	^stateStack
		collect:
			[:each | 
			| action |
			action := self actionForState: each and: self errorTokenId.
			(action bitAnd: self actionMask) = 1 ifTrue: [action bitShift: -2] ifFalse: [0]]!

errorTable
	^#()!

errorToken
	^errorToken!

errorTokenId
	^scanner errorTokenId!

findErrorHandlerIfNoneUseErrorNumber: anInteger
	| handlerStates index startingErrorToken |
	handlerStates := self errorHandlerStates reverse.
	startingErrorToken := currentToken.
	[index := (1 to: handlerStates size)
		detect:
			[:each | 
			| state newStack |
			state := handlerStates at: each.
			state ~= 0
				and:
					[newStack := stateStack copyFrom: 1 to: handlerStates size - each + 1.
					newStack addLast: state.
					self willShift: newStack]]
		ifNone: [nil].
	index isNil]
		whileTrue:
			[self dismissErrorToken.
			self isEOFToken
				ifTrue:
					[currentToken := startingErrorToken.
					self reportError: anInteger]].
	index - 1 timesRepeat: [self dismissStackTopForErrorRecovery].
	stateStack addLast: (handlerStates at: index).
	nodeStack addLast: startingErrorToken!

getNextToken
	currentToken isNil ifTrue: [currentToken := scanner next]!

handleError: anInteger
	errorToken isNil ifTrue: [errorToken := currentToken].
	(self isEOFToken or: [self hasErrorHandler not]) ifTrue: [self reportError: anInteger].
	self findErrorHandlerIfNoneUseErrorNumber: anInteger!

hasErrorHandler
	^self errorHandlerStates anySatisfy: [:each | each ~~ 0]!

initialize
	super initialize.
	nodeStack := OrderedCollection new!

isEOFToken
	^currentToken ids first = self emptySymbolTokenId!

liftFirstValue: aCollection 
	^aCollection first!

liftLastValue: aCollection 
	^aCollection last!

liftSecondValue: aCollection 
	^aCollection at: 2!

nil
	^nil!

parse
	| result |
	self setDefaultStartingStateIfNone.
	self performParsingLoop.
	result := nodeStack last.
	(result respondsTo: #comments:) ifTrue: [result comments: scanner comments].
	^result!

performAction: action
	| value actionType |
	actionType := action bitAnd: self actionMask.
	value := action bitShift: -2.
	actionType == self shiftAction
		ifTrue: [self shift: value]
		ifFalse: [actionType == self reduceAction ifTrue: [self reduce: value] ifFalse: [self handleError: value]]!

performParsingLoop
	| action |
	[self getNextToken.
	action := self actionForCurrentToken.
	action = self acceptAction] whileFalse: [self performAction: action].
	self checkForErrors!

performReduceMethod: aSymbol with: items
	^aSymbol last == $: ifTrue: [self perform: aSymbol with: items] ifFalse: [self perform: aSymbol]!

popNodeStack
	^nodeStack removeLast!

position
	^currentToken isNil ifTrue: [scanner position] ifFalse: [currentToken startPosition]!

position: anInteger
	scanner setPosition: anInteger!

reduce: anInteger
	| reduceEntry items size |
	reduceEntry := self reduceTable at: anInteger.
	items := Array new: (size := reduceEntry at: 2).
	size
		to: 1
		by: -1
		do:
			[:i | 
			items at: i put: self popNodeStack.
			stateStack removeLast].
	nodeStack addLast: (self performReduceMethod: (reduceEntry at: 3) with: items).
	stateStack addLast: ((self actionFor: (reduceEntry at: 1)) bitShift: -2)!

reduceAction
	^2r10!

reduceFor: aCollection
	| newCollection item |
	(aCollection allSatisfy: [:each | each class ~~ OrderedCollection]) ifTrue: [^aCollection asOrderedCollection].
	aCollection first class == OrderedCollection
		ifTrue:
			[newCollection := aCollection first.
			2
				to: aCollection size
				do:
					[:i | 
					item := aCollection at: i.
					item class == OrderedCollection ifTrue: [newCollection addAll: item] ifFalse: [newCollection add: item]].
			^newCollection].
	newCollection := OrderedCollection new.
	aCollection
		do: [:each | each class == OrderedCollection ifTrue: [newCollection addAll: each] ifFalse: [newCollection add: each]].
	^newCollection!

reduceTable
	^self subclassResponsibility!

reportError: anInteger
	self reportErrorMessage: (anInteger = 0 ifTrue: ['Token not expected'] ifFalse: [self errorTable at: anInteger])!

reportErrorMessage: aString
	SmaCCParserError signal: aString with: self!

scanner
	^scanner!

scanner: aScanner 
	scanner := aScanner!

setDefaultStartingStateIfNone
	stateStack isNil ifTrue: [self setStartingState: self class defaultStartingState]!

setStartingState: startingState 
	stateStack := OrderedCollection with: startingState!

shift: stateIndex
	stateStack addLast: stateIndex.
	nodeStack addLast: currentToken.
	currentToken := nil!

shiftAction
	^2r01!

state
	^scanner state!

state: aSymbol
	scanner state: aSymbol!

stringValue: anOrderedCollection 
	^anOrderedCollection first value!

symbolTypes
	^#()!

transitionTable
	^self subclassResponsibility!

tryAllTokens
	^false!

willShift: potentialStateStack
	| action compoundAction reduceEntry size |
	compoundAction := self actionForState: potentialStateStack last and: currentToken ids first.
	action := compoundAction bitAnd: self actionMask.
	action == self shiftAction ifTrue: [^true].
	action == self reduceAction
		ifTrue:
			[reduceEntry := self reduceTable at: (compoundAction bitShift: -2).
			size := reduceEntry at: 2.
			size timesRepeat: [potentialStateStack removeLast].
			potentialStateStack add: ((self actionForState: potentialStateStack last and: (reduceEntry at: 1)) bitShift: -2).
			^self willShift: potentialStateStack].
	^false! !
!SmaCCParser categoriesFor: #acceptAction!private! !
!SmaCCParser categoriesFor: #actionFor:!private! !
!SmaCCParser categoriesFor: #actionForCurrentToken!private! !
!SmaCCParser categoriesFor: #actionForState:and:!private! !
!SmaCCParser categoriesFor: #actionMask!private! !
!SmaCCParser categoriesFor: #add:to:!private! !
!SmaCCParser categoriesFor: #addAll:to:!private! !
!SmaCCParser categoriesFor: #binarySearchIn:for:size:!private! !
!SmaCCParser categoriesFor: #checkForErrors!error handling!private! !
!SmaCCParser categoriesFor: #currentState!private! !
!SmaCCParser categoriesFor: #dismissErrorToken!error handling!private! !
!SmaCCParser categoriesFor: #dismissStackTopForErrorRecovery!error handling!private! !
!SmaCCParser categoriesFor: #emptySymbolTokenId!accessing!private! !
!SmaCCParser categoriesFor: #errorAction!private! !
!SmaCCParser categoriesFor: #errorHandlerStates!error handling!private! !
!SmaCCParser categoriesFor: #errorTable!accessing!private! !
!SmaCCParser categoriesFor: #errorToken!accessing!private! !
!SmaCCParser categoriesFor: #errorTokenId!accessing!private! !
!SmaCCParser categoriesFor: #findErrorHandlerIfNoneUseErrorNumber:!error handling!private! !
!SmaCCParser categoriesFor: #getNextToken!private! !
!SmaCCParser categoriesFor: #handleError:!error handling!private! !
!SmaCCParser categoriesFor: #hasErrorHandler!error handling!private! !
!SmaCCParser categoriesFor: #initialize!initialize-release!public! !
!SmaCCParser categoriesFor: #isEOFToken!private!testing! !
!SmaCCParser categoriesFor: #liftFirstValue:!private!standard reduction rules! !
!SmaCCParser categoriesFor: #liftLastValue:!private!standard reduction rules! !
!SmaCCParser categoriesFor: #liftSecondValue:!private!standard reduction rules! !
!SmaCCParser categoriesFor: #nil!private!standard reduction rules! !
!SmaCCParser categoriesFor: #parse!accessing!private! !
!SmaCCParser categoriesFor: #performAction:!private! !
!SmaCCParser categoriesFor: #performParsingLoop!private! !
!SmaCCParser categoriesFor: #performReduceMethod:with:!private! !
!SmaCCParser categoriesFor: #popNodeStack!private! !
!SmaCCParser categoriesFor: #position!accessing!private! !
!SmaCCParser categoriesFor: #position:!accessing!private! !
!SmaCCParser categoriesFor: #reduce:!private! !
!SmaCCParser categoriesFor: #reduceAction!private! !
!SmaCCParser categoriesFor: #reduceFor:!private! !
!SmaCCParser categoriesFor: #reduceTable!accessing!private! !
!SmaCCParser categoriesFor: #reportError:!error handling!private! !
!SmaCCParser categoriesFor: #reportErrorMessage:!error handling!private! !
!SmaCCParser categoriesFor: #scanner!accessing!private! !
!SmaCCParser categoriesFor: #scanner:!initialize-release!private! !
!SmaCCParser categoriesFor: #setDefaultStartingStateIfNone!initialize-release!private! !
!SmaCCParser categoriesFor: #setStartingState:!initialize-release!private! !
!SmaCCParser categoriesFor: #shift:!private! !
!SmaCCParser categoriesFor: #shiftAction!private! !
!SmaCCParser categoriesFor: #state!accessing!private! !
!SmaCCParser categoriesFor: #state:!accessing!private! !
!SmaCCParser categoriesFor: #stringValue:!private!standard reduction rules! !
!SmaCCParser categoriesFor: #symbolTypes!accessing!private! !
!SmaCCParser categoriesFor: #transitionTable!accessing!private! !
!SmaCCParser categoriesFor: #tryAllTokens!private!testing! !
!SmaCCParser categoriesFor: #willShift:!error handling!private! !

!SmaCCParser class methodsFor!

defaultStartingState
	^1!

errorBlock: aBlock
	^[:ex | 
	| returnValue |
	returnValue := aBlock argumentCount = 3
		ifTrue:
			[aBlock
				value: ex tag
				value: ex description
				value: ex tag]
		ifFalse:
			[aBlock argumentCount = 1 ifTrue: [aBlock value: ex tag] ifFalse: [aBlock value: ex description value: ex tag position]].
	ex return: returnValue]!

fileExtension
	^''!

fileExtensionType
	^'All files'!

new
	^(super new)
		initialize;
		yourself!

on: aStream 
	| parser scanner |
	scanner := self scannerClass on: aStream.
	parser := self new.
	parser scanner: scanner.
	^parser!

parse: aString 
	^self parse: aString startingAt: self defaultStartingState!

parse: aString onError: aBlock
	^[self parse: aString] on: SmaCCParserError do: (self errorBlock: aBlock)!

parse: aString startingAt: anInteger
	| value |
	value := self parseStream: (ReadStream on: aString) startingAt: anInteger.
	(value isKindOf: SmaCCParseNode) ifTrue: [value completeSource: aString].
	^value!

parse: aString startingAt: anInteger onError: aBlock
	^[self parse: aString startingAt: anInteger] on: SmaCCParserError do: (self errorBlock: aBlock)!

parseFile: aString
	| stream |
	stream := FileStream read: aString.
	^[self parseStream: stream startingAt: self defaultStartingState] ensure: [stream close]!

parseStream: aStream 
	^self parseStream: aStream startingAt: self defaultStartingState!

parseStream: aStream onError: aBlock
	^[self parseStream: aStream] on: SmaCCParserError do: (self errorBlock: aBlock)!

parseStream: aStream startingAt: anInteger 
	| parser |
	parser := self on: aStream.
	parser setStartingState: anInteger.
	^parser parse!

parseStream: aStream startingAt: anInteger onError: aBlock
	^[self parseStream: aStream startingAt: anInteger] on: SmaCCParserError do: (self errorBlock: aBlock)!

resetCaches
	"self withAllSubclasses do: [:each | each resetCaches]"!

scannerClass
	^self subclassResponsibility! !
!SmaCCParser class categoriesFor: #defaultStartingState!accessing!private! !
!SmaCCParser class categoriesFor: #errorBlock:!accessing!private! !
!SmaCCParser class categoriesFor: #fileExtension!file types!public! !
!SmaCCParser class categoriesFor: #fileExtensionType!file types!public! !
!SmaCCParser class categoriesFor: #new!instance creation!private! !
!SmaCCParser class categoriesFor: #on:!instance creation!private! !
!SmaCCParser class categoriesFor: #parse:!accessing!private! !
!SmaCCParser class categoriesFor: #parse:onError:!accessing!private! !
!SmaCCParser class categoriesFor: #parse:startingAt:!accessing!private! !
!SmaCCParser class categoriesFor: #parse:startingAt:onError:!accessing!private! !
!SmaCCParser class categoriesFor: #parseFile:!accessing!private! !
!SmaCCParser class categoriesFor: #parseStream:!accessing!private! !
!SmaCCParser class categoriesFor: #parseStream:onError:!accessing!private! !
!SmaCCParser class categoriesFor: #parseStream:startingAt:!accessing!private! !
!SmaCCParser class categoriesFor: #parseStream:startingAt:onError:!accessing!private! !
!SmaCCParser class categoriesFor: #resetCaches!accessing!public! !
!SmaCCParser class categoriesFor: #scannerClass!accessing!private! !

SmaCCScanner guid: (GUID fromString: '{020AE7BF-BAA0-482D-9590-AED301A2702B}')!
SmaCCScanner comment: 'SmaCCScanner is an abstract class that represents a scanner for the parser. The scanner converts its string input into SmaCCToken objects that the parser then uses for its parsing.

Subclasses must implement the following messages:
	accessing
		emptySymbolTokenId
		errorTokenId
		scanForToken

Instance Variables:
	comments	<OrderedCollection>	a collection of comment intervals (array of start position & stop position)
	currentCharacter	<Character>	the current character we are scanning
	lastMatchWasEmpty	<Boolean>	was our last scanning match an empty string -- don''t allow two empty matches in a row
	lastOutputStreamMatchPosition	<Integer>	the position in the outputStream of the last match
	matchActions	<Array | Symbol>	the actions for the last match (a symbol means that the action should be performed on the scanner)
	matchEnd	<Integer>	the position of the last match in the stream (our input stream)
	outputStream	<PositionableStream>	the matched characters go in this stream. After a match is made, we take this stream''s contents and create a token object.
	returnMatchBlock	<BlockClosure>	when we match a token evaluate this block with the token (hack to return from multiple levels)
	start	<Integer>	the starting position of a match in the stream
	state	<Symbol>	the state of the scanner
	stream	<Stream>	our input


'!
!SmaCCScanner categoriesForClass!Unclassified! !
!SmaCCScanner methodsFor!

atEnd
	^stream atEnd!

checkForKeyword: aString
	| stateMap action oldActions |
	action := matchActions isSymbol ifTrue: [matchActions] ifFalse: [matchActions first].
	stateMap := self class keywordMap at: action ifAbsent: [nil].
	stateMap isNil ifTrue: [^self].
	oldActions := matchActions.
	matchActions := stateMap at: (self keywordFor: aString) ifAbsent: [matchActions].
	matchActions isInteger ifTrue: [matchActions := (Array with: matchActions) , oldActions]!

checkForValidMatch
	matchActions isNil ifTrue: [self scannerError]!

comment
	comments add: (Array with: start + 1 with: matchEnd).
	^self whitespace!

comments
	^comments!

contents
	| writeStream token |
	writeStream := WriteStream with: Array new.
	[self atEnd] whileFalse: 
			[token := self next.
			token notNil ifTrue: [writeStream nextPut: token]].
	^writeStream contents!

createTokenFor: string
	| token |
	token := self tokenClass
		value: string
		start: start + 1
		ids: matchActions.
	outputStream reset.
	^token!

emptySymbolTokenId
	^self subclassResponsibility!

eofTokenId
	^Array with: self emptySymbolTokenId!

errorTokenId
	^self subclassResponsibility!

initialize
	super initialize.
	outputStream := WriteStream with: String new.
	lastMatchWasEmpty := true.
	state := #default.
	comments := OrderedCollection new!

keywordFor: aString 
	"Subclasses can override this to ignore case"

	^aString!

lineNumber
	"This requires the stream to be a line number stream (see the #needsLineNumbers class method)."

	^stream lineNumber!

next
	self resetScanner.
	returnMatchBlock := [:match | ^match].
	^self scanForToken!

on: aStream 
	stream := aStream.
	start := stream position!

patternToken
	| token |
	token := SmaCCPatternToken
		value: outputStream contents
		start: start + 1
		ids: matchActions.
	outputStream reset.
	^token!

position
	^stream position!

position: anInteger
	^stream position: anInteger!

recordAndReportMatch: aCollection
	self recordMatch: aCollection.
	^self reportLastMatch!

recordMatch: aCollection 
	matchActions := aCollection.
	matchEnd := stream position.
	lastOutputStreamMatchPosition := outputStream position!

reportLastMatch
	"The scanner has found the end of a token and must report it"

	| string action |
	self checkForValidMatch.
	self resetOutputToLastMatch.
	stream position: matchEnd.
	string := outputStream contents.
	self checkForKeyword: string.
	action := matchActions isSymbol
		ifTrue:
			["This is only necessary to run version 1 parsers on version 2 SmaCC"
			matchActions]
		ifFalse: [self tokenActions at: matchActions first ifAbsent: [nil]].
	^action notNil ifTrue: [self perform: action] ifFalse: [self createTokenFor: string]!

resetOutputToLastMatch
	outputStream position: lastOutputStreamMatchPosition.
	lastOutputStreamMatchPosition == 0 
		ifTrue: 
			[lastMatchWasEmpty ifTrue: [self scannerError].
			lastMatchWasEmpty := true]
		ifFalse: [lastMatchWasEmpty := false]!

resetScanner
	matchActions := nil.
	start := stream position.
	outputStream reset.
	lastOutputStreamMatchPosition := 0!

scanForString: aString
	^aString allSatisfy: 
			[:each |
			self step.
			currentCharacter == each]!

scanForToken
	^self subclassResponsibility!

scannerError
	| string |
	(stream atEnd and: [start == stream position]) 
		ifTrue: 
			[string := ''.
			matchActions := self eofTokenId]
		ifFalse: 
			[stream position: start.
			string := String with: stream next.
			matchActions := #(0)].
	returnMatchBlock value: (self createTokenFor: string)!

setPosition: anInteger
	^stream position: anInteger!

state
	^state!

state: aSymbol
	state := aSymbol!

step
	stream atEnd ifTrue: [^returnMatchBlock value: self reportLastMatch].
	currentCharacter := stream next.
	outputStream nextPut: currentCharacter!

tokenActions
	^#()!

tokenClass
	^SmaCCToken!

whitespace
	"By default, eat the whitespace"

	self resetScanner.
	^self scanForToken! !
!SmaCCScanner categoriesFor: #atEnd!private!testing! !
!SmaCCScanner categoriesFor: #checkForKeyword:!private! !
!SmaCCScanner categoriesFor: #checkForValidMatch!private! !
!SmaCCScanner categoriesFor: #comment!default token handling!private! !
!SmaCCScanner categoriesFor: #comments!accessing!public! !
!SmaCCScanner categoriesFor: #contents!accessing!private! !
!SmaCCScanner categoriesFor: #createTokenFor:!private! !
!SmaCCScanner categoriesFor: #emptySymbolTokenId!accessing!private! !
!SmaCCScanner categoriesFor: #eofTokenId!accessing!private! !
!SmaCCScanner categoriesFor: #errorTokenId!accessing!private! !
!SmaCCScanner categoriesFor: #initialize!initialize-release!public! !
!SmaCCScanner categoriesFor: #keywordFor:!private!utility! !
!SmaCCScanner categoriesFor: #lineNumber!accessing!private! !
!SmaCCScanner categoriesFor: #next!accessing!private! !
!SmaCCScanner categoriesFor: #on:!initialize-release!private! !
!SmaCCScanner categoriesFor: #patternToken!private! !
!SmaCCScanner categoriesFor: #position!accessing!private! !
!SmaCCScanner categoriesFor: #position:!accessing!public! !
!SmaCCScanner categoriesFor: #recordAndReportMatch:!private! !
!SmaCCScanner categoriesFor: #recordMatch:!private! !
!SmaCCScanner categoriesFor: #reportLastMatch!private! !
!SmaCCScanner categoriesFor: #resetOutputToLastMatch!private! !
!SmaCCScanner categoriesFor: #resetScanner!private! !
!SmaCCScanner categoriesFor: #scanForString:!private! !
!SmaCCScanner categoriesFor: #scanForToken!accessing!private! !
!SmaCCScanner categoriesFor: #scannerError!private! !
!SmaCCScanner categoriesFor: #setPosition:!accessing!private! !
!SmaCCScanner categoriesFor: #state!accessing!private! !
!SmaCCScanner categoriesFor: #state:!accessing!private! !
!SmaCCScanner categoriesFor: #step!private! !
!SmaCCScanner categoriesFor: #tokenActions!private!utility! !
!SmaCCScanner categoriesFor: #tokenClass!private! !
!SmaCCScanner categoriesFor: #whitespace!default token handling!private! !

!SmaCCScanner class methodsFor!

initialize
	self initializeKeywordMap!

initializeKeywordMap
	keywordMap := Dictionary new!

keywordMap
	keywordMap isNil ifTrue: [self initializeKeywordMap].
	^keywordMap!

needsLineNumbers
	"Redefine to return true, if you need line number information"

	^false!

new
	^(super new)
		initialize;
		yourself!

on: aStream 
	^(self new)
		on: (self needsLineNumbers 
					ifTrue: [SmaCCLineNumberStream on: aStream]
					ifFalse: [aStream]);
		yourself! !
!SmaCCScanner class categoriesFor: #initialize!class initialization!private! !
!SmaCCScanner class categoriesFor: #initializeKeywordMap!class initialization!private! !
!SmaCCScanner class categoriesFor: #keywordMap!accessing!private! !
!SmaCCScanner class categoriesFor: #needsLineNumbers!private!testing! !
!SmaCCScanner class categoriesFor: #new!instance creation!private! !
!SmaCCScanner class categoriesFor: #on:!instance creation!private! !

SmaCCToken guid: (GUID fromString: '{B2B52C93-EB4A-441F-BCFF-08C4EB02FDB0}')!
SmaCCToken comment: 'SmaCCTokens are used as the interface objects between scanner and parser. They hold the string that was scanned and its position information. Also, included in the token is its id. The id specifies what type of token it is.

Instance Variables:
	id	<Array of: Integer>	the list of possible token types this represents. There can be overlapping tokens, so we list all of the id here. The default parser only looks at the first id, but we can redefine this behavior in a subclass to look at all possibilities until we find a valid token.
	start	<Integer>	the starting position of the token in the original input
	value	<Object>	the value of our token (normally a string, but could be anything)

'!
!SmaCCToken categoriesForClass!Unclassified! !
!SmaCCToken methodsFor!

= aSmaCCToken 
	^self class = aSmaCCToken class and: [self value = aSmaCCToken value]!

copyInContext: aDictionary
	^self!

hash
	^self value hash!

id
	^ids!

ids
	^ids!

ids: anArray
	ids := anArray!

isListMatch
	^false!

isPattern
	^false!

match: anObject inContext: aDictionary
	^self = anObject!

printOn: aStream
	aStream
		nextPut: ${;
		nextPutAll: self value;
		nextPut: $(;
		nextPutAll: self startPosition printString;
		nextPut: $,;
		nextPutAll: self stopPosition printString;
		nextPut: $,;
		nextPutAll: self ids printString;
		nextPutAll: ')}'!

startPosition
	^start!

stopPosition
	start isNil ifTrue: [^nil].
	^start + value size - 1!

value
	^value!

value: aString start: startPositionInteger ids: aCollection
	value := aString.
	start := startPositionInteger.
	ids := aCollection! !
!SmaCCToken categoriesFor: #=!comparing!private! !
!SmaCCToken categoriesFor: #copyInContext:!matching!private! !
!SmaCCToken categoriesFor: #hash!comparing!private! !
!SmaCCToken categoriesFor: #id!public! !
!SmaCCToken categoriesFor: #ids!accessing!private! !
!SmaCCToken categoriesFor: #ids:!accessing!private! !
!SmaCCToken categoriesFor: #isListMatch!public!testing! !
!SmaCCToken categoriesFor: #isPattern!private!testing! !
!SmaCCToken categoriesFor: #match:inContext:!matching!public! !
!SmaCCToken categoriesFor: #printOn:!printing!private! !
!SmaCCToken categoriesFor: #startPosition!accessing!private! !
!SmaCCToken categoriesFor: #stopPosition!accessing!private! !
!SmaCCToken categoriesFor: #value!accessing!private! !
!SmaCCToken categoriesFor: #value:start:ids:!initialize-release!private! !

!SmaCCToken class methodsFor!

value: aString
	^self
		value: aString
		start: 0
		ids: #()!

value: aString start: anInteger id: anObject
	^self value: aString start: anInteger ids: anObject!

value: aString start: anInteger ids: aCollection
	^(self new)
		value: aString
			start: anInteger
			ids: aCollection;
		yourself! !
!SmaCCToken class categoriesFor: #value:!instance creation!private! !
!SmaCCToken class categoriesFor: #value:start:id:!public! !
!SmaCCToken class categoriesFor: #value:start:ids:!instance creation!private! !

SmaCCParserError guid: (GUID fromString: '{8650EA85-7A5B-4230-BDF4-FCD365F9353F}')!
SmaCCParserError comment: 'SmaCCParserError is the exception raised when a parsing error occurs. The description of the exception will be the error message and the parameter of the exception is the parser. With this information, you can insert a custom error message in your text view that you are parsing. For example, in VisualWorks, the following code will insert an error message into your text view:
	textController insertAndSelect: ex description , '' ->'' at: ex parameter position
'!
!SmaCCParserError categoriesForClass!Unclassified! !
SmaCCPatternNode guid: (GUID fromString: '{EDEEB44D-6E9B-4D01-8EF8-3AC487BE1D2B}')!
SmaCCPatternNode comment: 'SmaCCPatternNode is a parse node that matches other parse nodes when you are searching or rewriting parse trees.

Instance Variables:
	patternToken	<SmaCCPatternToken>	the token that holds our name and properties

'!
!SmaCCPatternNode categoriesForClass!Unclassified! !
!SmaCCPatternNode methodsFor!

= anObject
	^super = anObject and: [self patternToken = anObject patternToken]!

copyInContext: aDictionary
	^(aDictionary at: self ifAbsent: [nil]) copy!

hash
	^self patternToken hash!

isListMatch
	^patternToken isListMatch!

match: aSmaCCParseNode inContext: aDictionary
	^(aDictionary at: self ifAbsentPut: [aSmaCCParseNode]) = aSmaCCParseNode
		and: [patternToken testBlock value: aSmaCCParseNode]!

name
	^patternToken name!

patternToken
	^patternToken!

patternToken: anObject
	patternToken := anObject!

tokenVariables
	^#(#patternToken)! !
!SmaCCPatternNode categoriesFor: #=!comparing!private! !
!SmaCCPatternNode categoriesFor: #copyInContext:!matching!private! !
!SmaCCPatternNode categoriesFor: #hash!comparing!private! !
!SmaCCPatternNode categoriesFor: #isListMatch!private!testing! !
!SmaCCPatternNode categoriesFor: #match:inContext:!matching!private! !
!SmaCCPatternNode categoriesFor: #name!accessing!private! !
!SmaCCPatternNode categoriesFor: #patternToken!accessing!private! !
!SmaCCPatternNode categoriesFor: #patternToken:!accessing!private! !
!SmaCCPatternNode categoriesFor: #tokenVariables!private!variables! !

!SmaCCPatternNode class methodsFor!

onToken: aSmaCCPatternToken
	^(self new)
		patternToken: aSmaCCPatternToken;
		yourself! !
!SmaCCPatternNode class categoriesFor: #onToken:!instance creation!private! !

SmaCCCaseInsensitiveToken guid: (GUID fromString: '{4FCAB4AF-2483-48BD-A53A-02DF53D2305A}')!
SmaCCCaseInsensitiveToken comment: 'SmaCCCaseInsensitiveToken represents a case insensitive token (e.g., "IF" and "if" will compare equal).'!
!SmaCCCaseInsensitiveToken categoriesForClass!Unclassified! !
!SmaCCCaseInsensitiveToken methodsFor!

= aSmaCCToken
	^self class = aSmaCCToken class and: [self value sameAs: aSmaCCToken value]!

hash
	^self value asUppercase hash! !
!SmaCCCaseInsensitiveToken categoriesFor: #=!comparing!private! !
!SmaCCCaseInsensitiveToken categoriesFor: #hash!comparing!private! !

SmaCCPatternToken guid: (GUID fromString: '{717A9362-C851-44AB-9DFB-808E21EBE0B3}')!
SmaCCPatternToken comment: 'SmaCCPatternToken is a token for pattern nodes.

Instance Variables:
	isList	<Boolean>	do we match collections or single objects
	isNode	<Boolean>	can we match other parse nodes
	isToken	<Boolean>	can we match other tokens
	nodeClassName	<Symbol>	when matching a parse node, limit it to these types of nodes
	testBlock	<BlockClosure>	after a match, run some code that tests the match

'!
!SmaCCPatternToken categoriesForClass!Unclassified! !
!SmaCCPatternToken methodsFor!

= aSmaCCToken
	^self class = aSmaCCToken class and: [self name = aSmaCCToken name]!

beList
	isList := true!

beNode
	isNode := true.
	isToken := false!

beToken
	isToken := true.
	isNode := false!

beTokenOrNode
	isToken := true.
	isNode := true!

buildStructure
	| index code |
	isList := isToken := false.
	isNode := true.
	testBlock := [:node | true].
	index := value indexOf: ${.
	index > 0 ifFalse: [^self].
	code := value copyFrom: index + 1 to: (value lastIndexOf: $}) - 1.
	Compiler
		evaluate: 'self ' , code
		for: self
		logged: false!

copyInContext: aDictionary
	^aDictionary at: self!

hash
	^self class hash bitXor: self name hash!

isListMatch
	^isList!

isNode
	^isNode!

isPattern
	^true!

isToken
	^isToken!

match: aSmaCCToken inContext: aDictionary
	^(aSmaCCToken isKindOf: SmaCCToken)
		and: [(aDictionary at: self ifAbsentPut: [aSmaCCToken]) = aSmaCCToken and: [self testBlock value: aSmaCCToken]]!

name
	| resultStream readStream char |
	(value first isAlphaNumeric or: [value first == $_]) ifTrue: [^value].
	resultStream := WriteStream on: String new.
	readStream := ReadStream on: value.
	readStream next.
	[char := readStream next.
	char isLetter or: [char == $_]] whileFalse.
	[resultStream nextPut: char.
	char := readStream next.
	char isAlphaNumeric or: [char == $_]] whileTrue.
	^resultStream contents!

nodeClassName
	^nodeClassName!

nodeClassName: aString
	nodeClassName := aString!

testBlock
	^[:node | 
	(nodeClassName isNil or: [node isKindOf: (node class environment at: self nodeClassName ifAbsent: [SmaCCParseNode])])
		and: [testBlock value: node]]!

testBlock: aBlock
	testBlock := aBlock!

value: aString start: startPositionInteger ids: aCollection
	super value: aString start: startPositionInteger ids: aCollection.
	self buildStructure! !
!SmaCCPatternToken categoriesFor: #=!comparing!public! !
!SmaCCPatternToken categoriesFor: #beList!accessing!private! !
!SmaCCPatternToken categoriesFor: #beNode!accessing!private! !
!SmaCCPatternToken categoriesFor: #beToken!accessing!private! !
!SmaCCPatternToken categoriesFor: #beTokenOrNode!accessing!private! !
!SmaCCPatternToken categoriesFor: #buildStructure!private! !
!SmaCCPatternToken categoriesFor: #copyInContext:!matching!private! !
!SmaCCPatternToken categoriesFor: #hash!comparing!public! !
!SmaCCPatternToken categoriesFor: #isListMatch!private!testing! !
!SmaCCPatternToken categoriesFor: #isNode!private!testing! !
!SmaCCPatternToken categoriesFor: #isPattern!private!testing! !
!SmaCCPatternToken categoriesFor: #isToken!private!testing! !
!SmaCCPatternToken categoriesFor: #match:inContext:!matching!private! !
!SmaCCPatternToken categoriesFor: #name!accessing!private! !
!SmaCCPatternToken categoriesFor: #nodeClassName!accessing!private! !
!SmaCCPatternToken categoriesFor: #nodeClassName:!accessing!public! !
!SmaCCPatternToken categoriesFor: #testBlock!accessing!private! !
!SmaCCPatternToken categoriesFor: #testBlock:!accessing!private! !
!SmaCCPatternToken categoriesFor: #value:start:ids:!initialize-release!private! !

SmaCCLineNumberStream guid: (GUID fromString: '{B6C249B5-5E92-49A5-805D-6D3BC7901CE8}')!
SmaCCLineNumberStream comment: 'SmaCCLineNumberStream is a wrapper for streams that calculates line numbers.

Instance Variables:
	eolPositions	<OrderedCollection of: Integer>	the positions of each end of line
	lastPosition	<Integer>	the position of the last character that we have calculated the end of line information for (we know the line number for all characters before this position and don''t know anything about the characters after this position)
	previousWasCR	<Boolean>	was the previous character a CR. This is used for CR LF streams. A CR LF combination should only increment the line counter by 1
	sourceStream	<Stream>	the stream that we are wrapping
'!
!SmaCCLineNumberStream categoriesForClass!Unclassified! !
!SmaCCLineNumberStream methodsFor!

atEnd
	^sourceStream atEnd!

columnNumber
	^self columnNumberFor: self position!

columnNumberFor: anInteger 
	^anInteger - (eolPositions at: (self lineNumberFor: anInteger)) + 1!

contents
	^sourceStream contents!

contentsSpecies
	^sourceStream contentsSpecies!

doesNotUnderstand: aMessage 
	^[sourceStream perform: aMessage selector withArguments: aMessage arguments] 
		ensure: [self position: sourceStream position]!

flush
	^sourceStream flush!

isReadable
	^sourceStream isReadable!

isWritable
	^sourceStream isWritable!

lineNumber
	^self lineNumberFor: self position!

lineNumberFor: anInteger
	| index start stop pos |
	pos := anInteger.
	pos >= eolPositions last ifTrue: [^eolPositions size].
	start := 1.
	stop := eolPositions size.
	[start + 1 < stop] whileTrue: 
			[index := (start + stop) // 2.
			(eolPositions at: index) <= pos 
				ifTrue: [start := index]
				ifFalse: [stop := index]].
	^start!

next
	| character streamPosition |
	character := sourceStream next.
	streamPosition := sourceStream position.
	streamPosition - 1 >= lastPosition
		ifTrue:
			[lastPosition := streamPosition.
			character == Character cr
				ifTrue:
					[eolPositions add: lastPosition.
					previousWasCR := true]
				ifFalse:
					[character isLinefeed
						ifTrue:
							[previousWasCR ifTrue: [eolPositions at: eolPositions size put: lastPosition] ifFalse: [eolPositions add: lastPosition]].
					previousWasCR := false]].
	^character!

nextPut: anObject 
	^sourceStream nextPut: anObject!

on: aReadStream 
	sourceStream := aReadStream.
	eolPositions := OrderedCollection with: aReadStream position.
	lastPosition := aReadStream position.
	previousWasCR := false!

peek
	^sourceStream peek!

position
	^sourceStream position!

position: anInteger 
	anInteger > lastPosition 
		ifTrue: 
			[sourceStream position: lastPosition.
			[sourceStream position < anInteger and: [sourceStream atEnd not]] 
				whileTrue: [self next]]
		ifFalse: [sourceStream position: anInteger]!

skip: anInteger
	^self position: self position + anInteger! !
!SmaCCLineNumberStream categoriesFor: #atEnd!private!testing! !
!SmaCCLineNumberStream categoriesFor: #columnNumber!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #columnNumberFor:!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #contents!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #contentsSpecies!private! !
!SmaCCLineNumberStream categoriesFor: #doesNotUnderstand:!error handling!public! !
!SmaCCLineNumberStream categoriesFor: #flush!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #isReadable!private!testing! !
!SmaCCLineNumberStream categoriesFor: #isWritable!private!testing! !
!SmaCCLineNumberStream categoriesFor: #lineNumber!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #lineNumberFor:!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #next!accessing!public! !
!SmaCCLineNumberStream categoriesFor: #nextPut:!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #on:!initialize-release!private! !
!SmaCCLineNumberStream categoriesFor: #peek!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #position!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #position:!accessing!private! !
!SmaCCLineNumberStream categoriesFor: #skip:!accessing!private! !

!SmaCCLineNumberStream class methodsFor!

on: aReadStream 
	^(self basicNew)
		on: aReadStream;
		yourself! !
!SmaCCLineNumberStream class categoriesFor: #on:!instance creation!private! !

"Binary Globals"!

