| package |
package := Package name: 'SmaCC Smalltalk Parser'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains an example Smalltalk parser.

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #StParser;
	add: #StScanner;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\Refactoring Browser\Parser\RBParser';
	add: '..\SmaCC Runtime\SmaCC Runtime';
	yourself).

package!

"Class Definitions"!

SmaCCParser subclass: #StParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCScanner subclass: #StScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

StParser guid: (GUID fromString: '{7E634C15-2500-4B1C-A9FB-6836BCD7854F}')!
StParser comment: ''!
!StParser categoriesForClass!Unclassified! !
!StParser methodsFor!

actionForCurrentToken
	| ids action |
	ids := currentToken ids.
	1
		to: ids size
		do:
			[:i | 
			action := self actionFor: (ids at: i).
			(action bitAnd: self actionMask) = self errorAction ifFalse: [^action].
			(ids at: i) = scanner negativeNumberId
				ifTrue:
					[currentToken := SmaCCToken
						value: '-'
						start: currentToken startPosition
						ids: (Array with: scanner binarySymbolId).
					self position: currentToken stopPosition - 1.
					^self actionForCurrentToken]].
	^self errorAction!

createMessageNodeFrom: aCollection 
	^RBMessageNode 
		receiver: aCollection first
		selector: (aCollection at: 2) first contents asSymbol
		arguments: (aCollection at: 2) last!

reduceActionForArray1: nodes
	^WriteStream on: Array new!

reduceActionForArray2: nodes
	^(nodes at: 1)
		nextPut: (nodes at: 2);
		yourself!

reduceActionForArray3: nodes
	(nodes at: 2) stopPosition + 1 = (nodes at: 3) startPosition
		ifTrue: [(nodes at: 1) nextPut: ((nodes at: 2) value , ':') asSymbol]
		ifFalse:
			[(nodes at: 1)
				nextPut: (nodes at: 2) value asSymbol;
				nextPut: #':'].
	self position: (nodes at: 3) startPosition.
	currentToken := nil.
	^nodes at: 1!

reduceActionForArrayLiteral2: nodes
	^(nodes at: 1) value asSymbol!

reduceActionForArrayLiteral6: nodes
	^(nodes at: 2) value asQualifiedReference!

reduceActionForArrayLiteral8: nodes
	^(nodes at: 2) contents!

reduceActionForBinaryMessage1: nodes
	^Array with: (ReadStream on: (nodes at: 1) value) with: (Array with: (nodes at: 2))!

reduceActionForBinaryMethodName1: nodes
	^Array with: (nodes at: 1) value asSymbol with: (Array with: (nodes at: 2))!

reduceActionForBlock1: nodes
	^RBBlockNode arguments: (nodes at: 2) body: (nodes at: 4)!

reduceActionForBlock2: nodes
	^RBBlockNode body: (nodes at: 2)!

reduceActionForBlock3: nodes
	^RBBlockNode arguments: (nodes at: 2) body: (RBSequenceNode statements: #())!

reduceActionForBlock4: nodes
	^RBBlockNode arguments: (nodes at: 2) body: (RBSequenceNode temporaries: (nodes at: 4) statements: (nodes at: 6))!

reduceActionForBlockArgs1: nodes
	^OrderedCollection with: (nodes at: 2)!

reduceActionForByteArray1: nodes
	^WriteStream on: ByteArray new!

reduceActionForByteArray2: nodes
	^(nodes at: 1)
		nextPut: (nodes at: 2) value;
		yourself!

reduceActionForCascade1: nodes
	| messages |
	(nodes at: 2) isEmpty ifTrue: [^nodes at: 1].
	messages := OrderedCollection new: (nodes at: 2) size + 1.
	messages add: (nodes at: 1).
	(nodes at: 2)
		do:
			[:each | 
			messages
				add:
					(RBMessageNode
						receiver: (nodes at: 1) receiver
						selector: each first contents asSymbol
						arguments: each last)].
	^RBCascadeNode messages: messages!

reduceActionForExpression1: nodes
	^RBAssignmentNode variable: (nodes at: 1) value: (nodes at: 3)!

reduceActionForKeywordMethodName1: nodes
	| stream |
	stream := WriteStream on: (String new: 30).
	stream nextPutAll: (nodes at: 1) value.
	^Array with: stream with: (OrderedCollection with: (nodes at: 2))!

reduceActionForKeywordMethodName2: nodes
	(nodes at: 1) first nextPutAll: (nodes at: 2) value.
	(nodes at: 1) last add: (nodes at: 3).
	^nodes at: 1!

reduceActionForLiteral1: nodes
	^true!

reduceActionForLiteral14: nodes
	^(nodes at: 3) contents!

reduceActionForLiteral15: nodes
	^(nodes at: 3) value asQualifiedReference!

reduceActionForLiteral2: nodes
	^false!

reduceActionForLiteral4: nodes
	^(nodes at: 1) value!

reduceActionForLiteral6: nodes
	^(nodes at: 1) value at: 2!

reduceActionForLiteral7: nodes
	^((nodes at: 1) value copyFrom: 2 to: (nodes at: 1) value size - 1) copyReplaceAll: '''''' with: ''''!

reduceActionForLiteral8: nodes
	^(((nodes at: 2) value copyFrom: 2 to: (nodes at: 2) value size - 1) copyReplaceAll: '''''' with: '''') asSymbol!

reduceActionForLiteral9: nodes
	^(nodes at: 2) value asSymbol!

reduceActionForMethod1: nodes
	^(RBMethodNode
		selector: (nodes at: 1) value asSymbol
		arguments: #()
		body: (nodes at: 2) last) yourself!

reduceActionForMethod2: nodes
	^(RBMethodNode
		selector: (nodes at: 1) first
		arguments: (nodes at: 1) last
		body: (nodes at: 2) last) yourself!

reduceActionForMethod3: nodes
	^(RBMethodNode
		selector: (nodes at: 1) first contents asSymbol
		arguments: (nodes at: 1) last
		body: (nodes at: 2) last) yourself!

reduceActionForMethodSequenceNode1: nodes
	^Array
		with:
			((nodes at: 1)
				addAll: (nodes at: 5);
				yourself)
		with: (RBSequenceNode temporaries: (nodes at: 3) statements: (nodes at: 6))!

reduceActionForMethodSequenceNode2: nodes
	^Array
		with:
			((nodes at: 1)
				addAll: (nodes at: 3);
				yourself)
		with: (RBSequenceNode temporaries: #() statements: (nodes at: 4))!

reduceActionForMethodSequenceNode3: nodes
	^Array with: (nodes at: 1) with: (RBSequenceNode temporaries: #() statements: (nodes at: 2))!

reduceActionForPrimary2: nodes
	^RBLiteralNode value: (nodes at: 1)!

reduceActionForPrimitives1: nodes
	^OrderedCollection new!

reduceActionForPrimitives2: nodes
	^(nodes at: 1)
		add: ((nodes at: 2) startPosition to: (nodes at: 4) startPosition);
		yourself!

reduceActionForSequenceNode1: nodes
	^RBSequenceNode temporaries: (nodes at: 1) statements: (nodes at: 2)!

reduceActionForStatementList2: nodes
	^(nodes at: 1)
		add: (nodes at: 3);
		yourself!

reduceActionForStatements3: nodes
	^(nodes at: 1)
		add: (RBReturnNode value: (nodes at: 4));
		yourself!

reduceActionForStatements4: nodes
	^OrderedCollection with: (RBReturnNode value: (nodes at: 2))!

reduceActionForTemporaries1: nodes
	^#()!

reduceActionForTemporaryVariables2: nodes
	^(nodes at: 1)
		add: (nodes at: 2);
		yourself!

reduceActionForUnaryMessage1: nodes
	^Array with: (ReadStream on: (nodes at: 1) value) with: #()!

reduceActionForVariable1: nodes
	^RBVariableNode named: (nodes at: 1) value!

reduceTable
^#(
	#(45 0 #reduceActionForTemporaries1:) 
	#(46 0 #reduceActionForPrimitives1:) 
	#(45 1 #reduceActionForTemporaries1:) 
	#(35 0 #reduceActionForPrimitives1:) 
	#(36 0 #reduceActionForPrimitives1:) 
	#(42 2 #reduceActionForMethod1:) 
	#(48 1 #reduceActionForVariable1:) 
	#(34 2 #reduceActionForKeywordMethodName1:) 
	#(58 2 #reduceActionForBinaryMethodName1:) 
	#(42 2 #reduceActionForMethod3:) 
	#(42 2 #reduceActionForMethod2:) 
	#(67 1 #nil) 
	#(67 1 #reduceActionForLiteral1:) 
	#(67 1 #reduceActionForLiteral2:) 
	#(67 1 #reduceActionForLiteral4:) 
	#(67 1 #reduceActionForLiteral7:) 
	#(67 1 #reduceActionForLiteral6:) 
	#(62 1 #liftFirstValue:) 
	#(66 2 #reduceActionForSequenceNode1:) 
	#(37 0 #reduceFor:) 
	#(38 1 #reduceFor:) 
	#(43 0 #reduceActionForPrimitives1:) 
	#(44 1 #liftFirstValue:) 
	#(40 1 #liftFirstValue:) 
	#(56 1 #liftFirstValue:) 
	#(62 1 #reduceActionForPrimary2:) 
	#(39 0 #reduceFor:) 
	#(59 2 #reduceActionForMethodSequenceNode3:) 
	#(34 3 #reduceActionForKeywordMethodName2:) 
	#(45 3 #liftSecondValue:) 
	#(35 2 #reduceActionForTemporaryVariables2:) 
	#(61 0 #reduceActionForArray1:) 
	#(60 0 #reduceActionForByteArray1:) 
	#(67 2 #reduceActionForLiteral9:) 
	#(67 2 #reduceActionForLiteral8:) 
	#(37 1 #reduceFor:) 
	#(36 2 #liftFirstValue:) 
	#(56 2 #reduceActionForCascade1:) 
	#(47 2 #createMessageNodeFrom:) 
	#(65 2 #createMessageNodeFrom:) 
	#(51 1 #reduceActionForUnaryMessage1:) 
	#(49 2 #createMessageNodeFrom:) 
	#(62 3 #liftSecondValue:) 
	#(36 3 #reduceActionForStatements4:) 
	#(41 2 #reduceActionForBlockArgs1:) 
	#(33 3 #reduceActionForBlock3:) 
	#(33 3 #reduceActionForBlock2:) 
	#(38 3 #reduceActionForStatementList2:) 
	#(54 1 #liftFirstValue:) 
	#(53 2 #reduceActionForKeywordMethodName1:) 
	#(55 1 #liftFirstValue:) 
	#(52 2 #reduceActionForBinaryMessage1:) 
	#(40 3 #reduceActionForExpression1:) 
	#(39 2 #nil) 
	#(46 4 #reduceActionForPrimitives2:) 
	#(59 4 #reduceActionForMethodSequenceNode2:) 
	#(67 4 #reduceActionForLiteral15:) 
	#(67 4 #reduceActionForLiteral14:) 
	#(57 1 #reduceActionForArrayLiteral2:) 
	#(57 1 #reduceActionForLiteral4:) 
	#(61 2 #reduceActionForArray2:) 
	#(57 1 #liftFirstValue:) 
	#(60 2 #reduceActionForByteArray2:) 
	#(41 3 #reduceActionForStatementList2:) 
	#(43 3 #reduceActionForStatementList2:) 
	#(50 1 #liftFirstValue:) 
	#(53 3 #reduceActionForKeywordMethodName2:) 
	#(61 3 #reduceActionForArray3:) 
	#(33 5 #reduceActionForBlock1:) 
	#(36 5 #reduceActionForStatements3:) 
	#(59 6 #reduceActionForMethodSequenceNode1:) 
	#(57 3 #reduceActionForArrayLiteral6:) 
	#(57 3 #reduceActionForArrayLiteral8:) 
	#(33 7 #reduceActionForBlock4:)
	).!

transitionTable
^#(
#[1 0 13 0 20 0 17 0 24 0 21 0 26 0 25 0 34 0 29 0 42 0 33 0 58] 
#[1 0 6 0 2 0 6 0 3 0 6 0 10 0 6 0 11 0 37 0 12 0 6 0 13 0 41 0 15 0 6 0 17 0 6 0 18 0 6 0 20 0 6 0 21 0 6 0 22 0 6 0 23 0 6 0 29 0 6 0 30 0 45 0 45 0 6 0 64 0 49 0 66] 
#[1 0 10 0 2 0 10 0 3 0 10 0 7 0 10 0 10 0 10 0 11 0 10 0 12 0 10 0 13 0 10 0 15 0 10 0 17 0 10 0 18 0 10 0 20 0 10 0 21 0 10 0 22 0 10 0 23 0 10 0 29 0 10 0 30 0 53 0 46 0 57 0 59 0 10 0 64] 
#[1 0 61 0 20 0 65 0 29 0 69 0 48] 
#[1 0 61 0 20 0 65 0 29 0 73 0 48] 
#[1 0 10 0 2 0 10 0 3 0 10 0 7 0 10 0 10 0 10 0 11 0 10 0 12 0 10 0 13 0 10 0 15 0 10 0 17 0 10 0 18 0 10 0 20 0 10 0 21 0 10 0 22 0 10 0 23 0 77 0 24 0 10 0 29 0 10 0 30 0 53 0 46 0 81 0 59 0 10 0 64] 
#[0 0 0 0 64] 
#[1 0 10 0 2 0 10 0 3 0 10 0 7 0 10 0 10 0 10 0 11 0 10 0 12 0 10 0 13 0 10 0 15 0 10 0 17 0 10 0 18 0 10 0 20 0 10 0 21 0 10 0 22 0 10 0 23 0 10 0 29 0 10 0 30 0 53 0 46 0 85 0 59 0 10 0 64] 
#[0 0 14 0 2 0 3 0 10 0 11 0 13 0 16 0 17 0 18 0 20 0 21 0 22 0 23 0 29 0 30 0 64] 
#[1 0 18 0 15 0 18 0 20 0 18 0 29 0 89 0 35] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 22 0 16 0 113 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 0 141 0 36 0 145 0 38 0 149 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 22 0 64 0 177 0 65 0 181 0 67] 
#[0 0 0 0 64] 
#[1 0 93 0 2 0 97 0 3 0 185 0 7 0 101 0 10 0 105 0 11 0 189 0 12 0 109 0 13 0 193 0 15 0 113 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 0 197 0 36 0 145 0 38 0 149 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 22 0 64 0 177 0 65 0 181 0 67] 
#[0 0 26 0 64] 
#[0 0 30 0 2 0 3 0 4 0 5 0 6 0 7 0 10 0 11 0 12 0 13 0 14 0 15 0 16 0 17 0 18 0 19 0 20 0 21 0 22 0 23 0 24 0 26 0 29 0 30 0 64] 
#[0 0 30 0 2 0 3 0 4 0 5 0 6 0 7 0 10 0 11 0 12 0 13 0 14 0 15 0 16 0 17 0 18 0 19 0 20 0 21 0 22 0 23 0 24 0 26 0 29 0 30 0 64] 
#[0 0 34 0 2 0 3 0 7 0 10 0 11 0 12 0 13 0 15 0 17 0 18 0 20 0 21 0 22 0 23 0 24 0 29 0 30 0 64] 
#[0 0 38 0 2 0 3 0 7 0 10 0 11 0 12 0 13 0 15 0 17 0 18 0 20 0 21 0 22 0 23 0 29 0 30 0 64] 
#[1 0 61 0 20 0 65 0 29 0 201 0 48] 
#[0 0 42 0 64] 
#[0 0 46 0 64] 
#[1 0 205 0 15 0 61 0 20 0 65 0 29 0 209 0 48] 
#[1 0 213 0 1 0 217 0 3 0 221 0 18 0 225 0 20 0 229 0 23 0 233 0 24 0 237 0 25 0 241 0 26] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 0 245 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 177 0 65 0 181 0 67] 
#[0 0 50 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 54 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 58 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 0 249 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 177 0 65 0 181 0 67] 
#[1 0 6 0 2 0 6 0 3 0 6 0 10 0 6 0 11 0 37 0 12 0 6 0 13 0 41 0 15 0 6 0 16 0 6 0 17 0 6 0 18 0 253 0 19 0 6 0 20 0 6 0 21 0 6 0 22 0 6 0 23 0 6 0 29 0 6 0 30 1 1 0 41 0 45 0 45 1 5 0 66] 
#[0 0 62 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 62 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 66 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 70 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 74 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 0 78 0 16 0 64] 
#[1 1 9 0 5 0 82 0 16 1 13 0 37 0 82 0 64] 
#[0 0 86 0 5 0 16 0 64] 
#[1 0 90 0 4 0 90 0 5 0 90 0 6 0 90 0 16 1 17 0 43 0 90 0 64] 
#[1 0 94 0 4 0 94 0 5 0 94 0 6 0 94 0 16 1 21 0 24 1 25 0 26 1 29 0 52 1 33 0 53 0 94 0 64] 
#[1 0 74 0 4 0 74 0 5 1 37 0 14 0 74 0 16 0 74 0 20 0 74 0 24 0 74 0 26 0 74 0 64] 
#[1 0 94 0 4 0 94 0 5 0 94 0 6 0 94 0 16 1 41 0 20 1 21 0 24 1 25 0 26 1 45 0 51 1 49 0 52 1 53 0 53 0 94 0 64] 
#[0 0 98 0 4 0 5 0 16 0 64] 
#[1 0 102 0 4 0 102 0 5 0 102 0 16 1 41 0 20 1 21 0 24 1 25 0 26 1 57 0 51 1 61 0 52 1 65 0 53 0 102 0 64] 
#[0 0 94 0 4 0 5 0 6 0 16 0 64] 
#[0 0 106 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[1 0 110 0 1 0 110 0 2 0 110 0 3 0 110 0 4 0 110 0 5 0 110 0 6 0 110 0 8 0 110 0 9 0 110 0 10 0 110 0 11 0 110 0 13 0 110 0 20 0 110 0 21 0 110 0 22 0 110 0 23 0 110 0 24 0 110 0 26 0 110 0 30 1 69 0 39] 
#[1 0 10 0 2 0 10 0 3 0 10 0 7 0 10 0 10 0 10 0 11 0 10 0 13 0 10 0 17 0 10 0 18 0 10 0 20 0 10 0 21 0 10 0 22 0 10 0 23 0 10 0 29 0 10 0 30 1 73 0 46 0 10 0 64] 
#[1 0 18 0 15 0 18 0 20 0 18 0 29 1 77 0 35] 
#[0 0 114 0 64] 
#[0 0 118 0 2 0 3 0 7 0 10 0 11 0 12 0 13 0 15 0 17 0 18 0 20 0 21 0 22 0 23 0 24 0 29 0 30 0 64] 
#[0 0 122 0 2 0 3 0 10 0 11 0 13 0 16 0 17 0 18 0 20 0 21 0 22 0 23 0 29 0 30 0 64] 
#[0 0 126 0 15 0 20 0 29] 
#[1 1 81 0 20 1 85 0 29] 
#[1 0 130 0 1 0 130 0 2 0 130 0 3 0 130 0 4 0 130 0 10 0 130 0 11 0 130 0 13 0 130 0 18 0 130 0 20 0 130 0 21 0 130 0 22 0 130 0 23 0 130 0 24 0 130 0 25 0 130 0 26 0 130 0 29 0 130 0 30 0 130 0 32 1 89 0 61] 
#[1 0 134 0 16 0 134 0 21 1 93 0 60] 
#[0 0 138 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 142 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 138 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 138 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 138 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 1 97 0 4] 
#[1 1 101 0 5 0 82 0 16 1 105 0 37 0 82 0 64] 
#[1 0 61 0 20 0 65 0 29 1 109 0 48] 
#[1 1 113 0 12 1 117 0 15 1 121 0 16 1 125 0 19] 
#[0 1 129 0 16] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 146 0 16 1 133 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 137 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 146 0 64 0 177 0 65 0 181 0 67] 
#[0 0 150 0 16 0 64] 
#[1 0 154 0 4 0 154 0 5 1 141 0 6 0 154 0 16 0 154 0 64] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 145 0 47 1 149 0 48 1 153 0 49 1 157 0 54 1 161 0 62 0 181 0 67] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 149 0 48 1 165 0 49 1 169 0 55 1 173 0 62 0 181 0 67] 
#[0 0 158 0 4 0 5 0 6 0 16 0 24 0 26 0 64] 
#[1 0 162 0 4 0 162 0 5 0 162 0 6 0 162 0 16 1 177 0 24 0 162 0 64] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 181 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 177 0 65 0 181 0 67] 
#[0 0 166 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 0 170 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 0 158 0 4 0 5 0 6 0 16 0 24 0 26 0 64] 
#[1 0 162 0 4 0 162 0 5 0 162 0 6 0 162 0 16 1 177 0 24 0 162 0 64] 
#[0 0 170 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 0 158 0 4 0 5 0 6 0 16 0 24 0 26 0 64] 
#[1 0 162 0 4 0 162 0 5 0 162 0 6 0 162 0 16 1 177 0 24 0 162 0 64] 
#[1 1 185 0 1 0 93 0 2 1 189 0 3 1 193 0 4 1 197 0 5 1 201 0 6 1 205 0 8 1 209 0 9 0 101 0 10 0 105 0 11 0 109 0 13 1 213 0 20 0 121 0 21 0 125 0 22 0 129 0 23 1 217 0 24 1 221 0 26 0 133 0 30 1 225 0 67] 
#[1 0 93 0 2 0 97 0 3 0 185 0 7 0 101 0 10 0 105 0 11 0 109 0 13 0 113 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 229 0 36 0 145 0 38 0 149 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 22 0 64 0 177 0 65 0 181 0 67] 
#[1 1 233 0 15 0 61 0 20 0 65 0 29 0 209 0 48] 
#[0 1 237 0 9] 
#[0 1 241 0 9] 
#[1 1 245 0 1 0 93 0 2 1 249 0 3 1 253 0 4 0 101 0 10 0 105 0 11 0 109 0 13 2 1 0 18 2 5 0 20 0 121 0 21 0 125 0 22 0 129 0 23 2 9 0 24 2 13 0 25 2 17 0 26 2 21 0 29 0 133 0 30 2 25 0 32 2 29 0 57 2 33 0 67] 
#[1 2 37 0 16 2 41 0 21] 
#[0 0 174 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 0 146 0 16 0 64] 
#[0 0 178 0 16 0 64] 
#[0 0 182 0 12 0 15 0 16 0 19] 
#[1 0 18 0 15 0 18 0 20 0 18 0 29 2 45 0 35] 
#[1 0 6 0 2 0 6 0 3 0 6 0 10 0 6 0 11 0 37 0 12 0 6 0 13 0 41 0 15 0 6 0 16 0 6 0 17 0 6 0 18 0 6 0 20 0 6 0 21 0 6 0 22 0 6 0 23 0 6 0 29 0 6 0 30 0 45 0 45 2 49 0 66] 
#[0 0 186 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[1 0 61 0 20 0 65 0 29 2 53 0 48] 
#[0 0 190 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 2 57 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 177 0 65 0 181 0 67] 
#[0 0 194 0 5 0 16 0 64] 
#[1 1 41 0 20 1 21 0 24 1 25 0 26 2 61 0 50 2 65 0 51 2 69 0 52 2 73 0 53] 
#[1 0 198 0 4 0 198 0 5 0 198 0 6 0 198 0 16 0 198 0 24 1 25 0 26 1 29 0 52 0 198 0 64] 
#[0 0 74 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[1 0 198 0 4 0 198 0 5 0 198 0 6 0 198 0 16 1 41 0 20 0 198 0 24 1 25 0 26 1 45 0 51 1 49 0 52 0 198 0 64] 
#[0 0 202 0 4 0 5 0 6 0 16 0 24 0 64] 
#[1 0 198 0 4 0 198 0 5 0 198 0 6 0 198 0 16 1 41 0 20 0 198 0 24 1 25 0 26 1 57 0 51 1 61 0 52 0 198 0 64] 
#[1 0 206 0 4 0 206 0 5 0 206 0 6 0 206 0 16 1 41 0 20 0 206 0 24 0 206 0 26 1 45 0 51 0 206 0 64] 
#[0 0 210 0 4 0 5 0 6 0 16 0 24 0 26 0 64] 
#[1 0 206 0 4 0 206 0 5 0 206 0 6 0 206 0 16 1 41 0 20 0 206 0 24 0 206 0 26 1 57 0 51 0 206 0 64] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 1 145 0 47 1 149 0 48 1 153 0 49 2 77 0 54 1 161 0 62 0 181 0 67] 
#[0 0 214 0 4 0 5 0 16 0 64] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 222 0 2 0 3 0 7 0 10 0 11 0 12 0 13 0 15 0 17 0 18 0 20 0 21 0 22 0 23 0 29 0 30 0 64] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 218 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 20 0 21 0 22 0 23 0 24 0 26 0 30] 
#[0 0 226 0 64] 
#[1 0 10 0 2 0 10 0 3 0 10 0 7 0 10 0 10 0 10 0 11 0 10 0 13 0 10 0 17 0 10 0 18 0 10 0 20 0 10 0 21 0 10 0 22 0 10 0 23 0 10 0 29 0 10 0 30 2 81 0 46 0 10 0 64] 
#[0 0 230 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 230 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[1 2 85 0 20 2 89 0 29] 
#[1 0 130 0 1 0 130 0 2 0 130 0 3 0 130 0 4 0 130 0 10 0 130 0 11 0 130 0 13 0 130 0 18 0 130 0 20 0 130 0 21 0 130 0 22 0 130 0 23 0 130 0 24 0 130 0 25 0 130 0 26 0 130 0 29 0 130 0 30 0 130 0 32 2 93 0 61] 
#[0 0 234 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[1 0 134 0 16 0 134 0 21 2 97 0 60] 
#[1 0 238 0 1 0 238 0 2 0 238 0 3 0 238 0 4 0 238 0 10 0 238 0 11 0 238 0 13 2 101 0 14 0 238 0 18 0 238 0 20 0 238 0 21 0 238 0 22 0 238 0 23 0 238 0 24 0 238 0 25 0 238 0 26 0 238 0 29 0 238 0 30 0 238 0 32] 
#[0 0 238 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 238 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 238 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 242 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 238 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 246 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 250 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 0 234 0 1 0 2 0 3 0 4 0 5 0 6 0 8 0 9 0 10 0 11 0 13 0 16 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32 0 64] 
#[0 0 254 0 16 0 21] 
#[1 2 105 0 15 0 61 0 20 0 65 0 29 0 209 0 48] 
#[0 2 109 0 16] 
#[0 1 2 0 12 0 15 0 16 0 19] 
#[1 1 101 0 5 0 82 0 16 2 113 0 37 0 82 0 64] 
#[0 1 6 0 4 0 5 0 6 0 16 0 64] 
#[0 1 10 0 4 0 5 0 6 0 16 0 64] 
#[0 1 10 0 4 0 5 0 6 0 16 0 64] 
#[1 1 10 0 4 1 10 0 5 1 10 0 6 1 10 0 16 1 177 0 24 1 10 0 64] 
#[0 1 14 0 4 0 5 0 6 0 16 0 24 0 64] 
#[1 0 93 0 2 0 97 0 3 0 185 0 7 0 101 0 10 0 105 0 11 0 109 0 13 0 113 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 2 117 0 36 0 145 0 38 0 149 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 22 0 64 0 177 0 65 0 181 0 67] 
#[0 2 121 0 9] 
#[0 2 125 0 9] 
#[1 1 245 0 1 0 93 0 2 1 249 0 3 2 129 0 4 0 101 0 10 0 105 0 11 0 109 0 13 2 1 0 18 2 5 0 20 0 121 0 21 0 125 0 22 0 129 0 23 2 9 0 24 2 13 0 25 2 17 0 26 2 21 0 29 0 133 0 30 2 25 0 32 2 29 0 57 2 33 0 67] 
#[1 2 133 0 16 2 41 0 21] 
#[0 1 18 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[1 0 93 0 2 0 97 0 3 0 101 0 10 0 105 0 11 0 109 0 13 0 22 0 16 0 113 0 17 0 117 0 18 0 61 0 20 0 121 0 21 0 125 0 22 0 129 0 23 0 65 0 29 0 133 0 30 0 137 0 33 2 137 0 36 0 145 0 38 0 149 0 40 0 153 0 44 0 157 0 47 0 161 0 48 0 165 0 49 0 169 0 56 0 173 0 62 0 177 0 65 0 181 0 67] 
#[0 1 22 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64] 
#[0 1 26 0 16 0 64] 
#[0 1 30 0 64] 
#[0 1 34 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 1 34 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 1 38 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 1 38 0 1 0 2 0 3 0 4 0 10 0 11 0 13 0 18 0 20 0 21 0 22 0 23 0 24 0 25 0 26 0 29 0 30 0 32] 
#[0 2 141 0 16] 
#[0 1 42 0 4 0 5 0 6 0 16 0 20 0 24 0 26 0 64]
	).! !
!StParser categoriesFor: #actionForCurrentToken!private! !
!StParser categoriesFor: #createMessageNodeFrom:!private! !
!StParser categoriesFor: #reduceActionForArray1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForArray2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForArray3:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForArrayLiteral2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForArrayLiteral6:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForArrayLiteral8:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBinaryMessage1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBinaryMethodName1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBlock1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBlock2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBlock3:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBlock4:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForBlockArgs1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForByteArray1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForByteArray2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForCascade1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForExpression1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForKeywordMethodName1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForKeywordMethodName2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral14:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral15:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral4:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral6:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral7:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral8:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForLiteral9:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethod1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethod2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethod3:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethodSequenceNode1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethodSequenceNode2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForMethodSequenceNode3:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForPrimary2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForPrimitives1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForPrimitives2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForSequenceNode1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForStatementList2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForStatements3:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForStatements4:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForTemporaries1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForTemporaryVariables2:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForUnaryMessage1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceActionForVariable1:!generated-reduction actions!public! !
!StParser categoriesFor: #reduceTable!generated!public! !
!StParser categoriesFor: #transitionTable!generated!public! !

!StParser class methodsFor!

definitionComment
"<name>
	: (<isLetter> | \_) (<isLetter> | [\d\_]) *
	;
<number>
	: \d
	;
<negativeNumber>
	: \- <number>
	;
<string>
	: \'  [^\']* \' (\' [^\']* \')*
	;
<keyword>
	: <name> \:
	;
<multikeyword>
	: <name> \: (<name> \: )+
	;
<binarySymbol>
	: [\~\-\!!\@\%\&\*\+\=\\\|\?\/\>\<\,] [\~\!!\@\%\&\*\+\=\\\|\?\/\>\<\,]?
	;
<whitespace>
	: <isSeparator>+
	;
<comment>
	: \"" [^\""]* \""
	;
<dottedName>
	: <name> (\. <name>)+
	;
<character>
	: \$ .
	;
<variableAssignment>
	: <name> \: \=
	;
<anyChar>
	:  .
	; # For VW literal arrays that handle #(;) -> #(#';')

%id <number> <negativeNumber> <name> <binarySymbol>;
%start SequenceNode ;

Method
	: <name> MethodSequenceNode 
			{(RBMethodNode 
				selector: '1' value asSymbol 
				arguments: #() 
				body: '2' last) 
					""tags: '2' first; ""
					yourself}
	| BinaryMethodName MethodSequenceNode 
			{(RBMethodNode 
				selector: '1' first 
				arguments: '1' last 
				body: '2' last) 
					""tags: '2' first; ""
					yourself}
	| KeywordMethodName MethodSequenceNode 
			{(RBMethodNode 
				selector: '1' first contents asSymbol 
				arguments: '1' last 
				body: '2' last) 
					""tags: '2' first; ""
					yourself}
	;
BinaryMethodName
	: <binarySymbol> Variable 
			{Array with: '1' value asSymbol with: (Array with: '2')}
	;
KeywordMethodName
	: <keyword> Variable 
			{| stream | 
			stream := WriteStream on: (String new: 30). 
			stream nextPutAll: '1' value. 
			Array with: stream with: (OrderedCollection with: '2')}
	| KeywordMethodName <keyword> Variable 
			{'1' first nextPutAll: '2' value. 
			'1' last add: '3'. 
			'1'}
	;
Primitives
	: 		{OrderedCollection new}
	| Primitives ""<"" PrimitiveMessageSend "">"" 
			{'1' 
				add: ('2' startPosition to: '4' startPosition); 
				yourself}
	;
PrimitiveMessageSend
	:
	| PrimitiveMessageSend <keyword> 
			{nil}
	| PrimitiveMessageSend Literal 
			{nil}
	| PrimitiveMessageSend ""("" 
			{nil}
	| PrimitiveMessageSend "")"" 
			{nil}
	| PrimitiveMessageSend ""{"" 
			{nil}
	| PrimitiveMessageSend ""}"" 
			{nil}
	| PrimitiveMessageSend "";"" 
			{nil}
	| PrimitiveMessageSend ""."" 
			{nil}
	| PrimitiveMessageSend <name> 
			{nil}
	| PrimitiveMessageSend <binarySymbol> 
			{nil}
	;
MethodSequenceNode
	: Primitives ""|"" TemporaryVariables ""|"" Primitives Statements 
			{Array with: ('1' addAll: '5'; yourself) with: (RBSequenceNode temporaries: '3' statements: '6')}
	| Primitives ""||"" Primitives Statements 
			{Array with: ('1' addAll: '3'; yourself) with: (RBSequenceNode temporaries: #() statements: '4')}
	| Primitives Statements 
			{Array with: '1' with: (RBSequenceNode temporaries: #() statements: '2')}
	;
SequenceNode
	: Temporaries Statements 
			{RBSequenceNode temporaries: '1' statements: '2'}
	;
Temporaries
	: 		{#()}
	| ""||""
			{#()}
	| ""|"" TemporaryVariables ""|"" 
			{'2'}
	;
TemporaryVariables
	: 		{OrderedCollection new}
	| TemporaryVariables Variable 
			{'1' 
				add: '2'; 
				yourself}
	;
Statements
	: 		{OrderedCollection new}
	| StatementList OptionalPeriod
			{#liftFirstValue:}
	| StatementList ""."" ""^"" Expression OptionalPeriod
			{'1' 
				add: (RBReturnNode value: '4'); 
				yourself}
	| ""^"" Expression OptionalPeriod
			{OrderedCollection with: (RBReturnNode value: '2')}
	;
OptionalPeriod
	: 
	| "".""
	;
StatementList
	: Expression
	| StatementList ""."" Expression 
			{'1' 
				add: '3'; 
				yourself}
	;
Block
	: ""["" BlockArgs ""|"" SequenceNode ""]"" 
			{RBBlockNode arguments: '2' body: '4'}
	| ""["" SequenceNode ""]"" 
			{RBBlockNode body: '2'}
	| ""["" BlockArgs ""]"" 
			{RBBlockNode arguments: '2' body: (RBSequenceNode statements: #())}
	| ""["" BlockArgs ""||"" TemporaryVariables ""|"" Statements ""]"" 
			{RBBlockNode arguments: '2' body: (RBSequenceNode temporaries: '4' statements: '6')}
	;
BlockArgs
	: "":"" Variable 
			{OrderedCollection with: '2'}
	| BlockArgs "":"" Variable 
			{'1' 
				add: '3'; 
				yourself}
	;
Expression
	: Variable "":="" Expression 
			{RBAssignmentNode variable: '1' value: '3'}
	| Cascade 
			{#liftFirstValue:}
	;
Cascade
	: MessageSend CascadeList 
			{| messages |
			'2' isEmpty ifTrue: [^'1'].
			messages := OrderedCollection new: '2' size + 1.
			messages add: '1'.
			'2' do: [:each | messages add: (RBMessageNode receiver: '1' receiver selector: each first contents asSymbol arguments: each last)].
			RBCascadeNode messages: messages}
	| Primary 
			{#liftFirstValue:}
	;
MessageSend
	: KeywordMessageSend 
			{#liftFirstValue:}
	| BinaryMessageSend 
			{#liftFirstValue:}
	| UnaryMessageSend 
			{#liftFirstValue:}
	;
CascadeList
	: 		{OrderedCollection new}
	| CascadeList "";"" SimpleMessage 
			{'1' 
				add: '3'; 
				yourself}
	;
SimpleMessage
	: UnaryMessage 
			{#liftFirstValue:}
	| BinaryMessage 
			{#liftFirstValue:}
	| KeywordMessage 
			{#liftFirstValue:}
	;
KeywordMessageSend
	: BinaryMessageSend KeywordMessage 
			{#createMessageNodeFrom:}
	| UnaryMessageSend KeywordMessage 
			{#createMessageNodeFrom:}
	| Primary KeywordMessage 
			{#createMessageNodeFrom:}
	;
KeywordMessage
	: <keyword> KeywordArgument 
			{| stream | 
			stream := WriteStream on: (String new: 30). 
			stream nextPutAll: '1' value. 
			Array with: stream with: (OrderedCollection with: '2')}
	| KeywordMessage <keyword> KeywordArgument 
			{'1' first nextPutAll: '2' value. 
			'1' last add: '3'. 
			'1'}
	;
KeywordArgument
	: BinaryMessageSend 
			{#liftFirstValue:}
	| UnaryMessageSend 
			{#liftFirstValue:}
	| Primary 
			{#liftFirstValue:}
	;
BinaryMessageSend
	: BinaryMessageSend BinaryMessage 
			{#createMessageNodeFrom:}
	| UnaryMessageSend BinaryMessage 
			{#createMessageNodeFrom:}
	| Primary BinaryMessage 
			{#createMessageNodeFrom:} 
	;
BinaryMessage 
	: <binarySymbol> BinaryArgument 
			{Array with: (ReadStream on: '1' value) with: (Array with: '2')}
	;
BinaryArgument
	: UnaryMessageSend 
			{#liftFirstValue:}
	| Primary 
			{#liftFirstValue:}
	;
UnaryMessageSend 
	: UnaryMessageSend UnaryMessage 
			{#createMessageNodeFrom:}
	| Primary UnaryMessage 
			{#createMessageNodeFrom:}
	;
UnaryMessage 
	: <name> 
			{Array with: (ReadStream on: '1' value) with: #()}
	;
Primary
	: ""("" Expression "")"" 
			{#liftSecondValue:}
	| Literal 
			{RBLiteralNode value: '1'}
	| Variable 
			{#liftFirstValue:}
	| Block 
			{#liftFirstValue:}
	;
Variable
	: <name> 
			{RBVariableNode named: '1' value}
	| <dottedName> 
			{RBVariableNode named: '1' value}
	;
Literal
	: ""true"" 
			{true}
	| ""false"" 
			{false}
	| ""nil"" 
			{nil}
	| <number> 
			{'1' value}
	| <negativeNumber> 
			{'1' value}
	| <character> 
			{'1' value at: 2}
	| <string> 
			{('1' value copyFrom: 2 to: '1' value size - 1) copyReplaceAll: '''''' with: ''''}
	|  ""#"" <string> 
			{(('2' value copyFrom: 2 to: '2' value size - 1) copyReplaceAll: '''''' with: '''') asSymbol}
	| ""#"" <name> 
			{'2' value asSymbol}
	| ""#"" <binarySymbol> 
			{'2' value asSymbol}
	| ""#"" <keyword> 
			{'2' value asSymbol}
	| ""#"" <multikeyword> 
			{'2' value asSymbol}
	| ""#"" ""["" ByteArray ""]"" 
			{'3' contents}
	| ""#"" ""("" Array "")"" 
			{'3' contents}
	| ""#"" ""{"" <name> ""}"" 
			{'3' value asQualifiedReference}
	| ""#"" ""{"" <dottedName> ""}"" 
			{'3' value asQualifiedReference}
	;
ByteArray
	: 		{WriteStream on: ByteArray new}
	| ByteArray <number> 
			{'1' 
				nextPut: '2' value; 
				yourself}
	;
Array
	: 		{WriteStream on: Array new}
	| Array ArrayLiteral 
			{'1' 
				nextPut: '2'; 
				yourself} 
	| Array <name> "":="" 
			{'2' stopPosition + 1 = '3' startPosition 
				ifTrue: ['1' nextPut: ('2' value, ':') asSymbol] 
				ifFalse: ['1' nextPut: '2' value asSymbol; nextPut: #':']. 
			self position: '3' startPosition. 
			currentToken := nil. 
			'1'}
	;
ArrayLiteral
	: Literal 
			{#liftFirstValue:}
	| <name> 
			{'1' value asSymbol}
	| <binarySymbol> 
			{'1' value asSymbol}
	| <keyword> 
			{'1' value asSymbol}
	| <multikeyword> 
			{'1' value asSymbol}
	| ""{"" <name> ""}"" 
			{'2' value asQualifiedReference}
	| ""{"" <dottedName> ""}"" 
			{'2' value asQualifiedReference}
	| ""("" Array "")"" 
			{'2' contents}
	| ""["" ByteArray ""]"" 
			{'2' contents}
	| <anyChar> 
			{'1' value asSymbol}
	| <dottedName> 
			{'1' value}
	;"!

fileExtension
	^'.st'!

fileExtensionType
	^'Smalltalk source'!

parseExpression: aString 
	^self simplifyExpression: (self parse: aString
				startingAt: self startingStateForSequenceNode)!

parseExpression: aString onError: aBlock 
	^self simplifyExpression: (self 
				parse: aString
				startingAt: self startingStateForSequenceNode
				onError: aBlock)!

parseMethod: aString 
	^self parse: aString startingAt: self startingStateForMethod!

parseMethod: aString onError: aBlock 
	^self 
		parse: aString
		startingAt: self startingStateForMethod
		onError: aBlock!

scannerClass
	^StScanner!

simplifyExpression: aSequenceNode 
	^(aSequenceNode statements size == 1 
		and: [aSequenceNode temporaries isEmpty]) 
			ifTrue: [aSequenceNode statements first]
			ifFalse: [aSequenceNode]!

startingStateForMethod
	^1!

startingStateForSequenceNode
	^2! !
!StParser class categoriesFor: #definitionComment!generated!public! !
!StParser class categoriesFor: #fileExtension!file types!public! !
!StParser class categoriesFor: #fileExtensionType!file types!public! !
!StParser class categoriesFor: #parseExpression:!accessing!private! !
!StParser class categoriesFor: #parseExpression:onError:!accessing!private! !
!StParser class categoriesFor: #parseMethod:!accessing!private! !
!StParser class categoriesFor: #parseMethod:onError:!accessing!private! !
!StParser class categoriesFor: #scannerClass!generated-accessing!public! !
!StParser class categoriesFor: #simplifyExpression:!private! !
!StParser class categoriesFor: #startingStateForMethod!generated!public! !
!StParser class categoriesFor: #startingStateForSequenceNode!generated!public! !

StScanner guid: (GUID fromString: '{A8586B15-A561-46CB-95D4-0C251B2937EA}')!
StScanner comment: ''!
!StScanner categoriesForClass!Unclassified! !
!StScanner methodsFor!

binarySymbolId
	^26!

emptySymbolTokenId
	^64!

errorTokenId
	^63!

nameId
	^20!

negativeNumber
	stream skip: -2.
	^self scanNumber: self negativeNumberId!

negativeNumberId
	^22!

number
	stream skip: -1.
	^self scanNumber: self numberId!

numberId
	^21!

scan1
	[self step.
	currentCharacter == $' ifTrue: [^self scan2]] repeat!

scan2
	self recordMatch: #(23).
	self step.
	currentCharacter == $' ifTrue: [^self scan1].
	^self reportLastMatch!

scan3
	self step.
	(currentCharacter isLetter or: [currentCharacter == $_])
		ifTrue:
			[[self recordMatch: #(29).
			self step.
			currentCharacter == $. ifTrue: [^self scan3].
			currentCharacter isLetter or: [currentCharacter isHexDigit or: [currentCharacter == $_]]] whileTrue].
	^self reportLastMatch!

scan4
	self recordMatch: #(24).
	self step.
	currentCharacter == $= ifTrue: [^self recordAndReportMatch: #(31)].
	(currentCharacter isLetter or: [currentCharacter == $_]) ifTrue: [^self scan5].
	^self reportLastMatch!

scan5
	[self step.
	currentCharacter == $:
		ifTrue:
			[self recordMatch: #(25).
			self step.
			(currentCharacter isLetter or: [currentCharacter == $_]) ifTrue: [^self scan5].
			^self reportLastMatch].
	currentCharacter isLetter or: [currentCharacter isHexDigit or: [currentCharacter == $_]]] whileTrue.
	^self reportLastMatch!

scanForToken
	self step.
	currentCharacter == $"
		ifTrue:
			[self recordMatch: #(32).
			self step.
			currentCharacter == $" ifTrue: [^self recordAndReportMatch: #(28)].
			true
				ifTrue:
					[[self step.
					currentCharacter == $" ifTrue: [^self recordAndReportMatch: #(28)]] repeat].
			^self reportLastMatch].
	currentCharacter == $# ifTrue: [^self recordAndReportMatch: #(2 32)].
	currentCharacter == $$
		ifTrue:
			[self recordMatch: #(32).
			self step.
			^self recordAndReportMatch: #(30)].
	currentCharacter == $'
		ifTrue:
			[self recordMatch: #(32).
			self step.
			currentCharacter == $' ifTrue: [^self scan2].
			^self scan1].
	currentCharacter == $( ifTrue: [^self recordAndReportMatch: #(3 32)].
	currentCharacter == $) ifTrue: [^self recordAndReportMatch: #(4 32)].
	currentCharacter == $-
		ifTrue:
			[self recordMatch: #(26 32).
			self step.
			currentCharacter isDigit ifTrue: [^self recordAndReportMatch: #(22)].
			(currentCharacter == $!!
				or:
					[(currentCharacter between: $% and: $&)
						or:
							[(currentCharacter between: $* and: $,)
								or: [currentCharacter == $/ or: [(currentCharacter between: $< and: $@) or: ['\|~' includes: currentCharacter]]]]])
				ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	currentCharacter == $. ifTrue: [^self recordAndReportMatch: #(5 32)].
	currentCharacter == $:
		ifTrue:
			[self recordMatch: #(19 32).
			self step.
			currentCharacter == $= ifTrue: [^self recordAndReportMatch: #(14)].
			^self reportLastMatch].
	currentCharacter == $; ifTrue: [^self recordAndReportMatch: #(6 32)].
	currentCharacter == $<
		ifTrue:
			[self recordMatch: #(7 26 32).
			self step.
			(currentCharacter == $!!
				or:
					[(currentCharacter between: $% and: $&)
						or:
							[(currentCharacter between: $* and: $,)
								or: [currentCharacter == $/ or: [(currentCharacter between: $< and: $@) or: ['\|~' includes: currentCharacter]]]]])
				ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	currentCharacter == $>
		ifTrue:
			[self recordMatch: #(8 26 32).
			self step.
			(currentCharacter == $!!
				or:
					[(currentCharacter between: $% and: $&)
						or:
							[(currentCharacter between: $* and: $,)
								or: [currentCharacter == $/ or: [(currentCharacter between: $< and: $@) or: ['\|~' includes: currentCharacter]]]]])
				ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	currentCharacter == $[ ifTrue: [^self recordAndReportMatch: #(18 32)].
	currentCharacter == $] ifTrue: [^self recordAndReportMatch: #(16 32)].
	currentCharacter == $^ ifTrue: [^self recordAndReportMatch: #(17 32)].
	currentCharacter == ${ ifTrue: [^self recordAndReportMatch: #(1 32)].
	currentCharacter == $|
		ifTrue:
			[self recordMatch: #(15 26 32).
			self step.
			(currentCharacter == $!!
				or:
					[(currentCharacter between: $% and: $&)
						or:
							[(currentCharacter between: $* and: $,)
								or: [currentCharacter == $/ or: [(currentCharacter between: $< and: $@) or: ['\|~' includes: currentCharacter]]]]])
				ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	currentCharacter == $} ifTrue: [^self recordAndReportMatch: #(9 32)].
	currentCharacter isDigit ifTrue: [^self recordAndReportMatch: #(21 32)].
	currentCharacter isWhitespace
		ifTrue:
			[self recordMatch: #(27 32).
			self step.
			currentCharacter isWhitespace
				ifTrue:
					[[self recordMatch: #(27).
					self step.
					currentCharacter isWhitespace] whileTrue].
			^self reportLastMatch].
	(currentCharacter == $!!
		or:
			[(currentCharacter between: $% and: $&)
				or:
					[(currentCharacter between: $* and: $,)
						or:
							[('/=' includes: currentCharacter) or: [(currentCharacter between: $? and: $@) or: ['\~' includes: currentCharacter]]]]])
		ifTrue:
			[self recordMatch: #(26 32).
			self step.
			(currentCharacter == $!!
				or:
					[(currentCharacter between: $% and: $&)
						or:
							[(currentCharacter between: $* and: $,)
								or: [currentCharacter == $/ or: [(currentCharacter between: $< and: $@) or: ['\|~' includes: currentCharacter]]]]])
				ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	(currentCharacter <= ##(Character value: 8)
		or:
			[(currentCharacter between: ##(Character value: 14) and: ##(Character value: 31))
				or:
					[currentCharacter == $`
						or:
							[(currentCharacter between: ##(Character value: 127) and: ##(Character value: 130))
								or:
									[(currentCharacter between: ##(Character value: 132) and: ##(Character value: 137))
										or:
											[('' includes: currentCharacter)
												or:
													[(currentCharacter between: ##(Character value: 143) and: ##(Character value: 153))
														or:
															[('' includes: currentCharacter)
																or:
																	[(currentCharacter between: ##(Character value: 160) and: ##(Character value: 169))
																		or:
																			[(currentCharacter between: ##(Character value: 171) and: ##(Character value: 180))
																				or:
																					[(currentCharacter between: ##(Character value: 182) and: ##(Character value: 185))
																						or:
																							[(currentCharacter between: ##(Character value: 187) and: ##(Character value: 191))
																								or: ['' includes: currentCharacter]]]]]]]]]]]]) ifTrue: [^self recordAndReportMatch: #(32)].
	true
		ifTrue:
			[self recordMatch: #(20 32).
			self step.
			currentCharacter == $. ifTrue: [^self scan3].
			currentCharacter == $: ifTrue: [^self scan4].
			(currentCharacter isLetter or: [currentCharacter isHexDigit or: [currentCharacter == $_]])
				ifTrue:
					[[self recordMatch: #(20).
					self step.
					currentCharacter == $. ifTrue: [^self scan3].
					currentCharacter == $: ifTrue: [^self scan4].
					currentCharacter isLetter or: [currentCharacter isHexDigit or: [currentCharacter == $_]]] whileTrue]].
	^self reportLastMatch!

scanNumber: id 
	| number |
	number := Number readFrom: stream.
	outputStream reset.
	matchActions := Array with: id.
	^self createTokenFor: number!

tokenActions
	^#(nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil #number #negativeNumber nil nil nil nil #whitespace #comment nil nil #variableAssignment)!

variableAssignment
	| token |
	outputStream skip: -2.
	stream skip: -2.
	matchActions := Array with: self nameId.
	token := self createTokenFor: outputStream contents.
	outputStream reset.
	^token! !
!StScanner categoriesFor: #binarySymbolId!generated!public! !
!StScanner categoriesFor: #emptySymbolTokenId!generated!public! !
!StScanner categoriesFor: #errorTokenId!generated!public! !
!StScanner categoriesFor: #nameId!generated!public! !
!StScanner categoriesFor: #negativeNumber!private!scanning! !
!StScanner categoriesFor: #negativeNumberId!generated!public! !
!StScanner categoriesFor: #number!private!scanning! !
!StScanner categoriesFor: #numberId!generated!public! !
!StScanner categoriesFor: #scan1!generated!public! !
!StScanner categoriesFor: #scan2!generated!public! !
!StScanner categoriesFor: #scan3!generated!public! !
!StScanner categoriesFor: #scan4!generated!public! !
!StScanner categoriesFor: #scan5!generated!public! !
!StScanner categoriesFor: #scanForToken!generated!public! !
!StScanner categoriesFor: #scanNumber:!private!scanning! !
!StScanner categoriesFor: #tokenActions!generated!public! !
!StScanner categoriesFor: #variableAssignment!private!scanning! !

!StScanner class methodsFor!

initializeKeywordMap
	keywordMap := Dictionary new.
	#(#(26 '||' 12) #(20 'false' 13) #(20 'nil' 10) #(20 'true' 11))
		do: [:each | (keywordMap at: each first ifAbsentPut: [Dictionary new]) at: (each at: 2) put: each last].
	^keywordMap! !
!StScanner class categoriesFor: #initializeKeywordMap!generated-initialization!public! !

"Binary Globals"!

