| package |
package := Package name: 'SmaCC Source Editing'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It has a special string implementation that is used when transforming code. 

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #SmaCCString;
	add: #SmaCCStringIndex;
	add: #SmaCCStringInterval;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	yourself).

package!

"Class Definitions"!

Object subclass: #SmaCCString
	instanceVariableNames: 'firstSentinel lastSentinel currentId cachedInterval'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #SmaCCStringInterval
	instanceVariableNames: 'id start stop next previous string isRemoved'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Magnitude subclass: #SmaCCStringIndex
	instanceVariableNames: 'index intervalId'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SmaCCString guid: (GUID fromString: '{708C078C-B528-4D1C-ACC5-A6DE051ADA18}')!
SmaCCString comment: 'SmaCCString is a special string like object that makes rewriting source code faster. It supports operations to insert and delete text from the middle of strings more efficiently than constructing new strings.

Instance Variables:
	currentId	<Integer>	the id that is used when constructing new SmaCCIntervals
	firstSentinel	<SmaCCStringInterval>	the head of the SmaCCInterval list
	lastSentinel	<SmaCCStringInterval>	the tail of the SmaCCInterval list
	cachedInterval	<SmaCCStringInterval>	the interval where the last action took place -- most likely it or something nearby will the next thing needed

'!
!SmaCCString categoriesForClass!Unclassified! !
!SmaCCString methodsFor!

, aString
	^(self copy)
		append: aString;
		yourself!

addAnnotationTo: aSmaCCStringInterval!

addNewLineWhitespace: aString from: startIndex to: stopIndex
	| currentInterval next |
	currentInterval := self intervalContaining: startIndex.
	[currentInterval isSentinel ifTrue: [^self].
	next := currentInterval next.
	currentInterval isRemoved
		ifFalse:
			[self
				addNewLineWhitespace: aString
				toInterval: currentInterval
				from: startIndex
				to: stopIndex].
	currentInterval containsIndex: stopIndex] whileFalse: [currentInterval := next]!

addNewLineWhitespace: aString toInterval: currentInterval from: startIndex to: stopIndex
	| start stop stream next location interval whitespaceInterval |
	interval := currentInterval.
	next := currentInterval next.
	start := (currentInterval containsIndex: startIndex)
		ifTrue: [self indexFor: startIndex]
		ifFalse: [currentInterval start].
	stop := (currentInterval containsIndex: stopIndex) ifTrue: [self indexFor: stopIndex] ifFalse: [currentInterval stop].
	stream := ReadStream on: (currentInterval string copyFrom: start to: stop).
	[stream atEnd]
		whileFalse:
			[| char |
			char := stream next.
			char = Character cr ifTrue: [stream peek = Character lf ifTrue: [stream next]].
			(char = Character lf or: [char = Character cr])
				ifTrue:
					[location := stream position + start.
					interval := location > stop ifTrue: [next] ifFalse: [interval splitAt: location].
					whitespaceInterval := self newIntervalFor: aString.
					interval previous next: whitespaceInterval.
					whitespaceInterval next: interval]]!

addPrefix: aString
	^self insert: aString at: self startIndex!

annotationRangesFor: aSmaCCAnnotation
	| ranges startIndex current |
	ranges := OrderedCollection new.
	startIndex := 1.
	current := firstSentinel next.
	[current isSentinel]
		whileFalse:
			[(current isRemoved not and: [current annotations includes: aSmaCCAnnotation])
				ifTrue: [ranges add: (startIndex to: startIndex + current size - 1)].
			startIndex := startIndex + current size.
			current := current next].
	^ranges!

annotationsInNewRange: range
	| startIndex first last annotations current |
	annotations := Set new.
	startIndex := 0.
	first := range start.
	last := range isEmpty ifTrue: [first] ifFalse: [range stop].
	current := firstSentinel next.
	[current isSentinel]
		whileFalse:
			[startIndex > last ifTrue: [^annotations asOrderedCollection].
			startIndex := startIndex + current size.
			startIndex >= first ifTrue: [annotations addAll: current annotations].
			current := current next].
	^annotations asOrderedCollection!

annotationsInOriginalRange: range
	| first last |
	first := self intervalContaining: range start.
	last := range isEmpty ifTrue: [first] ifFalse: [self intervalContaining: range stop].
	^(first annotationsTo: last) ifNil: [(last annotationsTo: first) ifNil: [#()]]!

append: aString
	^self insert: aString at: self stopIndex + 1!

asString
	| stream |
	stream := WriteStream on: (String new: self size).
	firstSentinel next do: [:each | each nextPutOn: stream].
	^stream contents!

at: anIndex
	^(self intervalContaining: anIndex) string at: (self indexFor: anIndex)!

basicNextIndex: anIndex
	| interval newIndex |
	interval := self intervalContaining: anIndex.
	newIndex := anIndex + 1.
	^(interval containsIndex: newIndex)
		ifTrue: [newIndex]
		ifFalse: [interval next isSentinel ifTrue: [self stopIndex + 1] ifFalse: [interval next startIndex]]!

basicPreviousIndex: anIndex
	| interval newIndex |
	interval := self intervalContaining: anIndex.
	newIndex := anIndex - 1.
	^((interval containsIndex: newIndex) and: [(self indexFor: newIndex) ~= 0])
		ifTrue: [newIndex]
		ifFalse: [interval previous isSentinel ifTrue: [nil] ifFalse: [interval previous stopIndex]]!

copy
	| new first last current newCurrent |
	first := last := nil.
	new := self class new.
	current := firstSentinel next.
	[current isSentinel]
		whileFalse:
			[newCurrent := current copy.
			first isNil ifTrue: [first := newCurrent].
			last notNil ifTrue: [last next: newCurrent].
			last := newCurrent.
			current := current next].
	first notNil ifTrue: [new insertFirst: first last: last at: 1].
	new setIntervalId.
	^new!

copyFrom: startIndex to: stopIndex
	| result |
	result := self class new.
	self
		copyFrom: startIndex
		to: stopIndex
		into: result
		at: 1.
	^result!

copyFrom: startIndex to: stopIndex into: aSmaCCString at: anInteger
	| startInterval stopInterval current newInterval first last |
	startInterval := self intervalContaining: startIndex.
	stopInterval := self intervalContaining: stopIndex.
	current := startInterval.
	last := first := nil.
	[(current isRemoved or: [current isSentinel])
		ifFalse:
			[newInterval := aSmaCCString newIntervalFor: current string.
			newInterval addAllAnnotations: current annotations.
			newInterval start: (current == startInterval ifTrue: [self indexFor: startIndex] ifFalse: [current start]).
			newInterval stop: (current == stopInterval ifTrue: [self indexFor: stopIndex] ifFalse: [current stop]).
			first isNil ifTrue: [first := newInterval].
			last notNil ifTrue: [last next: newInterval].
			last := newInterval].
	current ~~ stopInterval]
		whileTrue:
			[current isSentinel ifTrue: [^''].
			current := current next].
	first isNil ifTrue: [^aSmaCCString startIndex].
	^aSmaCCString
		insertFirst: first
		last: last
		at: anInteger!

deleteCharacter: anInteger 
	^self deleteFrom: anInteger to: anInteger!

deleteFrom: startIndex to: stopIndex
	| startInterval atEnd |
	startInterval := self intervalContaining: startIndex.
	startInterval := startInterval splitAt: (self indexFor: startIndex).
	[atEnd := startInterval containsIndex: stopIndex.
	(atEnd and: [startInterval stop ~= (self indexFor: stopIndex)])
		ifTrue: [startInterval splitAt: (self indexFor: stopIndex) + 1].
	cachedInterval := startInterval := self removeInterval: startInterval.
	atEnd or: [startInterval containsIndex: stopIndex]] whileFalse: [startInterval := startInterval next]!

deleteUpTo: anIndex
	self deleteFrom: self startIndex to: anIndex - 1!

deleteUpToEnd: anIndex
	self deleteFrom: anIndex to: self stopIndex!

deleteWhitespaceAfterIndex: stopPosition
	| stopIndex |
	stopIndex := self whitespaceEndIndexAfterIndex: stopPosition.
	(stopIndex isNil or: [self isIndex: stopIndex equalTo: stopPosition]) ifTrue: [^''].
	^self deleteFrom: (self nextIndex: stopPosition) to: stopIndex!

deleteWhitespaceBeforeIndex: startPosition
	| startIndex |
	startIndex := self whitespaceStartIndexBeforeIndex: startPosition.
	(startIndex isNil or: [self isIndex: startIndex equalTo: startPosition]) ifTrue: [^''].
	^self deleteFrom: startIndex to: (self basicPreviousIndex: startPosition)!

indexFor: anIndex
	^anIndex isInteger ifTrue: [anIndex] ifFalse: [anIndex index]!

initialize
	super initialize.
	firstSentinel := self intervalClass
		from: 1
		to: 0
		string: ''.
	firstSentinel beSentinel.
	lastSentinel := self intervalClass
		from: 1
		to: 0
		string: ''.
	lastSentinel beSentinel.
	firstSentinel next: lastSentinel.
	currentId := 0!

insert: aString at: anInteger
	| newFirstInterval |
	aString isEmpty ifTrue: [^anInteger].
	(aString isKindOf: SmaCCString)
		ifTrue:
			[^aString
				copyFrom: aString startIndex
				to: aString stopIndex
				into: self
				at: anInteger]
		ifFalse:
			[newFirstInterval := self newIntervalFor: aString.
			^self
				insertFirst: newFirstInterval
				last: newFirstInterval
				at: anInteger]!

insertFirst: first last: last at: anInteger
	| interval |
	interval := self intervalContaining: anInteger.
	interval := interval splitAt: (self indexFor: anInteger).
	interval previous next: first.
	last next: interval.
	^first startIndex!

intervalClass
	^SmaCCStringInterval!

intervalContaining: anIndex
	| prev next |
	(cachedInterval isNil or: [cachedInterval isSentinel])
		ifTrue:
			[prev := lastSentinel previous.
			next := firstSentinel next]
		ifFalse:
			[(cachedInterval containsIndex: anIndex) ifTrue: [^cachedInterval].
			prev := cachedInterval previous.
			next := cachedInterval next].
	[prev isSentinel not or: [next isSentinel not]]
		whileTrue:
			[(prev isSentinel not and: [prev containsIndex: anIndex]) ifTrue: [^cachedInterval := prev].
			(next isSentinel not and: [next containsIndex: anIndex]) ifTrue: [^cachedInterval := next].
			prev := prev previous.
			next := next next].
	^lastSentinel!

isEmpty
	firstSentinel next do: [:each | each isEmpty ifFalse: [^false]].
	^true!

isIndex: aIndex equalTo: bIndex
	aIndex = bIndex ifTrue: [^true].
	aIndex class = bIndex class ifTrue: [^false].
	^aIndex isInteger
		ifTrue: [bIndex intervalId = 1 and: [aIndex = bIndex index]]
		ifFalse: [bIndex isInteger and: [aIndex intervalId = 1 and: [aIndex index = bIndex]]]!

isModified
	^firstSentinel next ~~ lastSentinel previous!

isRemoved: anIndex
	| interval |
	interval := self intervalContaining: anIndex.
	^interval isSentinel or: [interval isRemoved]!

moveFrom: fromIndex to: toIndex after: newIndex
	self
		moveFrom: fromIndex
		to: toIndex
		location: newIndex
		before: false!

moveFrom: fromIndex to: toIndex before: newIndex
	self
		moveFrom: fromIndex
		to: toIndex
		location: newIndex
		before: true!

moveFrom: fromIndex to: toIndex location: newIndex before: aBoolean
	| startInterval stopInterval newInterval currentInterval afterInterval |
	startInterval := self intervalContaining: fromIndex.
	startInterval := startInterval splitAt: (self indexFor: fromIndex).
	currentInterval := startInterval.
	[currentInterval containsIndex: toIndex]
		whileFalse:
			[currentInterval isSentinel ifTrue: [^self].
			(currentInterval containsIndex: newIndex) ifTrue: [^self].
			currentInterval := currentInterval next].
	stopInterval := currentInterval.
	(stopInterval containsIndex: newIndex) ifTrue: [newIndex < toIndex ifTrue: [^self]].
	stopInterval := (stopInterval splitAt: (self indexFor: toIndex) + 1) previous.
	newInterval := self intervalContaining: newIndex.
	afterInterval := aBoolean
		ifTrue: [newInterval splitAt: (self indexFor: newIndex)]
		ifFalse:
			[(newInterval containsIndex: newIndex)
				ifTrue: [newInterval splitAt: (self indexFor: newIndex) + 1]
				ifFalse: [newInterval next]].
	startInterval previous next: stopInterval next.
	afterInterval previous next: startInterval.
	stopInterval next: afterInterval!

newIntervalFor: aString
	| newInterval |
	newInterval := self intervalClass
		from: 1
		to: aString size
		string: aString.
	currentId := currentId + 1.
	newInterval id: currentId.
	^newInterval!

nextIndex: anIndex
	| interval newIndex |
	interval := self intervalContaining: anIndex.
	newIndex := anIndex + 1.
	[(interval isRemoved not and: [interval containsIndex: newIndex])
		ifTrue: [^newIndex]
		ifFalse:
			[interval := interval next.
			newIndex := interval startIndex.
			newIndex isNil 
				ifTrue: [^self stopIndex + 1]]] repeat!

node
	^nil!

node: aSmaCCParseNode!

on: aString
	| firstInterval |
	firstInterval := self intervalClass
		from: 1
		to: aString size
		string: aString.
	firstInterval id: 1.
	currentId := 1.
	firstSentinel next: firstInterval.
	firstInterval next: lastSentinel!

previousIndex: anIndex
	| interval newIndex |
	interval := self intervalContaining: anIndex.
	newIndex := anIndex - 1.
	[(interval isRemoved not and: [(interval containsIndex: newIndex) and: [(self indexFor: newIndex) ~= 0]])
		ifTrue: [^newIndex]
		ifFalse:
			[interval previous isSentinel
				ifTrue: [^nil]
				ifFalse:
					[interval := interval previous.
					newIndex := interval stopIndex]]] repeat!

publishedAspects
	| aspects aspectClass |
	aspectClass := Smalltalk at: #Aspect.
	aspects := LookupTable new.
	aspects add: (self newAspect: #yourself class: aspectClass).
	aspects add: (aspectClass multilineString: #asString) beReadOnly.
	aspectClass := Smalltalk at: #InstVarAspect.
	1 to: self class instSize do: [:i | aspects add: ((self instVarAt: i) newAspect: i class: aspectClass)].
	^aspects!

removeInterval: startInterval
	^startInterval convertToRemoved!

replaceFrom: startIndex to: stopIndex with: aString
	| newIndex |
	newIndex := self insert: aString at: startIndex.
	self deleteFrom: startIndex to: stopIndex.
	^newIndex!

rule
	^nil!

rule: aSmaCCRewrite!

setIntervalId
	firstSentinel next do: [:each | currentId := currentId max: each id]!

size
	| current length |
	length := 0.
	current := firstSentinel next.
	[current isSentinel]
		whileFalse:
			[length := length + current size.
			current := current next].
	^length!

startIndex
	^firstSentinel next isSentinel ifTrue: [0] ifFalse: [firstSentinel next startIndex]!

startOfLineWhitespaceBeforeIndex: startPosition
	| startIndex |
	startIndex := self startOfLineWhitespaceStartIndexBeforeIndex: startPosition.
	startIndex isNil ifTrue: [^''].
	^(self copyFrom: startIndex to: (self basicPreviousIndex: startPosition)) asString!

startOfLineWhitespaceStartIndexBeforeIndex: startPosition
	| stopIndex currentIndex position |
	position := startPosition.
	position isNil ifTrue: [^nil].
	stopIndex := self previousIndex: position.
	stopIndex isNil ifTrue: [^nil].
	currentIndex := stopIndex.
	[currentIndex notNil
		and:
			[| char |
			char := self at: currentIndex.
			char isWhitespace and: [char ~= Character cr and: [char ~= Character lf]]]]
		whileTrue: [currentIndex := self previousIndex: currentIndex].
	^currentIndex isNil ifTrue: [self startIndex] ifFalse: [self basicNextIndex: currentIndex]!

stopIndex
	^lastSentinel previous isSentinel ifTrue: [0] ifFalse: [lastSentinel previous stopIndex]!

whitespaceAfterIndex: stopPosition
	| stopIndex |
	stopIndex := self whitespaceEndIndexAfterIndex: stopPosition.
	stopIndex isNil ifTrue: [^''].
	^(self copyFrom: (self basicNextIndex: stopPosition) to: stopIndex) asString!

whitespaceBeforeIndex: startPosition
	| startIndex |
	startIndex := self whitespaceStartIndexBeforeIndex: startPosition.
	startIndex isNil ifTrue: [^''].
	^(self copyFrom: startIndex to: (self basicPreviousIndex: startPosition)) asString!

whitespaceEndIndexAfterIndex: stopPosition
	| currentIndex position startIndex stopIndex |
	position := stopPosition.
	position isNil ifTrue: [^nil].
	startIndex := self nextIndex: position.
	startIndex isNil ifTrue: [^nil].
	currentIndex := startIndex.
	stopIndex := self stopIndex + 1.
	[currentIndex notNil and: [currentIndex ~= stopIndex and: [(self at: currentIndex) isWhitespace]]]
		whileTrue: [currentIndex := self nextIndex: currentIndex].
	(currentIndex isNil or: [currentIndex = stopIndex]) ifTrue: [^self stopIndex].
	[currentIndex := self basicPreviousIndex: currentIndex.
	(self isIndex: stopPosition equalTo: currentIndex) or: [(self isRemoved: currentIndex) not]] whileFalse.
	^currentIndex!

whitespaceStartIndexBeforeIndex: startPosition
	| stopIndex currentIndex position |
	position := startPosition.
	position isNil ifTrue: [^nil].
	stopIndex := self previousIndex: position.
	stopIndex isNil ifTrue: [^nil].
	currentIndex := stopIndex.
	[currentIndex notNil and: [(self at: currentIndex) isWhitespace]]
		whileTrue: [currentIndex := self previousIndex: currentIndex].
	currentIndex isNil ifTrue: [^self startIndex].
	[currentIndex := self basicNextIndex: currentIndex.
	(self isIndex: startPosition equalTo: currentIndex) or: [(self isRemoved: currentIndex) not]] whileFalse.
	^currentIndex! !
!SmaCCString categoriesFor: #,!copying!private! !
!SmaCCString categoriesFor: #addAnnotationTo:!accessing!public! !
!SmaCCString categoriesFor: #addNewLineWhitespace:from:to:!editing!public! !
!SmaCCString categoriesFor: #addNewLineWhitespace:toInterval:from:to:!editing!private! !
!SmaCCString categoriesFor: #addPrefix:!editing!private! !
!SmaCCString categoriesFor: #annotationRangesFor:!actions!public! !
!SmaCCString categoriesFor: #annotationsInNewRange:!actions!public! !
!SmaCCString categoriesFor: #annotationsInOriginalRange:!actions!public! !
!SmaCCString categoriesFor: #append:!editing!private! !
!SmaCCString categoriesFor: #asString!accessing!private! !
!SmaCCString categoriesFor: #at:!accessing!private! !
!SmaCCString categoriesFor: #basicNextIndex:!accessing!private! !
!SmaCCString categoriesFor: #basicPreviousIndex:!accessing!private! !
!SmaCCString categoriesFor: #copy!copying!public! !
!SmaCCString categoriesFor: #copyFrom:to:!copying!private! !
!SmaCCString categoriesFor: #copyFrom:to:into:at:!private! !
!SmaCCString categoriesFor: #deleteCharacter:!editing!private! !
!SmaCCString categoriesFor: #deleteFrom:to:!editing!private! !
!SmaCCString categoriesFor: #deleteUpTo:!editing!private! !
!SmaCCString categoriesFor: #deleteUpToEnd:!editing!private! !
!SmaCCString categoriesFor: #deleteWhitespaceAfterIndex:!accessing!private! !
!SmaCCString categoriesFor: #deleteWhitespaceBeforeIndex:!accessing!private! !
!SmaCCString categoriesFor: #indexFor:!accessing!private! !
!SmaCCString categoriesFor: #initialize!initialize-release!public! !
!SmaCCString categoriesFor: #insert:at:!editing!private! !
!SmaCCString categoriesFor: #insertFirst:last:at:!private! !
!SmaCCString categoriesFor: #intervalClass!private! !
!SmaCCString categoriesFor: #intervalContaining:!private! !
!SmaCCString categoriesFor: #isEmpty!private!testing! !
!SmaCCString categoriesFor: #isIndex:equalTo:!private! !
!SmaCCString categoriesFor: #isModified!public!testing! !
!SmaCCString categoriesFor: #isRemoved:!public!testing! !
!SmaCCString categoriesFor: #moveFrom:to:after:!editing!public! !
!SmaCCString categoriesFor: #moveFrom:to:before:!editing!public! !
!SmaCCString categoriesFor: #moveFrom:to:location:before:!editing!public! !
!SmaCCString categoriesFor: #newIntervalFor:!private! !
!SmaCCString categoriesFor: #nextIndex:!accessing!private! !
!SmaCCString categoriesFor: #node!accessing!public! !
!SmaCCString categoriesFor: #node:!accessing!public! !
!SmaCCString categoriesFor: #on:!initialize-release!private! !
!SmaCCString categoriesFor: #previousIndex:!accessing!private! !
!SmaCCString categoriesFor: #publishedAspects!development!public! !
!SmaCCString categoriesFor: #removeInterval:!editing!private! !
!SmaCCString categoriesFor: #replaceFrom:to:with:!editing!private! !
!SmaCCString categoriesFor: #rule!accessing!public! !
!SmaCCString categoriesFor: #rule:!accessing!public! !
!SmaCCString categoriesFor: #setIntervalId!private! !
!SmaCCString categoriesFor: #size!accessing!public! !
!SmaCCString categoriesFor: #startIndex!accessing!private! !
!SmaCCString categoriesFor: #startOfLineWhitespaceBeforeIndex:!accessing!private! !
!SmaCCString categoriesFor: #startOfLineWhitespaceStartIndexBeforeIndex:!accessing!private! !
!SmaCCString categoriesFor: #stopIndex!accessing!private! !
!SmaCCString categoriesFor: #whitespaceAfterIndex:!accessing!private! !
!SmaCCString categoriesFor: #whitespaceBeforeIndex:!accessing!private! !
!SmaCCString categoriesFor: #whitespaceEndIndexAfterIndex:!accessing!private! !
!SmaCCString categoriesFor: #whitespaceStartIndexBeforeIndex:!accessing!private! !

!SmaCCString class methodsFor!

new
	^(super new)
		initialize;
		yourself!

on: aString
	^(self new)
		insert: aString at: 0;
		yourself! !
!SmaCCString class categoriesFor: #new!instance creation!private! !
!SmaCCString class categoriesFor: #on:!instance creation!private! !

SmaCCStringInterval guid: (GUID fromString: '{EC29C583-6340-444C-9BBB-AD84DBC98E24}')!
SmaCCStringInterval comment: 'SmaCCStringInterval is a SmaCCInterval that represents a string that is insert into the SmaCCString.

Instance Variables:
	id	<Integer>	a unique id for the operation
	isRemoved	<Boolean>	is this interval removed?
	next	<SmaCCStringInterval>	the next interval in the string
	previous	<SmaCCStringInterval>	the previous interval in the string
	start	<Integer>	the starting location in the string
	stop	<Integer>	the ending location in the string
	string	<String>	the text that we are inserting -- we are only inserting from the start index to the stop index

'!
!SmaCCStringInterval categoriesForClass!Unclassified! !
!SmaCCStringInterval methodsFor!

addAllAnnotations: aCollection!

annotations
	^#()!

annotationsTo: lastInterval
	| annotations current |
	annotations := Set new.
	current := self.
	[annotations addAll: current annotations.
	current ~~ lastInterval]
		whileTrue:
			[current isSentinel ifTrue: [^nil].
			current := current next].
	^annotations asOrderedCollection!

asString
	(self isRemoved or: [self isEmpty]) ifTrue: [^''].
	^string copyFrom: start to: stop!

beSentinel
	id := -1.
	next := self.
	previous := self!

canBeMergedWith: nextSmaCCStringInterval
	^self id = nextSmaCCStringInterval id
		and: [self isRemoved = nextSmaCCStringInterval isRemoved and: [self stop + 1 = nextSmaCCStringInterval start]]!

containsIndex: anIndex
	| indexPosition indexId |
	self isSentinel ifTrue: [^true].
	anIndex isInteger
		ifTrue:
			[indexPosition := anIndex.
			indexId := 1]
		ifFalse:
			[indexPosition := anIndex index.
			indexId := anIndex intervalId].
	^id = indexId and: [(indexPosition between: start and: stop) or: [indexPosition = 0]]!

convertToRemoved
	| interval |
	(self isRemoved or: [self isSentinel]) ifTrue: [^self].
	isRemoved := true.
	(previous canBeMergedWith: self)
		ifTrue:
			[interval := previous.
			interval stop: self stop.
			interval next: next.
			self removed.
			^interval].
	(self canBeMergedWith: next)
		ifTrue:
			[interval := next.
			next start: self start.
			previous next: next.
			self removed.
			^interval].
	^self!

do: aBlock
	| current |
	current := self.
	[current isSentinel]
		whileFalse:
			[aBlock value: current.
			current := current next]!

id
	^id!

id: anInteger
	id := anInteger!

insertAfter: aSmaCCInterval
	self next: aSmaCCInterval next.
	aSmaCCInterval next: self!

isEmpty
	^start > stop!

isRemoved
	^isRemoved!

isRemoved: aBoolean
	isRemoved := aBoolean!

isSentinel
	^id = -1!

next
	^next!

next: aSmaCCInterval
	next := aSmaCCInterval.
	aSmaCCInterval previous: self!

nextPutOn: aWriteStream
	self isRemoved ifTrue: [^self].
	aWriteStream
		next: self size
		putAll: string
		startingAt: start!

previous
	^previous!

previous: aSmaCCInterval
	previous := aSmaCCInterval!

printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: id;
		nextPut: $,;
		print: start;
		nextPut: $-;
		print: stop;
		nextPut: $)!

publishedAspects
	| aspects aspectClass |
	aspectClass := Smalltalk at: #Aspect.
	aspects := LookupTable new.
	aspects add: (self newAspect: #yourself class: aspectClass).
	aspects add: (aspectClass multilineString: #asString) beReadOnly.
	self isRemoved ifTrue: [aspects add: (aspectClass multilineString: #removedString) beReadOnly].
	aspectClass := Smalltalk at: #InstVarAspect.
	1 to: self class instSize do: [:i | aspects add: ((self instVarAt: i) newAspect: i class: aspectClass)].
	^aspects!

removed
	next := nil.
	previous := nil!

removedString
	^string copyFrom: start to: stop!

size
	self isRemoved ifTrue: [^0].
	^stop - start + 1 max: 0!

splitAt: anIndex
	| new |
	(self isSentinel or: [anIndex = start]) ifTrue: [^self].
	new := self copy.
	self stop: anIndex - 1.
	new start: anIndex.
	new insertAfter: self.
	^new!

start
	^start!

start: anInteger
	start := anInteger!

startIndex
	self isSentinel ifTrue: [^nil].
	^self isEmpty ifTrue: [next startIndex] ifFalse: [SmaCCStringIndex index: start intervalId: id]!

stop
	^stop!

stop: anInteger
	stop := anInteger!

stopIndex
	^SmaCCStringIndex index: stop intervalId: id!

string
	^string!

string: aString
	isRemoved := false.
	string := aString! !
!SmaCCStringInterval categoriesFor: #addAllAnnotations:!private! !
!SmaCCStringInterval categoriesFor: #annotations!accessing!public! !
!SmaCCStringInterval categoriesFor: #annotationsTo:!accessing!public! !
!SmaCCStringInterval categoriesFor: #asString!accessing!private! !
!SmaCCStringInterval categoriesFor: #beSentinel!private! !
!SmaCCStringInterval categoriesFor: #canBeMergedWith:!public!testing! !
!SmaCCStringInterval categoriesFor: #containsIndex:!private!testing! !
!SmaCCStringInterval categoriesFor: #convertToRemoved!private! !
!SmaCCStringInterval categoriesFor: #do:!iterating!private! !
!SmaCCStringInterval categoriesFor: #id!accessing!private! !
!SmaCCStringInterval categoriesFor: #id:!accessing!private! !
!SmaCCStringInterval categoriesFor: #insertAfter:!editing!private! !
!SmaCCStringInterval categoriesFor: #isEmpty!private!testing! !
!SmaCCStringInterval categoriesFor: #isRemoved!private!testing! !
!SmaCCStringInterval categoriesFor: #isRemoved:!accessing!private! !
!SmaCCStringInterval categoriesFor: #isSentinel!private!testing! !
!SmaCCStringInterval categoriesFor: #next!accessing!private! !
!SmaCCStringInterval categoriesFor: #next:!accessing!private! !
!SmaCCStringInterval categoriesFor: #nextPutOn:!private! !
!SmaCCStringInterval categoriesFor: #previous!accessing!private! !
!SmaCCStringInterval categoriesFor: #previous:!accessing!private! !
!SmaCCStringInterval categoriesFor: #printOn:!printing!public! !
!SmaCCStringInterval categoriesFor: #publishedAspects!development!public! !
!SmaCCStringInterval categoriesFor: #removed!private! !
!SmaCCStringInterval categoriesFor: #removedString!accessing!private! !
!SmaCCStringInterval categoriesFor: #size!accessing!private! !
!SmaCCStringInterval categoriesFor: #splitAt:!editing!private! !
!SmaCCStringInterval categoriesFor: #start!accessing!private! !
!SmaCCStringInterval categoriesFor: #start:!accessing!private! !
!SmaCCStringInterval categoriesFor: #startIndex!accessing!private! !
!SmaCCStringInterval categoriesFor: #stop!accessing!private! !
!SmaCCStringInterval categoriesFor: #stop:!accessing!private! !
!SmaCCStringInterval categoriesFor: #stopIndex!accessing!private! !
!SmaCCStringInterval categoriesFor: #string!accessing!private! !
!SmaCCStringInterval categoriesFor: #string:!accessing!private! !

!SmaCCStringInterval class methodsFor!

from: aSmaCCIndex to: aSmaCCIndex2 string: aString
	^(self new)
		start: aSmaCCIndex;
		stop: aSmaCCIndex2;
		string: aString;
		yourself! !
!SmaCCStringInterval class categoriesFor: #from:to:string:!instance creation!private! !

SmaCCStringIndex guid: (GUID fromString: '{04943AF1-6D7B-4880-B1BC-16F9094CAB80}')!
SmaCCStringIndex comment: 'SmaCCStringIndex is used to index into a SmaCCString. When you perform an insert operation into a SmaCCString, you get a SmaCCStringIndex back so you can index into the newly inserted string.

Instance Variables:
	index	<Integer>	the index in the SmaCCInterval
	intervalId	<Integer>	the SmaCCInterval''s id

'!
!SmaCCStringIndex categoriesForClass!Unclassified! !
!SmaCCStringIndex methodsFor!

- anInteger
	^self + anInteger negated!

+ anInteger
	^self class index: index + anInteger intervalId: intervalId!

< aSmaCCStringIndex
	^self index < aSmaCCStringIndex index!

= anObject
	^self class = anObject class and: [self index = anObject index and: [self intervalId = anObject intervalId]]!

hash
	^index bitXor: intervalId!

index
	^index!

index: anInteger
	index := anInteger!

intervalId
	^intervalId!

intervalId: anInteger
	intervalId := anInteger!

printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: intervalId;
		nextPut: $,;
		print: index;
		nextPut: $)! !
!SmaCCStringIndex categoriesFor: #-!arithmetic!private! !
!SmaCCStringIndex categoriesFor: #+!arithmetic!private! !
!SmaCCStringIndex categoriesFor: #<!comparing!public! !
!SmaCCStringIndex categoriesFor: #=!comparing!private! !
!SmaCCStringIndex categoriesFor: #hash!comparing!private! !
!SmaCCStringIndex categoriesFor: #index!accessing!private! !
!SmaCCStringIndex categoriesFor: #index:!accessing!private! !
!SmaCCStringIndex categoriesFor: #intervalId!accessing!private! !
!SmaCCStringIndex categoriesFor: #intervalId:!accessing!private! !
!SmaCCStringIndex categoriesFor: #printOn:!printing!public! !

!SmaCCStringIndex class methodsFor!

index: anInteger intervalId: intervalId
	^(self new)
		index: anInteger;
		intervalId: intervalId;
		yourself! !
!SmaCCStringIndex class categoriesFor: #index:intervalId:!instance creation!private! !

"Binary Globals"!

