| package |
package := Package name: 'SmaCC Tests'.
package paxVersion: 1;
	basicComment: 'This package is part of the Smalltalk Compiler-Compiler (SmaCC) written by John Brant and Don Roberts. It contains some simple tests for SmaCC. If you wish to port SmaCC or if you want to verify you installation of SmaCC, then these may be useful to you. Under VisualWorks, these tests require loading the MethodWrappers package.

If you want more information about SmaCC, visit http://www.refactoryworkers.com/SmaCC/. 
-------------------------------------------------
Copyright (c) 2002-2015 John Brant, Don Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'.


package classNames
	add: #DuplicationNodeEliminationTest;
	add: #LineNumberStreamTest;
	add: #SmaCCBitCollectionTest;
	add: #SmaCCCharacterSetTest;
	add: #SmaCCEndToEndTest;
	add: #SmaCCIntegerSetTest;
	add: #SmaCCStringTest;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\SmaCC Development\SmaCC Development';
	add: '..\SmaCC GLR Runtime\SmaCC GLR Runtime';
	add: '..\SmaCC Runtime\SmaCC Runtime';
	add: '..\SmaCC Source Editing\SmaCC Source Editing';
	add: '..\..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #DuplicationNodeEliminationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #LineNumberStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #SmaCCBitCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #SmaCCEndToEndTest
	instanceVariableNames: 'currentClasses currentPackage manager'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #SmaCCStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCBitCollectionTest subclass: #SmaCCCharacterSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SmaCCBitCollectionTest subclass: #SmaCCIntegerSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

DuplicationNodeEliminationTest guid: (GUID fromString: '{8BA9D30E-2F8E-45EF-A358-02266FA41666}')!
DuplicationNodeEliminationTest comment: ''!
!DuplicationNodeEliminationTest categoriesForClass!Unclassified! !
!DuplicationNodeEliminationTest methodsFor!

testDuplicateLoop
	| node1 node2 node3 |
	node1 := SmaCCNode new.
	node1 action: 1.
	node2 := SmaCCNode new.
	node2 action: 1.
	node3 := SmaCCNode new.
	node3 action: 1.
	node1 addEdgeTo: node2 onCharacters: 'a'.
	node2 addEdgeTo: node3 onCharacters: 'a'.
	node3 addEdgeTo: node1 onCharacters: 'a'.
	node1 := node1 removeDuplicateNodes.
	self assert: node1 allStates size = 1.
	self assert: node1 transitions size = 1.
	self assert: (node1 action includes: 1).
	self assert: node1 transitions first transitionObjects asString = 'a'.
	self assert: node1 transitions first to = node1!

testNoDuplicates
	| node1 node2 node3 |
	node1 := SmaCCNode new.
	node1 action: 1.
	node2 := SmaCCNode new.
	node2 action: 2.
	node3 := SmaCCNode new.
	node3 action: 3.
	node1 addEdgeTo: node2 onCharacters: 'a'.
	node2 addEdgeTo: node3 onCharacters: 'a'.
	node3 addEdgeTo: node1 onCharacters: 'a'.
	node1 := node1 removeDuplicateNodes.
	self assert: node1 allStates size = 3.
	self assert: node1 transitions size = 1.
	self assert: (node1 action includes: 1).
	self assert: node1 transitions first transitionObjects asString = 'a'.
	node2 := node1 transitions first to.
	self deny: node1 = node2.
	self assert: node2 transitions size = 1.
	self assert: (node2 action includes: 2).
	self assert: node2 transitions first transitionObjects asString = 'a'.
	node3 := node2 transitions first to.
	self deny: node1 = node3.
	self deny: node2 = node3.
	self assert: node3 transitions size = 1.
	self assert: (node3 action includes: 3).
	self assert: node3 transitions first transitionObjects asString = 'a'.
	self assert: node3 transitions first to = node1!

testSimpleDuplicateWithEdges
	| node1 node2 node3 |
	node1 := SmaCCNode new.
	node2 := SmaCCNode new.
	node2 action: 2.
	node3 := SmaCCNode new.
	node3 action: 2.
	node1 addEdgeTo: node2 onCharacters: 'a'.
	node1 addEdgeTo: node3 onCharacters: 'b'.
	node2 addEdgeTo: node1 onCharacters: 'c'.
	node3 addEdgeTo: node1 onCharacters: 'c'.
	node1 := node1 removeDuplicateNodes.
	self assert: node1 allStates size = 2.
	self assert: node1 transitions size = 1.
	self assert: node1 transitions first transitionObjects asString = 'ab'.
	node2 := node1 transitions first to.
	self deny: node1 = node2.
	self assert: node2 transitions size = 1.
	self assert: (node2 action includes: 2).
	self assert: node2 transitions first transitionObjects asString = 'c'.
	self assert: node2 transitions first to = node1!

testSimpleTerminalDuplicate
	| node1 node2 node3 |
	node1 := SmaCCNode new.
	node2 := SmaCCNode new.
	node2 action: 2.
	node3 := SmaCCNode new.
	node3 action: 2.
	node1 addEdgeTo: node2 onCharacters: 'a'.
	node1 addEdgeTo: node3 onCharacters: 'b'.
	node1 := node1 removeDuplicateNodes.
	self assert: node1 allStates size = 2.
	self assert: node1 transitions size = 1.
	self assert: node1 transitions first transitionObjects asString = 'ab'.
	node2 := node1 transitions first to.
	self deny: node1 = node2.
	self assert: node2 transitions isEmpty.
	self assert: (node2 action includes: 2)! !
!DuplicationNodeEliminationTest categoriesFor: #testDuplicateLoop!public!tests! !
!DuplicationNodeEliminationTest categoriesFor: #testNoDuplicates!public!tests! !
!DuplicationNodeEliminationTest categoriesFor: #testSimpleDuplicateWithEdges!public!tests! !
!DuplicationNodeEliminationTest categoriesFor: #testSimpleTerminalDuplicate!public!tests! !

LineNumberStreamTest guid: (GUID fromString: '{5F08AB40-6608-44C7-8484-E43F07D41504}')!
LineNumberStreamTest comment: ''!
!LineNumberStreamTest categoriesForClass!Unclassified! !
!LineNumberStreamTest methodsFor!

addCRLFsTo: aString 
	^aString copyReplaceAll: '\'
		with: (String with: Character cr with: Character lf)!

addCRsTo: aString 
	^aString copyReplaceAll: '\' with: (String with: Character cr)!

addLFsTo: aString 
	^aString copyReplaceAll: '\' with: (String with: Character lf)!

testCRLFLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRLFsTo: '1234\1234\1234\1234')).
	1 to: 4
		do:
			[:each | 
			self assert: stream lineNumber = each.
			4 timesRepeat: [self assert: stream columnNumber = (stream next asInteger - $0 asInteger)].
			stream upTo: Character lf]!

testCRLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRsTo: '1234\1234\1234\1234')).
	1 to: 4
		do:
			[:each | 
			self assert: stream lineNumber = each.
			4 timesRepeat: [self assert: stream columnNumber = (stream next asInteger - $0 asInteger)].
			stream upTo: Character cr]!

testEmptyLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRsTo: '\\\')).
	1 to: 4
		do: 
			[:each | 
			self assert: stream lineNumber = each.
			stream upTo: Character cr]!

testForwardPositionCRLFLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRLFsTo: '1\2\3\4')).
	stream position: 3.
	self assert: stream lineNumber = 2.
	self assert: stream columnNumber = 1!

testLFLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addLFsTo: '1234\1234\1234\1234')).
	1 to: 4
		do: 
			[:each | 
			self assert: stream lineNumber = each.
			4 timesRepeat: [self assert: stream columnNumber = (stream next asInteger - $0 asInteger)].
			stream upTo: Character lf]!

testNoLines
	| stream index |
	stream := SmaCCLineNumberStream on: (ReadStream on: 'this is a test').
	self assert: stream lineNumber = 1.
	index := 1.
	stream do: 
			[:each | 
			self assert: stream lineNumber = 1.
			index := index + 1.
			self assert: stream columnNumber = index]!

testPositionCRLFLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRLFsTo: '1\2\3\4')).
	self assert: stream lineNumber = 1.
	stream upTo: $5.
	self assert: stream lineNumber = 4.
	stream position: 0.
	self assert: stream lineNumber = 1.
	self assert: stream columnNumber = 1.
	stream position: 3.
	self assert: stream lineNumber = 2.
	self assert: stream columnNumber = 1!

testPositionCRLines
	| stream |
	stream := SmaCCLineNumberStream 
				on: (ReadStream on: (self addCRsTo: '1\2\3\4')).
	self assert: stream lineNumber = 1.
	stream upTo: $5.
	self assert: stream lineNumber = 4.
	stream skip: -1.
	self assert: stream lineNumber = 4.
	self assert: stream columnNumber = 1.
	stream position: 0.
	self assert: stream lineNumber = 1.
	self assert: stream columnNumber = 1.
	stream position: 2.
	self assert: stream lineNumber = 2.
	self assert: stream columnNumber = 1! !
!LineNumberStreamTest categoriesFor: #addCRLFsTo:!private! !
!LineNumberStreamTest categoriesFor: #addCRsTo:!private! !
!LineNumberStreamTest categoriesFor: #addLFsTo:!private! !
!LineNumberStreamTest categoriesFor: #testCRLFLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testCRLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testEmptyLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testForwardPositionCRLFLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testLFLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testNoLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testPositionCRLFLines!private!tests! !
!LineNumberStreamTest categoriesFor: #testPositionCRLines!private!tests! !

SmaCCBitCollectionTest guid: (GUID fromString: '{212CAA4F-583B-466F-A18F-2521207CB7EA}')!
SmaCCBitCollectionTest comment: ''!
!SmaCCBitCollectionTest categoriesForClass!Unclassified! !
!SmaCCBitCollectionTest methodsFor!

byteSizeCollection
	^(0 to: 255) collect: self!

largeCollection
	^(1 to: 50000) collect: self!

largeValue
	^1000000!

mediumCollection
	^(1 to: 1000) collect: self!

newCollection
	^SmaCCIntegerSet new!

supportsLargeValues
	^true!

testABunch
	| collection count |
	collection := self newCollection.
	collection addAll: self byteSizeCollection.
	count := 0.
	collection do: [:each | count := count + 1].
	self assert: count = 256.
	self assert: collection size = 256.
	self assert: collection notEmpty.
	self assert: (self byteSizeCollection allSatisfy: [:each | collection includes: each]).
	self assert: collection asArray = self byteSizeCollection asArray!

testAdding
	| collection count |
	collection := self newCollection.
	collection add: (self value: 1).
	count := 0.
	collection do: [:each | count := count + 1].
	self assert: count = 1.
	self assert: collection size = 1.
	self assert: collection notEmpty.
	self assert: (collection includes: (self value: 1)).
	self deny: (collection includes: (self value: 0)).
	self deny: (collection includes: (self value: 2))!

testAddToNextByte
	| collection count |
	collection := self newCollection.
	collection add: (self value: 8).
	count := 0.
	collection do: [:each | count := count + 1].
	self assert: count = 1.
	self assert: collection size = 1.
	self assert: collection notEmpty.
	self assert: (collection includes: (self value: 8)).
	self deny: (collection includes: (self value: 7)).
	self deny: (collection includes: (self value: 9))!

testComparing
	| a b |
	a := self newCollection.
	a addAll: self mediumCollection.
	b := self newCollection.
	b addAll: self mediumCollection reverse.
	self assert: a hash = b hash.
	self assert: a = b.
	self supportsLargeValues ifFalse: [^self].
	b
		add: (self value: self largeValue);
		remove: (self value: self largeValue).
	self assert: a hash = b hash.
	self assert: a = b!

testCopying
	| a b |
	a := self newCollection.
	a addAll: self mediumCollection.
	b := a copy.
	b remove: (self value: 5).
	self assert: (a includes: (self value: 5)).
	self deny: (b includes: (self value: 5))!

testGrowing
	| collection count |
	collection := self newCollection.
	collection add: (self value: 256).
	count := 0.
	collection do: [:each | count := count + 1].
	self assert: count = 1.
	self assert: collection size = 1.
	self assert: collection notEmpty.
	self assert: (collection includes: (self value: 256)).
	self deny: (collection includes: (self value: 1)).
	collection remove: (self value: 256).
	self deny: (collection includes: (self value: 256)).
	self assert: collection size = 0!

testLessThan
	| a b |
	a := self newCollection.
	a addAll: self mediumCollection.
	b := self newCollection.
	b addAll: self mediumCollection.
	self deny: a < b.
	self deny: a > b.
	self assert: a <= b.
	self assert: a >= b.
	b remove: (self value: 1).
	self assert: a < b.
	self deny: a > b.
	self assert: a <= b.
	self deny: a >= b.
	b add: (self value: 1).
	b remove: (self value: 2).
	self assert: a < b.
	self deny: a > b.
	self assert: a <= b.
	self deny: a >= b.
	self supportsLargeValues ifFalse: [^false].
	b
		add: (self value: 2);
		remove: (self value: 1000).
	self deny: a < b.
	self assert: a > b.
	self deny: a <= b.
	self assert: a >= b!

testNew
	self assert: self newCollection size = 0.
	self assert: self newCollection isEmpty!

testPerformance
	| myTime setTime |
	myTime := Time
				millisecondsToRun: [self newCollection addAll: self largeCollection].
	setTime := Time millisecondsToRun: [Set new addAll: self largeCollection].
	setTime := setTime max: 1000.	"Some implementations of Set don't have performance problems"
	self assert: myTime < (setTime // 2)!

testRemoving
	| collection |
	collection := self newCollection.
	collection add: (self value: 1).
	self assert: collection size = 1.
	collection remove: (self value: 1).
	self assert: collection size = 0.
	self deny: (collection includes: (self value: 1))!

value: anInteger
	^anInteger! !
!SmaCCBitCollectionTest categoriesFor: #byteSizeCollection!private! !
!SmaCCBitCollectionTest categoriesFor: #largeCollection!private! !
!SmaCCBitCollectionTest categoriesFor: #largeValue!private! !
!SmaCCBitCollectionTest categoriesFor: #mediumCollection!private! !
!SmaCCBitCollectionTest categoriesFor: #newCollection!private! !
!SmaCCBitCollectionTest categoriesFor: #supportsLargeValues!private! !
!SmaCCBitCollectionTest categoriesFor: #testABunch!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testAdding!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testAddToNextByte!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testComparing!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testCopying!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testGrowing!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testLessThan!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testNew!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testPerformance!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #testRemoving!public!tests! !
!SmaCCBitCollectionTest categoriesFor: #value:!private! !

SmaCCEndToEndTest guid: (GUID fromString: '{A02E7552-7102-4D4C-B788-66012E49367C}')!
SmaCCEndToEndTest comment: ''!
!SmaCCEndToEndTest categoriesForClass!Unclassified! !
!SmaCCEndToEndTest methodsFor!

checkTokensAreInOrder: tokenCollection 
	tokenCollection inject: 0
		into: 
			[:sum :each | 
			self assert: sum < each startPosition.
			each startPosition]!

classesInPackage
	^self package classes!

expressionParserString
	^'
%left "+" "-" ;
%left "*" "/" ;
%right "^" ;

Expression: Expression ''first'' "+" Expression ''last'' {first + last}
	| Expression "-" Expression {''1'' - ''3''}
	| Expression "*" Expression {''1'' * ''3''}
	| Expression "/" Expression {''1'' / ''3''}
	| Expression "^" Expression {''1'' raisedTo: ''3''} 
	| "(" Expression ")" {''2''}
	| <number> {''1'' value asNumber};'!

expressionScannerString
	^'
<number>	:	[0-9]+ (\. [0-9]+) ? ;
<whitespace>	:	\s+;'!

lookupType: className 
	^self class environment at: className!

package
	^PackageManager current packageOfClass: self class!

parserClass
	^self lookupType: #TestParser!

removeClass: aBehavior 
	aBehavior removeFromSystem!

removeNewClasses
	| classesToRemove |
	classesToRemove := self classesInPackage reject: [:each | currentClasses includes: each].
	(classesToRemove asSortedCollection: [:a :b | a allSuperclasses size > b allSuperclasses size])
		do: [:each | self removeClass: each].
	PackageManager current defaultPackage: currentPackage!

scannerClass
	^self lookupType: #TestScanner!

setUp
	super setUp.
	SmaCCCodeGenerator reset.
	self setUpClasses!

setUpClasses
	currentClasses := self classesInPackage.
	currentPackage := PackageManager current defaultPackage.
	PackageManager current defaultPackage: self package!

supportsUnicodeStrings
	^
	[String with: (Character value: 16rFF00).
	true] on: Error
			do: [:ex | ex return: false]!

tearDown
	self removeNewClasses.
	super tearDown!

testAmbiguousReduceReduceConflict
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<int> : \d+; <whitespace>: \s+;
			%glr;
			Expression : Left "!!" "<" {''1''} | Right "!!" ">" {''1''} ;
			Left : <int> "+" <int> "*" <int> {''1'' value asNumber + ''3'' value asNumber * ''5'' value asNumber};
			Right : <int> "+" <int> "*" <int> {''1'' value asNumber + (''3'' value asNumber * ''5'' value asNumber)};
			'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: '2 + 3 * 4!!<') = 20.
	self assert: (self parserClass parse: '2 + 3 * 4!!>') = 14.
	self should: [self parserClass parse: '2 + 3 * 4!!!!'] raise: SmaCCParserError!

testAmbiguousResults
	| parserCompiler results result |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<int> : \d+; <whitespace>: \s+;
			%glr;
			Expression : Expression "+" Expression {''1'' + ''3''}
				| Expression "*" Expression {''1'' * ''3''}
				| <int> {''1'' value asNumber};
			'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self parserClass compile: 'alwaysPerformReduceAction ^true'.
	self assert: (self parserClass parse: '3 + 4') = 7.
	self assert: (self parserClass parse: '3 * 4') = 12.
	[result := self parserClass parse: '2 + 3 * 4']
		on: SmaCCAmbiguousResultNotification
		do: 
			[:ex |
			results := ex tag
						collect: [:each | each performOnSmaCCParser: ex parser].
			ex resume: 0].
	self assert: result = 0.
	self assert: results size = 2.
	self assert: (results includes: 20).
	self assert: (results includes: 14).

	"There are a bunch of ways to parse the following expression. If we don't merge the possibilities, then this will never complete."
	self
		assert: (self parserClass
				parse: '1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1')
					= 60!

testAmbiguousResultWithDifferentScannerTokens
	| parserCompiler results result |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%id <a> <aa>;
			<a> : a ; <aa> : aa;
			%glr;
			List : Item {''1''} | List Item {''1'' + ''2''} ;
			Item : <a> {1} | <aa> {1.5};
			'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self parserClass compile: 'alwaysPerformReduceAction ^true'.
	self parserClass
		compile: 'actionsForCurrentToken 
		| oldToken oldPosition |
		(currentToken ids includes: scanner aaId) ifTrue: [
			oldToken := currentToken.
			oldPosition := scanner position.
			self position: currentToken stopPosition - 1.
			currentToken := currentToken class value: ''a'' start: currentToken startPosition ids: (Array with: scanner aId).
			super actionsForCurrentToken do: [:each | self splitWithAction: each].
			currentToken := oldToken.
			self position: oldPosition.
		].
		^super actionsForCurrentToken'.
	self assert: (self parserClass parse: 'a') = 1.
	[result := self parserClass parse: 'aa'] on: SmaCCAmbiguousResultNotification
		do: 
			[:ex |
			results := ex tag
						collect: [:each | each performOnSmaCCParser: ex parser].
			ex resume: 0].
	self assert: result = 0.
	self assert: results size = 2.
	self assert: (results includes: 2).
	self assert: (results includes: 1.5)!

testAutoAnnotateTokens
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<name> : \w+; <whitespace> : \s+; 
			%annotate_tokens;
			%root FooNode;
			A : "(" <name> "<<" "test" ")" {{FooANode}} ; '.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '( foo << test )'.
	self assert: node nameToken value = 'foo'.
	self assert: node leftParenToken value = '('.
	self assert: node lessThanLessThanToken value = '<<'.
	self assert: node testToken value = 'test'.
	self assert: node rightParenToken value = ')'!

testBadCharacterBlock
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	self should: 
			[parserCompiler buildDefinition: '<foo> : [a-] ;
					Start : <foo>;'.
			parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser']
		raise: SmaCCParserError!

testBadNode
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%root FooNode;
			A : B "a" ''a'' {{FooNode}} ;
			B : A | C ;
			C : "c" ''c''; '.
	self
		assert:
			([compiler compileInto: 'TestScanner' andParser: 'TestParser'.
			false]
				on: SmaCCCompilationNotification
				do:
					[:ex | ('Node creation' = ex messageText and: ['*>>>A<<<*' match: ex tag]) ifTrue: [ex return: true] ifFalse: [ex resume: nil]])!

testBadSmalltalkBlock
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	self should: 
			[parserCompiler
				buildDefinition: '<foo> : foo ;
					Start : <foo> {self printOn: };'.
			parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser']
		raise: SmaCCParserError!

testChildrenVariables
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			%hierarchy If (IfElse);
			%left "+" "-" ;
			If : "if" number ''a'' "then" number ''b'' {{If}}
				| "if" number ''a'' "then" number ''b'' "else" number ''c'' {{IfElse}};
			number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'if 1 then 2 else 3'.
	self assert: node class name asString = 'FooIfElseNode'.
	self assert: node children size = 3!

testChildrenWhenNodeTypeNotSpecified
	| compiler node values |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '%root Program;
			%prefix Test;
			%suffix Node;
			<id> : [a-zA-Z]+;
			<num> : [0-9]+;
			<whitespace> : [\s]+;
			Start : Expr ";" ;
			Expr : Term ''left'' "+" ''op''  Expr ''right''   {{Binary}}
			     | Term ''left'' "-" ''op'' Expr ''right''        {{Binary}}
			     | Term ;
			Term : Factor ''left'' "*" ''op'' Term ''right'' {{Binary}}
			     | Factor ''left'' "/" ''op'' Term ''right''       {{Binary}}
			     | Factor ;
			Factor : <num> ''value''                  {{Constant}}
			        | <id> ''name'' "(" Expr ''argument'' ")"        {{Function}}
			        | "(" Expr ")" ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 + 2;'.
	self assert: node children size = 2.
	values := #('1' '2') asSet.
	node children do: [:each | values remove: each value value]!

testChildrenWhenNodeTypeNotSpecifiedAndCouldBeToken
	| compiler node values |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '%root Program;
			%prefix Test;
			%suffix Node;
			<id> : [a-zA-Z]+;
			<num> : [0-9]+;
			<whitespace> : [\s]+;
			Start : Expr ";" ;
			Expr : Term ''left'' "+" ''op''  Expr ''right''   {{Binary}}
			     | Term ''left'' "-" ''op'' Expr ''right''        {{Binary}}
			     | Term ;
			Term : Factor ''left'' "*" ''op'' Term ''right'' {{Binary}}
			     | Factor ''left'' "/" ''op'' Term ''right''       {{Binary}}
			     | Factor ;
			Factor : <num> ''value''                  {{Constant}}
			        | <id> ''name'' "(" Expr ''argument'' ")"        {{Function}}
				  | "(" Q ")" ;
			Q : Expr | <id> ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 + 2;'.
	self assert: node children size = 2.
	values := #('1' '2') asSet.
	node children do: [:each | values remove: each value value]!

testCollectionSuperclass
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; %hierarchy Expression (Number Binary); 
			%left "+" "-" ;
			Expression : Expression ''left'' "+" ''operator'' Expression ''right'' {{Binary}}
				| Expression ''left'' "-" ''operator'' Expression ''right'' {{Binary}}
				| <number> ''number'' {{Number}}
				| "(" ''leftParen'' Expression ")" ''rightParen'' {{Expression}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '(2 + 3 - (4))'.
	self assert: node class name asString = 'FooBinaryNode'.
	self assert: node operator value = '-'.
	self assert: node leftParens size = 1.
	self assert: node class superclass name asString = 'FooExpressionNode'.
	self assert: node right class name asString = 'FooNumberNode'.
	self assert: node right leftParens size = 1.
	self assert: node right parent == node.
	self assert: node left parent == node.
	self assert: node parent isNil.
	self assert: node children size = 2.
	self assert: (node children includes: node left).
	self assert: (node children includes: node right).
	self assert: node right number value = '4'.
	self assert: node left operator value = '+'.
	self assert: node left left number value = '2'.
	self assert: node left right number value = '3'.
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy left ~~ node left.
	self assert: copy right ~~ node right.
	self assert: copy operator ~~ node operator.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testCollectionSuperclassWithCodeNode
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; %hierarchy Expression (Number Binary); 
			%left "+" "-" ;
			Expression : Expression ''left'' "+" ''operator'' Expression ''right'' {{Binary}}
				| Expression ''left'' "-" ''operator'' Expression ''right'' {{Binary}}
				| Number {''1''}
				| "(" ''leftParen'' Expression ")" ''rightParen'' {{Expression}} ;
			Number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '(2 + 3 - (4))'.
	self assert: node class name asString = 'FooBinaryNode'.
	self assert: node operator value = '-'.
	self assert: node leftParens size = 1.
	self assert: node class superclass name asString = 'FooExpressionNode'.
	self assert: node right class name asString = 'FooNumberNode'.
	self assert: node right leftParens size = 1.
	self assert: node right parent == node.
	self assert: node left parent == node.
	self assert: node parent isNil.
	self assert: node children size = 2.
	self assert: (node children includes: node left).
	self assert: (node children includes: node right).
	self assert: node right number value = '4'.
	self assert: node left operator value = '+'.
	self assert: node left left number value = '2'.
	self assert: node left right number value = '3'.
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy left ~~ node left.
	self assert: copy right ~~ node right.
	self assert: copy operator ~~ node operator.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testCollectionSuperclassWithImplicitPullUp
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; %hierarchy Expression (Number Binary); 
			%left "+" "-" ;
			Expression : Expression ''left'' "+" ''operator'' Expression ''right'' {{Binary}}
				| Expression ''left'' "-" ''operator'' Expression ''right'' {{Binary}}
				| Number
				| "(" ''leftParen'' Expression ")" ''rightParen'' {{Expression}} ;
			Number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '(2 + 3 - (4))'.
	self assert: node class name asString = 'FooBinaryNode'.
	self assert: node operator value = '-'.
	self assert: node leftParens size = 1.
	self assert: node class superclass name asString = 'FooExpressionNode'.
	self assert: node right class name asString = 'FooNumberNode'.
	self assert: node right leftParens size = 1.
	self assert: node right parent == node.
	self assert: node left parent == node.
	self assert: node parent isNil.
	self assert: node children size = 2.
	self assert: (node children includes: node left).
	self assert: (node children includes: node right).
	self assert: node right number value = '4'.
	self assert: node left operator value = '+'.
	self assert: node left left number value = '2'.
	self assert: node left right number value = '3'.
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy left ~~ node left.
	self assert: copy right ~~ node right.
	self assert: copy operator ~~ node operator.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testComments
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<a> : /* q */ /***/ /****/ a # b 
				;
			/* */ /***/ /****/
			#this is a comment
			begin : <a> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'a')!

testCopyCollection
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node;
			%left "+" "-" ;
			ExpressionList : ExpressionList Expression ''expression'' {{ExpressionList}}  | Expression ''expression'' {{ExpressionList}};
			Expression : Number ;
			Number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 2 3 4'.
	self assert: (node copyInContext: Dictionary new) = node!

testCrLf
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: '<a> : \r? \n | \r ;
			A : <a> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self
		assert: (self parserClass parse: (String with: Character cr with: Character lf)).
	self
		should: [self parserClass parse: (String with: Character cr with: Character cr)]
		raise: SmaCCParserError!

testDefaultClassNameWithUnderscore
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%root Test ; %prefix Foo; %suffix Node; 
			test_class : "a" ''letter'' {{}};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a'.
	self assert: node class name asString = 'FooTestClassNode'.
	self assert: node class superclass name asString = 'FooTestNode'.
	(self lookupType: #FooTestNodeVisitor) new acceptNode: node!

testEmptyAlternative
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '%root FooNode;
			A : B ''component'' {{FooANode}} ;
			B :  | C ; 
			C : "c" ''c'' {{FooCNode}} ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'c'.
	self assert: node class name asString = 'FooANode'.
	self assert: node component c value = 'c'.
	self assert: node component parent == node.
	node := self parserClass parse: ''.
	self assert: node component isNil!

testErrors
	| parserCompiler position |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: self expressionScannerString , self expressionParserString.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self parserClass parse: '+3*4' onError: [:str :pos | position := pos].
	self assert: position = 1.
	self parserClass parseStream: (ReadStream on: '1+a+4')
		onError: [:str :pos | position := pos].
	self assert: position = 3!

testErrorToken
	| parserCompiler countingBlock |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<name> : [a-zA-Z]+; <whitespace>: \s+;
			%left "+"; Expression : <name> | Expression "+" Expression {[Notification signal] on: Error do: [:ex | ex return: (Notification raise)]} | error "+" Expression ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	countingBlock :=
			[:string |
			| count |
			count := 0.
			
			[[self parserClass parse: string] on: Notification
				do: 
					[:ex |
					count := count + 1.
					[ex resume] on: Error
						do: 
							[:ex1 |
							ex isResumable: true.
							ex1 retry]]]
					on: SmaCCParserError
					do: [:ex | ex return].
			count].
	#(#('a + b + c' 2) #('+ a + b + c' 2) #('a + b +' 1) #('a + 1 / 2 + b + c + 3' 2) #('1 + 1 / 2 + b + c + 3' 1))
		do: [:each | self assert: (countingBlock value: each first) = each last]!

testExcludeScannerState
	| parserCompiler parser |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%excludes a;
			<foo> : b;
			a <bar> : a;
			Start : <foo> {true} | <bar> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'b').
	self should: [self parserClass parse: 'a'] raise: SmaCCParserError.
	parser := self parserClass on: (ReadStream on: 'a').
	parser state: #a.
	self deny: parser parse.
	parser := self parserClass on: (ReadStream on: 'b').
	parser state: #a.
	self should: [parser parse] raise: SmaCCParserError!

testExistingVariables
	| parserCompiler |
	SmaCCScanner
		subclass: #TestScanner
		instanceVariableNames: 'scanner1 scanner2'
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	SmaCCParser
		subclass: #TestParser
		instanceVariableNames: 'parser1 parser2'
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: (self scannerClass instVarNames includes: 'scanner1').
	self assert: (self scannerClass instVarNames includes: 'scanner2').
	self assert: (self parserClass instVarNames includes: 'parser1').
	self assert: (self parserClass instVarNames includes: 'parser2')!

testExpressions
	#(#LR1 #LALR1) do: 
			[:each |
			| parserCompiler |
			parserCompiler := SmaCCGrammarCompiler new.
			parserCompiler grammar type: each.
			parserCompiler
				buildDefinition: self expressionScannerString , self expressionParserString.
			parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser'.
			self assert: (self parserClass parse: '2 + 3 * 4 / 5 ^ 3 ^ 2')
						= (2 + (3 * 4 / (5 raisedTo: (3 raisedTo: 2)))).
			self assert: (self parserClass parse: '1.0*2/3+4.0^2^3+3')
						= (1.0 * 2 / 3 + (4.0 raisedTo: (2 raisedTo: 3)) + 3).
			self assert: (self parserClass parse: '(((1 + 2) * 3) ^ 2) ^ 3')
						= (((1 + 2) * 3 raisedTo: 2) raisedTo: 3)]!

testGLRHierarchy
	| parserCompiler |
	SmaCCGLRParser
		subclass: #FooParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(Smalltalk at: #FooParser)
		subclass: #TestParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase; %glr;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self parserClass superclass name asString = 'FooParser'!

testHierarchy
	| parserCompiler |
	SmaCCScanner
		subclass: #FooScanner
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(Smalltalk at: #FooScanner)
		subclass: #TestScanner
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	SmaCCParser
		subclass: #FooParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(Smalltalk at: #FooParser)
		subclass: #TestParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self scannerClass superclass name asString = 'FooScanner'.
	self assert: self parserClass superclass name asString = 'FooParser'!

testIgnoreCaseKeywords
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%ignorecase;
			<identifier> : [a-z_]+ ; <whitespace> : \s+ ;
			Start ::= "Foo" {true} | <identifier> <identifier> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'FOo').
	self deny: (self parserClass parse: 'FoObAr ASDS')!

testIgnoreCaseTokens
	| parserCompiler result |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%ignorecase;
			<whitespace> : \s+ ;
			Start ::= "a" "a" {Array with: ''1'' with: ''2''};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	result := self parserClass parse: 'a A'.
	self assert: result first = result last.
	self assert: result first hash = result last hash!

testIgnoreCaseWithMultipleKeywordCases
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%ignorecase;
			<whitespace> : \s+ ;
			Start ::= "if" "If" "iF" "IF" {true}; '.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'IF IF IF IF').
	self assert: (self parserClass parse: 'iF If IF If')!

testImplicitParserTokens
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: 'A : <isLetter> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'a').
	self should: [self parserClass parse: ' '] raise: SmaCCParserError!

testImplicitScannerTokens
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: '<a> : <isLetter> + ;
			A : <a> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'asdfssdafADSFS').
	self should: [self parserClass parse: ' asdf asdf'] raise: SmaCCParserError!

testImplicitTokenPrecedence
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<a> : . ;
			A : <a> {true} | <isLetter> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	0 to: 255
		do: [:each | self assert: (self parserClass parse: (String with: (Character value: each)))]!

testInvertedCharactersBlock
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%ignorecase;
			<A> : a{1,3} ;
			<ID> : [^a]+ ;
			<comment> : \s+ ;
			begin : {0} | begin <A> {''1'' + 1} | begin <ID> {''1'' - 1};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'aa') = 1.
	self assert: (self parserClass parse: 'aaaa') = 2.
	self assert: (self parserClass parse: 'a bab') = 0!

testKeywords
	| parserCompiler size |
	size := SmaCCGrammar maximumCharacterValue.
	self supportsUnicodeStrings
		ifTrue: [SmaCCGrammar maximumCharacterValue: (2 raisedTo: 16) - 1].
	
	[parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: '%ignorecase;
			Start ::= "Foo" "bar";'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self parserClass parse: 'FoObAr']
			ensure: [SmaCCGrammar maximumCharacterValue: size]!

testLALRErrorHandler
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<whitespace> : \s+ ;
			Start : Foo "]" Foo ")";
			Foo : "a" Bar ;
			Bar : "b" | error ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self should: [self parserClass parse: 'a ] a b )'] raise: SmaCCParserError.
	self should: [self parserClass parse: 'a ) a b )'] raise: SmaCCParserError.
	self should: [self parserClass parse: 'a b ] a  ]'] raise: SmaCCParserError!

testLeftAndRightRecursive
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			Foo : Number ''number'' {{Collection}}
				| "(" ''leftParen'' Foo ")" ''rightParen'' {{Collection}} ;
			Number : <number> ''token'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '(((4)))'.
	self assert: node leftParens size = 3.
	self assert: node rightParens size = 3.
	self assert: node number token value = '4'.
	self checkTokensAreInOrder: node leftParens.
	self checkTokensAreInOrder: node rightParens.
	self assert: node children size = 1.
	self assert: node children first == node number.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testLeftRecursive
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			Foo : Number ''number'' {{Collection}}
				| Foo "," ''comma'' Number ''number'' {{Collection}} ;
			Number : <number> ''token'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 , 2 , 3 , 4'.
	self assert: node numbers size = 4.
	node numbers keysAndValuesDo: 
			[:key :each |
			self assert: key printString = each token value.
			self assert: each parent == node.
			self assert: (node children includes: each)].
	self assert: node commas size = 3.
	node commas do: [:each | self assert: each value = ','].
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy numbers ~~ node numbers.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testMatchCollection
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node;
			%left "+" "-" ;
			ExpressionList : ExpressionList Expression ''expression'' {{ExpressionList}}  | Expression ''expression'' {{ExpressionList}};
			Expression : Number ;
			Number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 2 3 4'.
	self assert: (node match: node inContext: Dictionary new)!

testMoveFromGLRHierarchy
	| parserCompiler |
	SmaCCGLRParser
		subclass: #FooParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(Smalltalk at: #FooParser)
		subclass: #TestParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self parserClass superclass name asString = 'SmaCCParser'!

testMoveToGLRHierarchy
	| parserCompiler |
	SmaCCParser
		subclass: #FooParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(Smalltalk at: #FooParser)
		subclass: #TestParser
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase; %glr;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self parserClass superclass name asString = 'SmaCCGLRParser'!

testMultipleComponents
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '%root FooNode;
			A : B C {{FooANode}} ;
			B : "b" ''b'' ;
			C : "c" ''c'';'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'bc'.
	self assert: node class name asString = 'FooANode'.
	self assert: node b startPosition = 1.
	self assert: node c startPosition = 2!

testMultipleStartingPositions
	| parserCompiler position |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<name> : \w+;
			%start plus ;
			multiply: plus "*" plus {true};
			plus: <name> {false} | <name> "+" <name> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'a*a').
	self assert: (self parserClass parse: 'a+a*a').
	self should: [self parserClass parse: 'a+a'] raise: SmaCCParserError.
	self
		deny: (self parserClass parse: 'a+a' startingAt: self parserClass startingStateForplus).
	self parserClass
		parse: 'a*a'
		startingAt: self parserClass startingStateForplus
		onError: [:str :pos | position := pos].
	self assert: position = 2!

testMultipleTypes
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B | C ;
			B : "b" ''b'' D {{FooBNode}} ;
			C : "c" ''c'' D {{FooCNode}};
			D : Number ''number'' | D "," ''comma'' Number ''number'' ;
			Number : <number> ''number'' {{FooNumberNode}};'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'b 1, 2, 3, 4'.
	self assert: node class name asString = 'FooBNode'.
	node numbers keysAndValuesDo: 
			[:key :each |
			self assert: key printString = each number value.
			self assert: each parent == node].
	self assert: node numbers size = 4.
	self assert: node commas size = 3!

testName
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<name> : \w+; <whitespace> : \s+; 
			%root FooNode;
			A : name ''nameNode'' {{FooANode}} ;
			name : simple | composite ;
			simple : <name> ''name'' {{FooNameNode}};
			composite : name ''base'' "." ''period'' <name> ''name'' {{FooFullNameNode}}; '.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a.b.c'.
	self assert: node nameNode name value = 'c'.
	self assert: node nameNode parent == node.
	self assert: node nameNode base name value = 'b'.
	self assert: node nameNode base parent == node nameNode.
	self assert: node nameNode base base name value = 'a'.
	self assert: node nameNode base base parent == node nameNode base!

testNamedSubComponent
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : {{FooANode}} | "a" ''a'' A ''sub'' {{FooANode}} ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'aaa'.
	self assert: node class name asString = 'FooANode'.
	self assert: node a startPosition = 1.
	self assert: node sub a startPosition = 2.
	self assert: node sub parent == node.
	self assert: node sub sub a startPosition = 3.
	self assert: node sub sub parent == node sub.
	self assert: node sub sub sub a isNil.
	self assert: node sub sub sub parent == node sub sub.
	self assert: node sub sub sub sub isNil!

testNodeAndExpression
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '%root FooNode;
			A : B "a" ''a'' {{FooNode}} ;
			B : {Object new} ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a'.
	self assert: node a value = 'a'!

testNodeClassName
	| compiler pattern |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <name> : [a-z]+; <whitespace> : \s+; <patternToken> : (\`  [^\`]* \`)+ ; %glr;
			%root FooNode;
			A : Name | Number ;
			Name : <name>''name'' {{FooNameNode}};
			Number : <number> ''number'' {{FooNumberNode}};'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	pattern := self parserClass parse: '`a{nodeClassName: #FooNumberNode}`'.
	self assert: (pattern match: (self parserClass parse: '123')
				inContext: Dictionary new).
	self
		deny: (pattern match: (self parserClass parse: 'abc') inContext: Dictionary new)!

testNodeWithoutRoot
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: 'test : "a" ''letter'' {{FooNode}};'.
	self should: 
			[parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser']
		raise: SmaCCCompilationNotification!

testNonAssociativeTokens
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<IDENTIFIER> : \w+ ; <whitespace> : \s+ ;
			%left "and" ;
			%nonassoc "=" "<=";
			%left "+" ;
			expr : expr "=" expr {true} | expr "<=" expr {true} | expr "and" expr {true} | expr "+" expr {true} | <IDENTIFIER> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self should: [self parserClass parse: 'a=b=c'] raise: SmaCCParserError.
	self should: [self parserClass parse: 'a=b<=c'] raise: SmaCCParserError.
	self should: [self parserClass parse: 'a=b+c=d'] raise: SmaCCParserError.
	self assert: (self parserClass parse: 'a=b and c=d')!

testNonRecursiveVariableInsideRecursiveProduction
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<whitespace> : \s+; 
			%root FooNode;
			A : B "a" ''letter'' {{FooNode}} ;
			B : A | C ;
			C : "c" ''c'' {{FooNode}};'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'c a a a'.
	self assert: node c value = 'c'.
	self assert: node letters size = 3!

testOptionalCollection
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B_opt {{FooANode}} ;
			B_opt : B | ;
			B : Number ''numberNode'' | B Number ''numberNode'' ;
			Number : <number> ''number'' {{FooNumberNode}};'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 2 3 4'.
	node numberNodes keysAndValuesDo: 
			[:key :each |
			self assert: key printString = each number value.
			self assert: each parent == node].
	self assert: node numberNodes size = 4!

testOptionalComponent
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B {{FooANode}} ;
			B : "b" ''b'' | ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: ''.
	self assert: node b isNil.
	node := self parserClass parse: 'b'.
	self assert: node b startPosition = 1!

testOptionalVariableType
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%root FooNode;
			A : Comma {{FooANode}} ;
			Comma : "," ''comma'' |  ;'.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self parserClass basicNew symbolTypes asSet asSortedCollection asArray = #(#FooANode #SmaCCToken)!

testOverlappingTokens
	| parserCompiler result |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<id1> : \w+; <id2> : \w+;
			%glr;
			Test : "keyword" {''1''} | <id1> {''1''} | <id2> {''1''} ;
			'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	result := self parserClass parse: 'keyword'.
	self assert: result ids size = 3!

testParseAllPattern
	| compiler trees node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; <patternToken> : (\`  [^\`]* \`)+ ; 
			%root FooNode; %glr;
			A : "f:" ''f'' Z ''z'' {{FooANode}} | A "f:" ''f'' Z ''z'' {{FooANode}};
			Z : B | C;
			B : "b" ''b'' {{FooBNode}};
			C : "c" ''c'' {{FooCNode}}; '.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.

	trees := self parserClass parseAll: 'f: `a` f: `b`' startingAt: self parserClass defaultStartingState.
	self assert: trees size = 1.
	node := trees first.
	self assert: node fs size = 2.
	
	trees := self parserClass parseAll: 'f: `a{beToken}` f: `b{beToken}`' startingAt: self parserClass defaultStartingState.
	self assert: trees size = 4.
	node := trees first.
	self assert: node fs size = 2.
	!

testPassThrough
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B ''b'' {{FooANode}} ;
			B : C ;
			C : Number;
			Number : <number> ''number'' {{FooNumberNode}}; '.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '12'.
	self assert: node class name asString = 'FooANode'.
	self assert: node b parent == node.
	self assert: node b number startPosition = 1.
	self assert: node b number value = '12'!

testPerformanceMatchCollection
	| parserCompiler node match fastTime slowTime |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d; <whitespace> : \s+; 
			<patternToken>: (\`  [^\`]* \`)+ ;
			%glr;
			%root Parse ; %prefix Foo; %suffix Node;
			%left "+" "-" ;
			ExpressionList : ExpressionList Expression ''expression'' {{ExpressionList}}  | Expression ''expression'' {{ExpressionList}};
			Expression : Number ;
			Number : <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: (String new: 1000 withAll: $1) , '22'
						, (String new: 1000 withAll: $1).
	fastTime := Time
				millisecondsToRun: [self assert: (node match: node inContext: Dictionary new)].
	match := self parserClass parse: '`a{beList}` 22 `b{beList}`'.
	slowTime := Time millisecondsToRun: 
					[self assert: (match match: node inContext: Dictionary new)].
	self assert: (fastTime max: 1) * 10 > slowTime!

testPrefixSuffix
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%root Test ; %prefix Foo; %suffix Node; 
			test : "a" ''letter'' {{Test}};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a'.
	self assert: node class name asString = 'FooTestNode'.
	self assert: node class superclass = SmaCCParseNode.
	(self lookupType: #FooTestNodeVisitor) new acceptNode: node!

testPrefixSuffixWithDefaultClassName
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%root Test ; %prefix Foo; %suffix Node; 
			test : "a" ''letter'' {{}};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a'.
	self assert: node class name asString = 'FooTestNode'.
	self assert: node class superclass = SmaCCParseNode.
	(self lookupType: #FooTestNodeVisitor) new acceptNode: node!

testPriority
	| parserCompiler |
	SmaCCScanner
		subclass: #TestScanner
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		classInstanceVariableNames: ''.
	(self lookupType: #TestScanner) compile: 'all ^self recordAndReportMatch: (Array with: self _allId)'.
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition:
			'%ignorecase;
			<a> : \{ | \< \% ;
			<all> : .;
			%id <_all> <a> <all>;  
			begin : A {true}
					| All {false};
			A : <a> ;
			All: <_all>;'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: (self parserClass parse: '{').
	self assert: (self parserClass parse: '<%').
	self deny: (self parserClass parse: '}')!

testPullUpCollectionVariables
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : AList {{FooNode}} ;
			AList : B | AList B;
			B : <number> ''number'' ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 2 3 4'.
	node numbers
		keysAndValuesDo: [:key :each | self assert: key printString = each value].
	self assert: node numbers size = 4!

testPullUpMultipleTypes
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : D B {{FooBNode}} | D C {{FooCNode}};
			B : "b" ''b'' D ;
			C : "c" ''c'' D ;
			D : Number ''number'' | D Number ''number'' ;
			Number : <number> ''number'' {{FooNumberNode}};'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '0 b 1 2 3 4'.
	self assert: node class name asString = 'FooBNode'.
	node numbers keysAndValuesDo: 
			[:key :each |
			self assert: (key - 1) printString = each number value.
			self assert: each parent == node].
	self assert: node numbers size = 5!

testPullUpVariablesForLaterSymbols
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : "a" ''a'' B_opt {{FooNode}} ;
			B_opt : B | ;
			B : <number> ''number'' | B <number> ''number'' ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a 1 2 3 4'.
	node numbers
		keysAndValuesDo: [:key :each | self assert: key printString = each value].
	self assert: node numbers size = 4.
	node := self parserClass parse: 'a'.
	self assert: node numbers isEmpty!

testRecursiveRule
	#(#LR1 #LALR1) do: 
			[:each |
			| parserCompiler |
			parserCompiler := SmaCCGrammarCompiler new.
			parserCompiler grammar type: each.
			parserCompiler buildDefinition: self expressionScannerString
						, 'Test: Test1 Recursive;
							Test1: <number> {''1'' value};
							Recursive: | Recursive "|" <number> {''1'' add: ''3'' value; yourself};'.
			parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser'.
			self
				assert: ((parserCompiler grammar nonTerminalSymbolNamed: 'Recursive')
						firstTerminals
							includes: (parserCompiler grammar keywordTerminalNamed: '"|"')).
			self assert: (self parserClass parse: '2|3|4') asOrderedCollection
						= (OrderedCollection with: '2' with: '3' with: '4')]!

testReduceActionReuse
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition:
			'<a> : a ;
			begin : <a> {#()};'.
	parserCompiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: (self parserClass selectors anySatisfy: [:each | each beginsWith: 'reduceActionFor'])!

testReduceReduceConflict
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<letter> : \w ;
			<whitespace>: \s+	;
			begin : Foo <letter> {true} | Bar <letter> {false};
			Foo :  ;
			Bar : ;'.
	self
		assert: (
			[parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser'.
			false]
					on: SmaCCCompilationNotification
					do: 
						[:ex |
						('*Reduce/Reduce*' match: ex messageText)
							ifTrue: [ex return: true]
							ifFalse: [ex resume: nil]])!

testReduceRules
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<char> : [a-zA-Z] ; 
			<ws> : \s ;
			Start ::= Chars {#liftFirstValue:};
			Chars : WS A WS {#liftSecondValue:};
			A : WS WS C {#liftLastValue:} | C WS {#liftLastValue:};
			WS : <ws> {#nil};
			C : <char> {#stringValue:};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: ' a  ') isNil.
	self assert: (self parserClass parse: '   a ') = 'a'!

testRenamedNullableVariable
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%glr;
			%root FooNode;
			A : B ''separator'' C {{FooANode}} ;
			B : C | ;
			C : Comma ''c'';
			Comma : "," ''comma'' {{FooCommaNode}};'.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: ',,'.
	self assert: node separator comma value = ','.
	self assert: node separator comma startPosition = 1.
	self assert: (node nodeVariables includes: #separator).
	self assert: node c comma value = ','.
	self assert: node c comma startPosition = 2.
	self deny: (self parserClass basicNew symbolTypes includes: #Object).
	node := self parserClass parse: ','.
	self assert: node separator isNil!

testRenamedVariable
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%root FooNode;
			A : Comma ''separator'' Comma {{FooANode}} ;
			Comma : "," ''comma'' ;'.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: ',,'.
	self assert: node separator value = ','.
	self assert: node separator startPosition = 1.
	self assert: (node tokenVariables includes: #separator).
	self assert: node comma value = ','.
	self assert: node comma startPosition = 2.
	self deny: (self parserClass basicNew symbolTypes includes: #Object)!

testRenamedVariableCollection
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%root FooNode;
			A : Commas ''separator'' "a" ''a'' Commas {{FooANode}} ;
			Commas : | Commas "," ''comma'' ;'.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: ',,,a,,,'.
	self assert: node separators size = 3.
	self assert: node separators first value = ','.
	self assert: node separators first startPosition = 1.
	self assert: node compositeTokenVariables asSortedCollection asArray = #(#commas #separators).
	self deny: (self parserClass basicNew symbolTypes includes: #Object)!

testRepeat
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<letters> : [a-zA-Z];
			<test> : (\`[^\`]*\`)+;
			All : | All Test ;
			Test : <letters> | <test> ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: '`a`b`c`') size = 3!

testRepeatingCharactersBlock
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<A> : a{1,3} ;
			<ID> : \w+ ;
			<whitespace> : \s+ ;
			begin : {true} | begin <A> {''1''} | begin <ID> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'aa').
	self deny: (self parserClass parse: 'aaaa')!

testReuseCompositeToken
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<LETTERDIGIT> : [a-zA-Z_] \d ;
			<AAorBB> :  aa | bb;
			<IDENTIFIER>: <LETTERDIGIT> | <AAorBB>	;
			<whitespace>: \s+	;
			begin : <IDENTIFIER> begin {false} | <IDENTIFIER> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self should: [self parserClass parse: '2'] raise: SmaCCParserError.
	self assert: (self parserClass parse: 'aa').
	self assert: (self parserClass parse: 'bb').
	self assert: (self parserClass parse: 'a4').
	self should: [self parserClass parse: 'ab'] raise: SmaCCParserError.
	self should: [self parserClass parse: '4b'] raise: SmaCCParserError!

testReuseSimpleToken
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<LETTER> : [a-zA-Z_\\] ;
			<DIGIT> : [0-9] ;
			<IDENTIFIER>: <LETTER> (<LETTER>|<DIGIT>)*	;
			<whitespace>: \s+	;
			begin : <IDENTIFIER> begin {false} | <IDENTIFIER> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self should: [self parserClass parse: '2'] raise: SmaCCParserError.
	self assert: (self parserClass parse: '\')!

testRightRecursive
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			Foo : Number ''number'' {{Collection}}
				| Number ''number''  "," ''comma'' Foo {{Collection}} ;
			Number : <number> ''token'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 , 2 , 3 , 4'.
	self assert: node numbers size = 4.
	node numbers keysAndValuesDo: 
			[:key :each |
			self assert: key printString = each token value.
			self assert: each parent == node.
			self assert: (node children includes: each)].
	self assert: node commas size = 3.
	node commas inject: 0
		into: 
			[:sum :each |
			self assert: sum < each startPosition.
			each startPosition].
	node commas do: [:each | self assert: each value = ','].
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy numbers ~~ node numbers.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testScannerState
	| parserCompiler parser |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%states a ;
			<foo> : b;
			a <bar> : a;
			Start : <foo> {true} | <bar> {false};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass parse: 'b').
	self should: [self parserClass parse: 'a'] raise: SmaCCParserError.
	parser := self parserClass on: (ReadStream on: 'a').
	parser state: #a.
	self deny: parser parse.
	parser := self parserClass on: (ReadStream on: 'b').
	parser state: #a.
	self assert: parser parse!

testSimple
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler buildDefinition: '<a> : a ;
			begin : <a> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self should: [self parserClass parse: '2'] raise: SmaCCParserError.
	self assert: (self parserClass parse: 'a')!

testSimpleHierarchy
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			%hierarchy Binary (Plus Minus);
			%left "+" "-" ;
			Expression : Expression ''left'' "+" ''operator'' Expression ''right'' {{Plus}}
				| Expression ''left'' "-" ''operator'' Expression ''right'' {{Minus}}
				| <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '2 + 3 - 4'.
	self assert: node class name asString = 'FooMinusNode'.
	self assert: node operator value = '-'.
	self assert: node class superclass name asString = 'FooBinaryNode'.
	self assert: node right class name asString = 'FooNumberNode'.
	self assert: node left class name asString = 'FooPlusNode'.
	self assert: node right parent == node.
	self assert: node left parent == node.
	self assert: node parent isNil.
	self assert: node children size = 2.
	self assert: (node children includes: node left).
	self assert: (node children includes: node right).
	self assert: node right number value = '4'.
	self assert: node left operator value = '+'.
	self assert: node left left number value = '2'.
	self assert: node left right number value = '3'.
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy left ~~ node left.
	self assert: copy right ~~ node right.
	self assert: copy operator ~~ node operator.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testSimpleNode
	| parserCompiler node |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%root FooNode ;
			test : "a" ''letter'' {{FooNode}};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: 'a'.
	self assert: node class name asString = 'FooNode'.
	self assert: node letter value = 'a'.
	self assert: node letter startPosition = 1.
	self assert: node letter stopPosition = 1.
	self assert: node class superclass = SmaCCParseNode.
	self assert: node copy ~~ node.
	self assert: node copy letter ~~ node letter.
	self assert: node = node copy.
	self assert: node hash = node copy hash.
	(self lookupType: #FooNodeVisitor) new acceptNode: node!

testSimpleTree
	| parserCompiler node copy |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root Parse ; %prefix Foo; %suffix Node; 
			%left "+" "-" ;
			Expression : Expression ''left'' "+" ''operator'' Expression ''right'' {{Binary}}
				| Expression ''left'' "-" ''operator'' Expression ''right'' {{Binary}}
				| <number> ''number'' {{Number}} ;'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '2 + 3 - 4'.
	self assert: node class name asString = 'FooBinaryNode'.
	self assert: node operator value = '-'.
	self assert: node class superclass name asString = 'FooParseNode'.
	self assert: node right class name asString = 'FooNumberNode'.
	self assert: node right parent == node.
	self assert: node left parent == node.
	self assert: node parent isNil.
	self assert: node children size = 2.
	self assert: (node children includes: node left).
	self assert: (node children includes: node right).
	self assert: node right number value = '4'.
	self assert: node left operator value = '+'.
	self assert: node left left number value = '2'.
	self assert: node left right number value = '3'.
	copy := node copy.
	self assert: copy ~~ node.
	self assert: copy left ~~ node left.
	self assert: copy right ~~ node right.
	self assert: copy operator ~~ node operator.
	self assert: copy = node.
	self assert: copy hash = node hash.
	(self lookupType: #FooParseNodeVisitor) new acceptNode: node!

testSingleVariableCollection
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; <patternToken> : (\`  [^\`]* \`)+ ; <integer> : \d+;
			%root P; %glr; %prefix Foo; %suffix Node;
			A : B {{}};
			B : B C ''item'' | ; 
			C : "#(" D ")" {{}};
			D :  | D <integer> ''item''; '.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: '#(1)#(2 3 4)'.
	self assert: node items size = 2.
	self assert: node items first items size = 1.
	self assert: node items last items size = 3!

testSpecialCharacters
	| parserCompiler maxValue |
	[String with: (Character value: 16rFF00)] on: Error do: [:ex | ^self].	"Unicode characters not supported"
	maxValue := SmaCCGrammar maximumCharacterValue.
	SmaCCGrammar maximumCharacterValue: 65535.
	
	[parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '%ignorecase;
			<name> : \x3g \x20 \xFF \xFF1 \xFF00 \cC;
			begin ::= <name> {true};'.
	parserCompiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	self assert: (self parserClass
				parse: (String withAll: (#(3 103 16r20 16rFF 16rFF1 16rFF00 2)
								collect: [:each | Character value: each])))]
			ensure: [SmaCCGrammar maximumCharacterValue: maxValue]!

testTokenCollection
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; <patternToken> : (\`  [^\`]* \`)+ ; 
			%root P; %glr; %prefix Foo; %suffix Node;
			A : C B ''b'' C {{}};
			B : C {{}};
			C : "," ''comma''; '.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: ',,,'.
	self assert: node b comma startPosition = 2.
	self assert: node commas first startPosition = 1.
	self assert: node commas last startPosition = 3.
	self assert: node commas size =2!

testUnnamedCollectionSymbol
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B {{FooANode}} ;
			B : "b" ''b'' {{FooBNode}} | B "b" ''b'' {{FooBNode}} ;'.
	self
		assert:
			([compiler compileInto: 'TestScanner' andParser: 'TestParser'.
			false]
				on: SmaCCCompilationNotification
				do:
					[:ex | ('Node creation' = ex messageText and: ['*>>>B<<<*' match: ex tag]) ifTrue: [ex return: true] ifFalse: [ex resume: nil]])!

testUnnamedNodeVariable
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'%glr;
			<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B D {{FooANode}};
			B : <number> ''a'' <number> ''b'' {{FooBNode}} | ;
			D : <number> ''c''; '.
	self
		assert:
			([compiler compileInto: 'TestScanner' andParser: 'TestParser'.
			false]
				on: SmaCCCompilationNotification
				do:
					[:ex | ('Node creation' = ex messageText and: ['*>>>B<<<*' match: ex tag]) ifTrue: [ex return: true] ifFalse: [ex resume: nil]])!

testUnnamedSymbol
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : B {{FooANode}} ;
			B : "b" ''b'' {{FooBNode}} ;'.
	self
		assert:
			([compiler compileInto: 'TestScanner' andParser: 'TestParser'.
			false]
				on: SmaCCCompilationNotification
				do:
					[:ex | ('Node creation' = ex messageText and: ['*>>>B<<<*' match: ex tag]) ifTrue: [ex return: true] ifFalse: [ex resume: nil]])!

testUnusedSymbols
	| parserCompiler |
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<letters> : \w+ ;
			<whitespace>: \s+	;
			%start Bar;
			begin : Foo;
			Foo :  <letters>;
			Bar : Baz ;
			Baz : "test" ;'.
	self
		assert: (
			[parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser'.
			true]
					on: SmaCCCompilationNotification
					do: [:ex | ex return: false]).
	parserCompiler := SmaCCGrammarCompiler new.
	parserCompiler
		buildDefinition: '<letters> : \w+ ;
			<whitespace>: \s+	;
			begin : Foo;
			Foo :  <letters>;
			Bar : Baz ;
			Baz : "test" ;'.
	self
		assert: (
			[parserCompiler compileInto: 'TestScanner'
				andParser: 'TestParser'.
			false]
					on: SmaCCCompilationNotification
					do: [:ex | ex return: true])!

testUseWithoutClassDefinition
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition: '<number> : \d+; <whitespace> : \s+; 
			%root FooNode;
			A : <number> ''left'' B ''operator'' <number> ''right'' {{FooNode}} ;
			B : C ;
			C : "+" | "-" | "*" | "/" ;'.
	compiler compileInto: 'TestScanner'
		andParser: 'TestParser'.
	node := self parserClass parse: '1 + 3'.
	self assert: node operator value = '+'!

testVariableCollection
	| compiler node |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; <patternToken> : (\`  [^\`]* \`)+ ; 
			%root P; %glr; %prefix Foo; %suffix Node;
			A : B {{}};
			B : C | "d" ''d'';
			C : "," ''comma''; '.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	node := self parserClass parse: ','.
	self assert: node comma startPosition = 1.
	self assert: node d isNil.
	node := self parserClass parse: 'd'.
	self assert: node comma isNil.
	self assert: node d startPosition = 1!

testVariableCollectionType
	| compiler |
	compiler := SmaCCGrammarCompiler new.
	compiler
		buildDefinition:
			'<whitespace> : \s+; 
			%root FooNode;
			A : Commas {{FooANode}} ;
			Commas : "," ''comma'' | Commas "," ''comma'' ;'.
	compiler compileInto: 'TestScanner' andParser: 'TestParser'.
	self assert: self parserClass basicNew symbolTypes asSet asSortedCollection asArray = #(#FooANode #OrderedCollection #SmaCCToken)! !
!SmaCCEndToEndTest categoriesFor: #checkTokensAreInOrder:!private! !
!SmaCCEndToEndTest categoriesFor: #classesInPackage!private! !
!SmaCCEndToEndTest categoriesFor: #expressionParserString!private!tests! !
!SmaCCEndToEndTest categoriesFor: #expressionScannerString!private!tests! !
!SmaCCEndToEndTest categoriesFor: #lookupType:!private! !
!SmaCCEndToEndTest categoriesFor: #package!private! !
!SmaCCEndToEndTest categoriesFor: #parserClass!private! !
!SmaCCEndToEndTest categoriesFor: #removeClass:!private! !
!SmaCCEndToEndTest categoriesFor: #removeNewClasses!private! !
!SmaCCEndToEndTest categoriesFor: #scannerClass!private! !
!SmaCCEndToEndTest categoriesFor: #setUp!private! !
!SmaCCEndToEndTest categoriesFor: #setUpClasses!private! !
!SmaCCEndToEndTest categoriesFor: #supportsUnicodeStrings!private! !
!SmaCCEndToEndTest categoriesFor: #tearDown!private! !
!SmaCCEndToEndTest categoriesFor: #testAmbiguousReduceReduceConflict!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testAmbiguousResults!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testAmbiguousResultWithDifferentScannerTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testAutoAnnotateTokens!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testBadCharacterBlock!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testBadNode!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testBadSmalltalkBlock!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testChildrenVariables!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testChildrenWhenNodeTypeNotSpecified!public!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testChildrenWhenNodeTypeNotSpecifiedAndCouldBeToken!public!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testCollectionSuperclass!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testCollectionSuperclassWithCodeNode!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testCollectionSuperclassWithImplicitPullUp!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testComments!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testCopyCollection!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testCrLf!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testDefaultClassNameWithUnderscore!public!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testEmptyAlternative!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testErrors!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testErrorToken!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testExcludeScannerState!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testExistingVariables!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testExpressions!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testGLRHierarchy!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testHierarchy!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testIgnoreCaseKeywords!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testIgnoreCaseTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testIgnoreCaseWithMultipleKeywordCases!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testImplicitParserTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testImplicitScannerTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testImplicitTokenPrecedence!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testInvertedCharactersBlock!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testKeywords!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testLALRErrorHandler!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testLeftAndRightRecursive!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testLeftRecursive!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testMatchCollection!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testMoveFromGLRHierarchy!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testMoveToGLRHierarchy!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testMultipleComponents!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testMultipleStartingPositions!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testMultipleTypes!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testName!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testNamedSubComponent!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testNodeAndExpression!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testNodeClassName!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testNodeWithoutRoot!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testNonAssociativeTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testNonRecursiveVariableInsideRecursiveProduction!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testOptionalCollection!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testOptionalComponent!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testOptionalVariableType!public! !
!SmaCCEndToEndTest categoriesFor: #testOverlappingTokens!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testParseAllPattern!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPassThrough!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPerformanceMatchCollection!public!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPrefixSuffix!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPrefixSuffixWithDefaultClassName!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPriority!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testPullUpCollectionVariables!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPullUpMultipleTypes!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testPullUpVariablesForLaterSymbols!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testRecursiveRule!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testReduceActionReuse!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testReduceReduceConflict!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testReduceRules!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testRenamedNullableVariable!public! !
!SmaCCEndToEndTest categoriesFor: #testRenamedVariable!public! !
!SmaCCEndToEndTest categoriesFor: #testRenamedVariableCollection!public! !
!SmaCCEndToEndTest categoriesFor: #testRepeat!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testRepeatingCharactersBlock!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testReuseCompositeToken!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testReuseSimpleToken!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testRightRecursive!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testScannerState!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testSimple!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testSimpleHierarchy!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testSimpleNode!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testSimpleTree!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testSingleVariableCollection!public! !
!SmaCCEndToEndTest categoriesFor: #testSpecialCharacters!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testTokenCollection!public! !
!SmaCCEndToEndTest categoriesFor: #testUnnamedCollectionSymbol!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testUnnamedNodeVariable!public! !
!SmaCCEndToEndTest categoriesFor: #testUnnamedSymbol!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testUnusedSymbols!public!tests! !
!SmaCCEndToEndTest categoriesFor: #testUseWithoutClassDefinition!private!tests-parse tree creation! !
!SmaCCEndToEndTest categoriesFor: #testVariableCollection!public! !
!SmaCCEndToEndTest categoriesFor: #testVariableCollectionType!public! !

SmaCCStringTest guid: (GUID fromString: '{CF9C3E6A-2B92-43A1-910B-089BFE38E664}')!
SmaCCStringTest comment: ''!
!SmaCCStringTest categoriesForClass!SUnit! !
!SmaCCStringTest methodsFor!

testCombiningDeletedIntervals
	| string intervalCount current |
	string := SmaCCString on: '12345'.
	#(3 2 4) do: [:each | string deleteCharacter: each].
	self assert: string asString = '15'.
	intervalCount := 0.
	current := (string instVarNamed: 'firstSentinel') next.
	[current isSentinel]
		whileFalse:
			[intervalCount := intervalCount + 1.
			current := current next].
	self assert: intervalCount = 3!

testCopyFromTo
	| string |
	string := SmaCCString on: '1245'.
	string insert: '3' at: 3.
	self assert: (string copyFrom: 2 to: 3) asString = '234'!

testDeleteAcrossInsert
	| string |
	string := SmaCCString on: '1233345'.
	string insert: '234' at: 5.
	string deleteFrom: 4 to: 5.
	self assert: string asString = '12345'!

testDeleteAcrossInserts
	| string startIndex stopIndex |
	string := SmaCCString on: '16'.
	startIndex := string insert: '233' at: 2.
	stopIndex := string insert: '445' at: 2.
	string insert: 'asdf' at: stopIndex.
	string deleteFrom: startIndex + 2 to: stopIndex.
	self assert: string asString = '123456'!

testDeleteAll
	| string |
	string := SmaCCString on: 'asdf'.
	string deleteFrom: 1 to: 4.
	self assert: string asString = ''!

testDeleteFirst
	| string |
	string := SmaCCString on: 'asdf'.
	string deleteFrom: 1 to: 1.
	self assert: string asString = 'sdf'!

testDeleteLast
	| string |
	string := SmaCCString on: 'asdf'.
	string deleteFrom: 4 to: 4.
	self assert: string asString = 'asd'!

testDeleteUpTo
	| string |
	string := SmaCCString on: '123456'.
	#(1 3) do: [:each | string deleteCharacter: each].
	string deleteUpTo: 5.
	self assert: string asString = '56'!

testDeleteUpToEnd
	| string |
	string := SmaCCString on: '123456'.
	#(1 5) do: [:each | string deleteCharacter: each].
	string deleteUpToEnd: 3.
	self assert: string asString = '2'!

testDeleteWhitespaceBeforeAfter
	| string |
	string := SmaCCString on: '0 asdf 9'.
	string deleteWhitespaceBeforeIndex: 4.
	self assert: string asString = '0 asdf 9'.
	string deleteWhitespaceAfterIndex: 4.
	self assert: string asString = '0 asdf 9'.
	string deleteWhitespaceBeforeIndex: 3.
	self assert: string asString = '0asdf 9'.
	string deleteWhitespaceAfterIndex: 6.
	self assert: string asString = '0asdf9'!

testDoubleInsertInMiddle
	| string index |
	string := SmaCCString on: '15'.
	index := string insert: '24' at: 2.
	string insert: '3' at: index + 1.
	self assert: string asString = '12345'!

testInsertAtBeginning
	| string |
	string := SmaCCString on: '12345'.
	string insert: '0' at: 1.
	self assert: string asString = '012345'!

testInsertAtEnd
	| string |
	string := SmaCCString on: '12345'.
	string insert: '6' at: 6.
	self assert: string asString = '123456'!

testInsertInMiddle
	| string |
	string := SmaCCString on: '1245'.
	string insert: '3' at: 3.
	self assert: string asString = '12345'!

testInsertOnEmptyString
	| string |
	string := SmaCCString on: ''.
	string insert: '123456' at: 1.
	self assert: string asString = '123456'!

testInsertWhitespace
	| string |
	string := SmaCCString
		on:
			'1' , (String with: Character cr with: Character lf) , '2' , (String with: Character cr) , '3'
				, (String with: Character lf) , '4' , (String with: Character lf) , (String with: Character cr) , '5'
				, (String with: Character lf).
	string
		addNewLineWhitespace: ' '
		from: 1
		to: string size.
	self
		assert:
			string asString
				=
					('1' , (String with: Character cr with: Character lf) , ' 2' , (String with: Character cr) , ' 3'
						, (String with: Character lf) , ' 4' , (String with: Character lf) , ' ' , (String with: Character cr) , ' 5'
						, (String with: Character lf) , ' ')!

testMoveAfter
	| string |
	string := SmaCCString on: '123456'.
	string
		moveFrom: 1
		to: 2
		after: 3.
	self assert: string asString = '312456'.
	string deleteFrom: 1 to: 4.
	self assert: string asString = '356'.
	string := SmaCCString on: '123456'.
	string
		moveFrom: 1
		to: 3
		after: 6.
	self assert: string asString = '456123'!

testMoveBefore
	| string |
	string := SmaCCString on: '123456'.
	string
		moveFrom: 4
		to: 6
		before: 3.
	self assert: string asString = '124563'.
	string deleteFrom: 5 to: 5.
	self assert: string asString = '12463'.
	string := SmaCCString on: '123456'.
	string
		moveFrom: 4
		to: 6
		before: 1.
	self assert: string asString = '456123'!

testMultipleDeleteAll
	| string |
	string := SmaCCString on: 'asdf'.
	#(1 3 4 2) do: [:each | string deleteCharacter: each].
	self assert: string asString = ''!

testMultipleDeleteWhitespaceAfter
	| string |
	string := SmaCCString on: '0 asdf 9'.
	6
		to: 3
		by: -1
		do:
			[:i | 
			string
				deleteWhitespaceAfterIndex: i;
				deleteCharacter: i].
	self assert: '0 9' = string asString!

testMultipleDeleteWhitespaceBefore
	| string |
	string := SmaCCString on: '0 asdf 9'.
	3
		to: 6
		do:
			[:i | 
			string
				deleteWhitespaceBeforeIndex: i;
				deleteCharacter: i].
	self assert: '0 9' = string asString!

testMultipleInserts
	| string |
	string := SmaCCString on: '125'.
	string insert: '3' at: 3.
	string insert: '4' at: 3.
	self assert: string asString = '12345'!

testMultipleInsertsAndDeletes
	| string indices last |
	string := SmaCCString on: ''.
	last := 0.
	indices := (1 to: 5) collect: [:each | last := string insert: each printString , 'X' at: last + 1].
	string deleteFrom: last + 1 to: indices first + 1.
	self assert: string asString = '12345'!

testNoEdits
	| string |
	string := SmaCCString on: 'asdf'.
	self assert: string asString = 'asdf'!

testReplaceInserted
	| string index |
	string := SmaCCString on: '1245'.
	index := string insert: 'x' at: 3.
	string
		replaceFrom: index
		to: index
		with: '3'.
	self assert: string asString = '12345'! !
!SmaCCStringTest categoriesFor: #testCombiningDeletedIntervals!public!tests! !
!SmaCCStringTest categoriesFor: #testCopyFromTo!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteAcrossInsert!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteAcrossInserts!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteAll!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteFirst!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteLast!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteUpTo!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteUpToEnd!public!tests! !
!SmaCCStringTest categoriesFor: #testDeleteWhitespaceBeforeAfter!public!tests! !
!SmaCCStringTest categoriesFor: #testDoubleInsertInMiddle!public!tests! !
!SmaCCStringTest categoriesFor: #testInsertAtBeginning!public!tests! !
!SmaCCStringTest categoriesFor: #testInsertAtEnd!public!tests! !
!SmaCCStringTest categoriesFor: #testInsertInMiddle!public!tests! !
!SmaCCStringTest categoriesFor: #testInsertOnEmptyString!public!tests! !
!SmaCCStringTest categoriesFor: #testInsertWhitespace!public!tests! !
!SmaCCStringTest categoriesFor: #testMoveAfter!public!tests! !
!SmaCCStringTest categoriesFor: #testMoveBefore!public!tests! !
!SmaCCStringTest categoriesFor: #testMultipleDeleteAll!public!tests! !
!SmaCCStringTest categoriesFor: #testMultipleDeleteWhitespaceAfter!public!tests! !
!SmaCCStringTest categoriesFor: #testMultipleDeleteWhitespaceBefore!public!tests! !
!SmaCCStringTest categoriesFor: #testMultipleInserts!public!tests! !
!SmaCCStringTest categoriesFor: #testMultipleInsertsAndDeletes!public!tests! !
!SmaCCStringTest categoriesFor: #testNoEdits!public!tests! !
!SmaCCStringTest categoriesFor: #testReplaceInserted!public!tests! !

SmaCCCharacterSetTest guid: (GUID fromString: '{911A45EF-BF20-4331-AEDB-FB650DB4B2FF}')!
SmaCCCharacterSetTest comment: ''!
!SmaCCCharacterSetTest categoriesForClass!Unclassified! !
!SmaCCCharacterSetTest methodsFor!

newCollection
	^SmaCCCharacterSet new!

supportsLargeValues
	[Character value: 1000] on: Error do: [:ex | ^false].
	^true!

value: anInteger
	^Character value: (self supportsLargeValues
				ifTrue: [anInteger]
				ifFalse: [anInteger bitAnd: 16rFF])! !
!SmaCCCharacterSetTest categoriesFor: #newCollection!private! !
!SmaCCCharacterSetTest categoriesFor: #supportsLargeValues!private! !
!SmaCCCharacterSetTest categoriesFor: #value:!private! !

SmaCCIntegerSetTest guid: (GUID fromString: '{50966C85-0888-4B5B-AB75-C6AC10E16B11}')!
SmaCCIntegerSetTest comment: ''!
!SmaCCIntegerSetTest categoriesForClass!Unclassified! !
!SmaCCIntegerSetTest methodsFor!

testNegative
	| collection |
	collection := self newCollection.
	collection add: 1.
	collection add: -1.
	collection add: -100000.
	self assert: collection size = 3.
	self assert: (collection includes: 1).
	self assert: (collection includes: -1).
	self assert: (collection includes: -100000).
	self deny: (collection includes: 0)! !
!SmaCCIntegerSetTest categoriesFor: #testNegative!public!tests! !

"Binary Globals"!

